/*******************************************************************************

FILE		:   UI_LM983x.h
Revision	:   1.4	
Date		:   10/18/99	

DESCRIPTION	:   Any TWAIN user interface module must conform to the interface
                presented in this file. The UI in essence must allow 
                activation / deactivation , work off of an external message
                pump and be prepared to pump messages via a callback during
                lengthy operations. The UI must also maintain state information
                to implement a simple state machine where it is in one of the 
                two states : IDLE or SCANNING.
                The actual interfaces to invoke these operations are described 
                in detail below.

Copyright (c) 1999 - National Semiconductor Ltd - All rights reserved
*******************************************************************************/

#ifndef _UI_LM983x_H__
#define _UI_LM983x_H__

/*******************************************************************************
Required headers 
*******************************************************************************/
#include "Twain_LM983x.h"

/*******************************************************************************
Defines & Macros
*******************************************************************************/
#if _BUILDING_UI_MODULE_

#define UI_LM983x_EXPORT _declspec(dllexport)

#else

#define UI_LM983x_EXPORT _declspec(dllimport)

#endif

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************************************
UI Module error codes 
*******************************************************************************/
// Result codes returned from the user interface module
typedef long					UI_RESULT;

#define UI_OK					TWAIN_OK
#define UI_ERROR				TWAIN_ERROR
#define UI_DONE					TWAIN_DONE
#define UI_ABORT				TWAIN_ABORT
#define UI_UNKNOWN_MSG			TWAIN_UNKNOWN_MSG


/*******************************************************************************
UI Module Misc
*******************************************************************************/


/*******************************************************************************
Exported Functions 
*******************************************************************************/
// This method will launch the UI window or close the UI windows after 
// providing appropriate feedback to the user
UI_LM983x_EXPORT TWAIN_RESULT UIActivate(BOOL open);
 
// This method will process messages from an external message loop and return 
// TWAIN_OK if the message was meant for a window created by this UI and 
// TWAIN_UNKNOWN_MSG Otherwise
UI_LM983x_EXPORT TWAIN_RESULT UIHandleMessage(MSG *pMsg);

// This method is provided to allow the UI to refresh its display and
// generally process its message queue during a lengthy operation
// This method should retreive and dispatch messages from the 
// message queue using the appropriate calls.
// The method should return TWAIN_ERROR if the current operation has to 
// be aborted otherwise the method should return TWAIN_OK
UI_LM983x_EXPORT TWAIN_RESULT UICallback(BOOL check_messages);

// This method informs the UI a pending scan request has completed
// The UI must now tranistion to the "Idle" state and provide
// appropriate feedback to the user 
UI_LM983x_EXPORT void	UIEndScan( TWAIN_RESULT tResult );

#ifdef __cplusplus
}
#endif

#endif //_UI_LM983x_H__
