/*****************************************************************************
M+
    Copyright (c) 1998,1999 by National Semiconductor
        All Rights Reserved.

M- 
*****************************************************************************/
// File: scan_DriverUSB.cpp 
// 

#include "stdafx.h"
#include "scan_driverusb.h"

#include "usbscan.h"				 // STI USB Interface
#include "sti.h"                     // Still Image services

/*****************************************************************************

  Global still image information

*****************************************************************************/
PSTI					g_pSti = NULL;        // handle to STI subsystem
PVOID                   g_pStiInfo = NULL;    // STI device info buffer
PSTI_DEVICE_INFORMATION g_pStiInfoPtr = NULL; // pointer to device in pStiBuffer
int                     g_nStiNumber = 0;     // 0 based index into g_pStiInfo
DWORD                   g_dwStiTotal = 0;     // total number of STI devices found
PSTIDEVICE				g_pStiDevice = NULL;  // STI device being used

/*****************************************************************************

  Module specific Consts and # Defines 

*****************************************************************************/

// The first byte for the bulk register reads is the mode 
// byte and this is bit mapped 
// Bit 0 , 1 - Read, 0 - Write
// Bit 1 , 1 - Inc, 0 - No Inc
const BYTE MODE_INC_READ	= 0x03;
const BYTE MODE_NOINC_READ	= 0x01;
const BYTE MODE_INC_WRITE	= 0x02;
const BYTE MODE_NOINC_WRITE = 0x00;

#define COMMAND_BYTE_COUNT	4
#define LOCK_TIMEOUT		2000

/*****************************************************************************

 Private STI api helper method prototypes

*****************************************************************************/
HRESULT StiImageRelease();
HRESULT StiDeviceRelease();
HRESULT StiWriteErrLog(DWORD,LPCWSTR);

/*****************************************************************************

 CDriverUSB Implementation

*****************************************************************************/
//
// Construct of class CDriverUSB
//
CDriverUSB::CDriverUSB()
{
}

//
// Function CDriverUSB destructor
//
CDriverUSB::~CDriverUSB()
{
//	CloseDriver();
}


//
// Function to initialize device driver
//
BOOL CDriverUSB::InitDriver(LPWSTR Merlin_Manufacturer,LPWSTR Merlin_Device)
{
    HRESULT hres = STI_OK;
	
	// Close the driver if it has been already opened
	CloseDriver();

	//
	// The StiCreateInstance interface locates the primary still image interface.
	// Use this call to optain the pointer to the IStillImage interface.
	//
	hres = StiCreateInstance(
		GetModuleHandle(NULL),  // instance handle of this application
		STI_VERSION,            // STI version
		&g_pSti,                  // pointer to IStillImage interface
		NULL                    // pointer to controlling unknown of OLE aggregation
		);
	
	if (FAILED(hres))
		return false;

	// Get the device list and try to locate our device
    DWORD   dwCounter;

    // Enumerate devices
	g_dwStiTotal = 0;
	g_pStiInfo = NULL;

	//
	// The GetDeviceList interface is used to get a list of the installed still
	// image devices. Use this call to obtain a STI_DEVICE_INFORMATION array
	// filled with info on all currently installed STI devices. 
	// * NOTE: the STI subsystem allocates memory for the STI device information
	// buffer, but the caller needs to free this memory with LocalFree().
	//
    hres = g_pSti->GetDeviceList(
			NULL,           // Type (reserved, use NULL)
			NULL,           // Flags (reserved, use NULL)
			&g_dwStiTotal,    // address of variable to return number of devices found
			&g_pStiInfo       // STI device info buffer
			);

    if (! SUCCEEDED(hres))
		return false;

    //
    // Compare the info on each device found with the info for our
	// device
    //
 	for (dwCounter = 0,g_pStiInfoPtr = (PSTI_DEVICE_INFORMATION) g_pStiInfo;
        dwCounter < g_dwStiTotal;
        dwCounter++, g_pStiInfoPtr++) 
	{
		if (!wcsicmp(g_pStiInfoPtr->pszVendorDescription, Merlin_Manufacturer) &&
			!wcsicmp(g_pStiInfoPtr->pszDeviceDescription, Merlin_Device) )
		{
			//
			// Open the device - it is now pointed to by g_pStiInfoPtr
			// The CreateDevice interface creates an IStiDevice object.
			// The IStiDevice object provides access to the IStiDevice interface
			// and device specific Imaging functionality.
			//
			hres = g_pSti->CreateDevice(
				g_pStiInfoPtr->szDeviceInternalName,  // internal device name
				STI_DEVICE_CREATE_DATA,	  // device creation mode
				&g_pStiDevice,            // pointer where IStiDevice object is to be stored
				NULL );                 // pointer to controlling unknown of OLE aggregation
			if (SUCCEEDED(hres)) return true;
		}
    }

    return false;
}
						
// Close Driver
BOOL CDriverUSB::CloseDriver()
{
	HRESULT hres = STI_OK;

	hres = StiDeviceRelease();
	
//	if (FAILED(hres))
//		;// VOTODO - Log it ??

	hres = StiImageRelease();

	return SUCCEEDED(hres);
}

#if 0
//Reg in
//NOTE: This is not functional with standard LM9831 ROM - Use BulkIn
BOOL	CDriverUSB::RegIn(UCHAR Addr, UCHAR *pValue)
{
	if (!g_pStiDevice)
		return FALSE;

	HRESULT hres = STI_OK;
	
	IO_BLOCK	stIoBlock;

	stIoBlock.uOffset = Addr;
	stIoBlock.uLength = 1;
	stIoBlock.pbyData = pValue;
	stIoBlock.uIndex  = 0;

	DWORD cbBytes = sizeof(UCHAR);

	hres = g_pStiDevice->LockDevice(LOCK_TIMEOUT);

	if (SUCCEEDED(hres)) 
	{
		hres = 	g_pStiDevice->RawReadCommand(&stIoBlock, &cbBytes, NULL);
	
		g_pStiDevice->UnLockDevice();
	
	}

	return SUCCEEDED(hres);
}

//Reg out
//NOTE: This is currently not functional - Use BulkOut
BOOL	CDriverUSB::RegOut(UCHAR Addr,UCHAR Value)
{
	if (!g_pStiDevice)
		return FALSE;

	HRESULT hres = STI_OK;
	
	UCHAR		localValue(Value);
	IO_BLOCK	stIoBlock;

	stIoBlock.uOffset = Addr;
	stIoBlock.uLength = 1;
	stIoBlock.pbyData = &localValue;
	stIoBlock.uIndex  = 0;

	DWORD cbBytes = sizeof(UCHAR);

	hres = g_pStiDevice->LockDevice(LOCK_TIMEOUT);
	
	if (SUCCEEDED(hres)) 
	{
		hres = 	g_pStiDevice->RawWriteCommand(&stIoBlock, cbBytes, NULL);

		g_pStiDevice->UnLockDevice();
	}

	return SUCCEEDED(hres);
}
#endif

//Bulk in
BOOL	CDriverUSB::BulkIn(UCHAR Addr,UCHAR *pBuffer,UINT Len, bool bIncrement)
{
	if (!g_pStiDevice)
		return FALSE;

	HRESULT hres;
	DWORD cbBytes;
for (int retry = 0; retry < 1; retry++)
{		
	hres = STI_OK;

	// To read registers via the bulk in endpoint send the 
	// command byte stream on the Bulk out endpoint and then
	// send in the read request on the bulk in endpoint

	hres = g_pStiDevice->LockDevice(LOCK_TIMEOUT);
	
	//must handle >= 64K size 
	const UINT Max_Bulk_In_Size=0xffff; 
	BYTE pCommandBytes[COMMAND_BYTE_COUNT];

	for (UINT i = 0; i < Len; )
	{
		cbBytes = min(Max_Bulk_In_Size,Len-i);

		if (SUCCEEDED(hres)) 
		{
			pCommandBytes[0] = bIncrement ? MODE_INC_READ : MODE_NOINC_READ;
			pCommandBytes[1] = Addr+(bIncrement ? i : 0);
			pCommandBytes[2] = (BYTE) (cbBytes >> 8); 
			pCommandBytes[3] = (BYTE) (cbBytes & 0xff);
			hres = 	g_pStiDevice->RawWriteData(&pCommandBytes, 
												COMMAND_BYTE_COUNT, 
												NULL);
			if (SUCCEEDED(hres) )
			{
				hres = 	g_pStiDevice->RawReadData(&pBuffer[i], &cbBytes, NULL);
			}
		}

		i+=cbBytes;
		if (!cbBytes) break; //in case less data
	}
	g_pStiDevice->UnLockDevice();

	if (SUCCEEDED(hres))
	{
		break;
	}
}

	if (!cbBytes) 
		return FALSE;

	return SUCCEEDED(hres);
}

//Bulk out
BOOL	CDriverUSB::BulkOut(UCHAR Addr,UCHAR *pBuffer,UINT Len, bool bIncrement)
{
	if (!g_pStiDevice)
		return FALSE;

	HRESULT hres;
for (int retry = 0; retry < 1; retry++)
{	
	hres = STI_OK;

	// To read registers via the bulk in endpoint send the 
	// command byte stream on the Bulk out endpoint and then
	// send in the write request on the bulk out endpoint

	hres = g_pStiDevice->LockDevice(LOCK_TIMEOUT);

	const UINT Max_Ct_Size=0xffff;
	BYTE pCommandBytes[Max_Ct_Size];
	DWORD cbBytes;
	const UINT Max_Bulk_Out_Size=60;
	//must handle >= 0xffff size 
	for (UINT i = 0; i < Len; )
	{
		UINT j;
		for (j = 0; j < (Max_Ct_Size-(COMMAND_BYTE_COUNT+Max_Bulk_Out_Size)) &&
				(i < Len); )
		{
			cbBytes = min(Max_Bulk_Out_Size,Len-i);
			pCommandBytes[j++] = bIncrement ? MODE_INC_WRITE : MODE_NOINC_WRITE;
			pCommandBytes[j++] = Addr+(bIncrement ? i : 0);
			pCommandBytes[j++] = (BYTE) (cbBytes >> 8); 
			pCommandBytes[j++] = (BYTE) (cbBytes & 0xff);
			for (UINT k = 0; k < cbBytes; k++)
				pCommandBytes[j++] = pBuffer[i++];
		}
		if (SUCCEEDED(hres)) 
		{
			hres = 	g_pStiDevice->RawWriteData( &pCommandBytes, 
												j, 
												NULL);
		}
	}

	g_pStiDevice->UnLockDevice();

	if (SUCCEEDED(hres))
	{
		break;
	}
}
	return SUCCEEDED(hres);
}

#if 0
BOOL CDriverUSB::Reset()
{
	if (!g_pStiDevice)
		return FALSE;

	HRESULT hres = g_pStiDevice->LockDevice(LOCK_TIMEOUT);
	
	if (SUCCEEDED(hres)) 
	{

		hres = g_pStiDevice->DeviceReset();

		g_pStiDevice->UnLockDevice();
	}

	return SUCCEEDED(hres);
}
#endif

/*****************************************************************************
    HRESULT StiDeviceRelease( )
        Close the Sti subsystem

    Parameters:
        none

    Return:
        HRESULT of last Sti call

*****************************************************************************/
HRESULT StiDeviceRelease( )
{
	HRESULT hres = STI_OK;

	//
	// The STI_DEVICE_INFORMATION array returned by GetDeviceList needs to
	// be freed with LocalFree(). Also, resetting internal Sti device counter.
	//
	if (g_pStiInfo)
		LocalFree(g_pStiInfo);
	g_pStiInfo = g_pStiInfoPtr = NULL;
	g_dwStiTotal = 0;

	//
    // close device if any are open
	//
    if (g_pStiDevice) {
		//
		// Close an open device.
		//
        hres = g_pStiDevice->Release();

		//
        // clear the Sti device pointer
		//
        g_pStiDevice = NULL;

	}

	return (hres);
}


/*****************************************************************************
    HRESULT StiImageRelease( )
        Close the Sti subsystem

    Parameters:
        none

    Return:
        HRESULT of last Sti call

*****************************************************************************/
HRESULT StiImageRelease( )
{
    HRESULT hres = STI_OK;

	//
    // if Sti subsystem is open, close it
	//
    if (g_pSti) {
		//
		// Close the Still Imaging subsystem.
		//
        hres = g_pSti->Release();
		//
		// clear the Sti subsystem pointer
		//
		g_pSti = NULL;

    }

	return (hres);
}

/*****************************************************************************
    HRESULT StiWriteErrLog(DWORD dwSeverity,LPCWSTR pszMessage)
		Write a string to the error log        

    Parameters:
		DWORD severity, which can be
			STI_TRACE_INFORMATION
			STI_TRACE_WARNING
			STI_TRACE_ERROR
		Wide character message to write to log.

    Return:
        HRESULT of last Sti call

*****************************************************************************/
HRESULT StiWriteErrLog(DWORD dwSeverity,LPCWSTR pszMessage)
{
    HRESULT hres = STI_OK;

    //
    // check that STI subsystem is loaded
    //
    if (! g_pSti)
        return (-1);

    //
    // WriteToErrorLog can be used to write debugging and diagnostic 
    // information into the STI log file, located in the Windows directory
    // STI_TRACE.LOG. The user can control whether informational, warning or
    // error messages, or any combination of these three are put in the log
    // file through the Scanners & Cameras control panel.
    //
    hres = g_pSti->WriteToErrorLog(
		dwSeverity,			// severity of error
		pszMessage			// string to write to log
		);

   return (hres);
}


