// FILE		: Scan_hardware.H

// DESCRIPTION	: header file and class definitions for the CHardware class
//		  this is the hardware interface

// REVISIONS	: BEB first version 2.0

 // i/o routine return values
#define	SUCCESS 0
#define	FAILURE 1

// calibration pause limit
//#define CALIBRATION_PAUSE_LIMIT 120

#define MCLKDIV_SCALING 2	//scaling for clock divider

#define INIFILE_VERSION "11"

#include "scan_lm983x.h"

/////////////////////////////////////////////////////////////////////////////
// CHardware class
//---------------------------------------------------------------
// CLASS	:	CHardware

// DESCRIPTION	:	hardware interface

//---------------------------------------------------------------
class CHardware
{
// Construction
public:
	CHardware();
	~CHardware();
#define LINESIZ 256
	char m_gammafilename[LINESIZ]; //gamma file name
	char m_errorString[LINESIZ]; //error_string
	int m_membufSize;
	int m_debug;
	BOOL m_colorMode;
	char m_scanStatus[LINESIZ];
	FN_SCAN_CALLBACK m_callbackfn;				//Call back fn
	SCAN_RESULT Callback(BOOL check_messages);
	BOOL m_home;

	BOOL ReadData(BYTE readaddress,BYTE *bufptr,int maxtoread);
	BOOL WriteRegister(unsigned char Address, unsigned char Value);
	BOOL ReadRegister(unsigned char Address,BYTE *pValue);
	void delay(long count);
	BOOL SendData(BYTE writeaddress,BYTE *bufptr,int len);//,BOOL forceSPP);
	BOOL HomeSensor(BOOL wait=TRUE);
	BOOL CheckScannerReady(void);
	int  ComputeMinMCLK(void);
	BOOL InitUSB(BOOL full);
	void ComputePauseLimit(int *regs);
	void ComputeStepSize(int vertres,int *regs);
	int ComputeDPD(int *regs,BOOL bWantCM=TRUE);
	int ComputeTR(int *regs,BOOL bWantCM=TRUE);
	BOOL WriteMerlin(int *regs);
	BOOL ReadMerlin(int *regs);
	void SetMerlinParm(int i,LPSTR ptr);
	BOOL CheckINI(int *iniVals,char *filename);
	BOOL ReadMerlinINI(char *filename,SCAN_PARAM_REC *pCurrentScanSettings);
	BOOL OnStartupSendGamma(int bpp);
	BOOL SaveDefaultGamma(FILE* fptr);
	BOOL SetGammaCurves(FILE* fptr);
	void NSCStatusOut(const char *format, ...);
	void NSCAfxMessageBox(const char *format, ...);
	BOOL SetupStandardRegisters(int *regs,BOOL calibration);
	BOOL LM9831();
	BOOL LM9832();
	BOOL LM9833();
	int  ComputeReg8Value(int mclk);
	int	 SizeOfKCount(int count);
	int  KCountFromSize(int size);
	BOOL EnableStopScan();
	BOOL DisableStopScan();
	BOOL CloseDriver();
	BOOL GetIntegerArg(char* arg,int& integer);
	BOOL GetFloatingArg(char* arg,double& floating);
	void LimitStepSize(int& stepsize,int *regs);
	char* GetFullFilePath(char* file);

// structure used to store merlin ini file settings
struct 
{
	BYTE version;		// merlin version
	//[File Settings]
	int file_version;	// ini file version
	int chip;			// chip (LM9831 or LM9832)
	char USB_Device[MAX_PATH]; //USB device name
	char USB_Manufacturer[MAX_PATH]; //USB Manufacturer name
	int first_white_pixel;	// calculated in coarse calibration, 
				// used to set active pixel start reg 1e:1f

	//[Scanner Software Settings]
	int red_target; // = 64000		red target code
	int green_target; // = 64000		green target code
	int blue_target; // = 64000		blue target code
	int min_red_target; //black level
	int min_green_target;//black level
	int min_blue_target;//black level
	double Line_Separation; //red to green
	double EvenOdd_Line_Separation; //even to odd
	int Black_Fine_Calibration;	//flag for black fine calibration
	int Max_Sensor_Resolution; //maximum sensor resolution

	//[Scanner Hardware Settings]
	int Start_of_Scan;	// reg 4a/4b
	int Start_of_Calibration;	// reg 4a/4b
	int Mirror_Image;	//Order of pixels
	int Crystal_Frequency; // 48000000
	int Motor_Steps_Per_Inch; // = 600	//Number of fullsteps to make motor
						//move one inch
	double Max_Speed; // = 1.1		//Maximum sensor speed in inches/s
	int Calibration_Strip_Height; // = 50	//Minimum height of calibration
						//strip from home position
						//in units of full steps
	int Image_Pixel_Width; // = 5300	//scan line width in pixels
	double Scanner_Length;	// = 11		// length of the scanner bed in inches

	int mclk;				// not an ini file setting
	int minmclk;			// not an ini file setting
	int timeadj;			// not an ini file setting

	//[Sensor Settings]
	int Input_Polarity; // = Negative	//Negative or Positive	reg B b0
	int CDS; // = On			//On or Off		reg B b1
	int Sensor_Type; // = EvenOdd		//EvenOdd or Standard	reg B b2
	int Optical_Resolution; // = 600	//300 or 600 		reg 0B b4

	int Total_Pixels; // = 5400		//Number of pixels in CCD
	double Maximum_Integration_Time_Highlamp; // = 12	//Max Integration Time before
					//AFE or CCD saturates for high PWM lamp
					//(in milliseconds)
	double Maximum_Integration_Time_Lowlamp; // = 12	//Max Integration Time low PWM
	double Minimum_Integration_Time_LowRes; // = 12	//Min Integration Time hdiv>2
	double Minimum_Integration_Time_HighRes; // = 12//Min Integration Time hdiv<=2
	int Maximum_IntTimeAdj; //Max IntTimeAdj before switch to Line Rate
	double Minimum_Static_Gain; // min static gain to apply
	int Phi1_Polarity; // = Positive	//Positive or Negative	reg C b0
	int Phi2_Polarity; // = Positive	//Positive or Negative	reg C b1
	int RS_Polarity; // = Positive		//Positive or Negative	reg C b2
	int CP1_Polarity; // = Positive		//Positive or Negative	reg C b3
	int CP2_Polarity; // = Positive		//Positive or Negative	reg C b4
	int TR1_Polarity; // = Negative		//Positive or Negative	reg C b5
	int TR2_Polarity; // = Positive		//Positive or Negative	reg C b6

	int Phi1_Status; // = Active		//Active or Disabled	reg D b0
	int Phi2_Status; // = Active		//Active or Disabled	reg D b1
	int RS_Status; // = Active		//Active or Disabled	reg D b2
	int CP1_Status; // = Disabled		//Active or Disabled	reg D b3
	int CP2_Status; // = Disabled		//Active or Disabled	reg D b4
	int TR1_Status; // = Active		//Active or Disabled	reg D b5
	int TR2_Status; // = Disabled		//Active or Disabled	reg D b6
	int TR_Pulse; // = 1			//1 or 2		reg D b7

	int TR_Pulse_Duration; // = 4		//0 - 15		reg E b0..3
	int TR_PHI_Guardband_Duration; // = 1	//0 - 15		reg E b4..7

	//[Pixel Rate Timing]
	//Use these parameters for all pixel rate scanning
	int Pixl_Optical_Black_Clamp_Start; // = 6	//0 - 23		reg F b0..4
	int Pixl_Optical_Black_Clamp_End; // = 23	//0 - 23		reg 10 b0..4
	int Pixl_Reset_Pulse_Start; // = 1		//0 - 23		reg 11 b0..b4
	int Pixl_Reset_Pulse_End; // = 2		//0 - 23		reg 12 b0..b4
	int Pixl_CP1_Pulse_Start; // = 0		//0 - 23		reg 13 b0..b4
	int Pixl_CP1_Pulse_End; // = 0			//0 - 23		reg 14 b0..b4
	int Pixl_CP2_Pulse_Start; // = 0		//0 - 23		reg 15 b0..b4
	int Pixl_CP2_Pulse_End; // = 0			//0 - 23		reg 16 b0..b4
	int Pixl_Reference_Sample; // = 11		//0 - 23		reg 17 b0..b4
	int Pixl_Signal_Sample; // = 1			//0 - 23		reg 18 b0..b4

	//[Line Rate Timing]
	//Use these parameters for all line rate scanning
	int Line_Optical_Black_Clamp_Start; // = 2	//0 - 7			reg F b0..4
	int Line_Optical_Black_Clamp_End; // = 8	//0 - 7			reg 10 b0..4
	int Line_Reset_Pulse_Start; // = 0		//0 - 7			reg 11 b0..b4
	int Line_Reset_Pulse_End; // = 1		//0 - 7			reg 12 b0..b4
	int Line_CP1_Pulse_Start; // = 0		//0 - 7			reg 13 b0..b4
	int Line_CP1_Pulse_End; // = 0			//0 - 7			reg 14 b0..b4
	int Line_CP2_Pulse_Start; // = 0		//0 - 7			reg 15 b0..b4
	int Line_CP2_Pulse_End; // = 0			//0 - 7			reg 16 b0..b4
	int Line_Reference_Sample; // = 4		//0 - 7			reg 17 b0..b4
	int Line_Signal_Sample; // = 0			//0 - 7			reg 18 b0..b4

	//[Misc Timing]
	int CIS_TR_Timing; // = Off		//Off, Mode1, or Mode2	reg 19 b0..1
	int Fake_Optical_Black_Pixels; // = Off	//Off or On		reg 19 b2

	int TR_Stepper_Correction; //=1					reg 1a,1b 

	int Optical_Black_Pixels_Start; // = 15	//0-255			reg 1C
	int Optical_Black_Pixels_End; // = 60	//0-255			reg 1D

	int Color_Mode; // = 0		//0 = 3 Channel Pixel Rate	reg 26 b0..2
					//1 = 3 Channel Line Rate
					//3 = 1 Channel Line Rate
					//2 or any others: Scanner
					//   doesn't support color

	int Gray_Channel; // = Green		//Red, Green or Blue 	reg 26 b3..4

	//[3 Channel Line Rate Settings]
	int TR_Red; // = 0			//0 or 1		reg 26 b5
	int TR_Green; // = 0			//0 or 1		reg 26 b6
	int TR_Blue; // = 0			//0 or 1		reg 26 b7
	int TR_Red_Drop; // = 0			//0-2			reg 27 b0..1
	int TR_Green_Drop; // = 0		//0-2			reg 27 b2..3
	int TR_Blue_Drop; // = 0		//0-2			reg 27 b4..5
	int Triple_TR;	  // = 0		//0 or 1		reg 27 b6

	//[Illumination Settings]
	int Illumination_Mode; // = Mode1	//Off, Mode1, Mode2, Mode3	reg 29 b0..b1
	int Green_PWM_Duty_Cycle; 
	int Green_PWM_Duty_Cycle_High; // = 4095	//0-4095		reg 2A:2B
	int Green_PWM_Duty_Cycle_Low; // = 3000	//0-4095		reg 2A:2B
	int Red_Lamp_On; // = 1			//0-16383		reg 2C:2D
	int Red_Lamp_Off; // = 16383		//0-16383		reg 2E:2F
	int Green_Lamp_On; // = 0		//0-16383		reg 30:31
	int Green_Lamp_Off; // = 0		//0-16383		reg 32:33
	int Blue_Lamp_On; // = 32		//0-16383		reg 34:35
	int Blue_Lamp_Off; // = 48		//0-16383		reg 36:37

	//[I/O Settings]
	int RAM_WordSize; // = 256			// 256			reg 43 b0..b1

	//[Stepper Motor Settings]
	int Steps_to_Scan_After_PS2; 		// 0..16383		reg 4c:4d
	int Stepping; // = Micro		//Full or Micro		reg 45 b0
	// If it's Micro, current sensing = 2 phases,		reg 45 b1
	// If it's Full, current sensing = 1 phase
	int Phase_A_Polarity; // = Positive	//Positive or Negative	reg 45 b2
	int Phase_B_Polarity; // = Positive	//Positive or Negative	reg 45 b3
	//Vertical DPI divider bits should always be 0.
//	int Resume_Scan_Threshold; //=2		// in k bytes		reg 4f

	int Steps_To_Reverse; // = 40		//0-63			reg 50
	int Steps_Stopped; // = 3		//0-3			reg 51 b0..1
	int Steps_25; // = 3			//0-3			reg 51 b2..3
	int Steps_50; // = 3			//0-3			reg 51 b4..5
	int Non_Reversing_Extra_Lines; // = 3	//0-7			reg 54 b0..2
	//Set "First Line to Process" to this same value as "Extra..." reg 54 b3..5

	int KickStart_Steps; // = 3		//0-7			reg 55 b0..2
	int Hold_Current_Timeout; // = 2	//0-31			reg 54 b3..7
	int Stepper_PWM_Frequency; // = 8	//0-255			reg 56
	int Stepper_PWM_Duty_Cycle; // = 23	//0-63			reg 57

	//[Paper Sensor]
	int Home_Sensor; // = 2			//1 or 2 - P.S. that detects home
	//use that to tell you which sensor to program to stop motor
	int Paper_Sensor_1_Polarity; // = High	//High or Low		reg 58 b0
	int Paper_Sensor_1_Trigger; // = Edge	//Edge or Level		reg 58 b1
	int Paper_Sensor_1_Stop_Scan; // = No	//Yes or No		reg 58 b2
	int Paper_Sensor_2_Polarity; // = High	//High or Low		reg 58 b3
	int Paper_Sensor_2_Trigger; // = Edge	//Edge or Level		reg 58 b4
	int Paper_Sensor_2_Stop_Scan; // = Yes	//Yes or No		reg 58 b5

	//[Misc I/O]
	int MiscIO_1_Type; // = Output		//Input or Output	reg 59 b0
	int MiscIO_1_Polarity; // = High	//High or Low		reg 59 b1
	int MiscIO_1_Trigger; // = Edge		//Edge or Level		reg 59 b2
	int MiscIO_1_Output_State; // = High	//High or Low		reg 59 b3
	int MiscIO_2_Type; // = Output		//Input or Output	reg 59 b4
	int MiscIO_2_Polarity; // = High	//High or Low		reg 59 b5
	int MiscIO_2_Trigger; // = Edge		//Edge or Level		reg 59 b6
	int MiscIO_2_Output_State; // = High	//High or Low		reg 59 b7
	int MiscIO_3_Type; // = Output		//Input or Output	reg 59 b0
	int MiscIO_3_Polarity; // = High	//High or Low		reg 59 b1
	int MiscIO_3_Trigger; // = Edge		//Edge or Level		reg 59 b2
	int MiscIO_3_Output_State; // = High	//High or Low		reg 59 b3
	int MiscIO_4_Type; // = Output		//Input or Output	reg 59 b4
	int MiscIO_4_Polarity; // = High	//High or Low		reg 59 b5
	int MiscIO_4_Trigger; // = Edge		//Edge or Level		reg 59 b6
	int MiscIO_4_Output_State; // = High	//High or Low		reg 59 b7
	int MiscIO_5_Type; // = Output		//Input or Output	reg 59 b0
	int MiscIO_5_Polarity; // = High	//High or Low		reg 59 b1
	int MiscIO_5_Trigger; // = Edge		//Edge or Level		reg 59 b2
	int MiscIO_5_Output_State; // = High	//High or Low		reg 59 b3
	int MiscIO_6_Type; // = Output		//Input or Output	reg 59 b4
	int MiscIO_6_Polarity; // = High	//High or Low		reg 59 b5
	int MiscIO_6_Trigger; // = Edge		//Edge or Level		reg 59 b6
	int MiscIO_6_Output_State; // = High	//High or Low		reg 59 b7
//LM9832 only
	int Bias_Current; // = 100								reg 9  b6..7
	int LampB_Output_ITA; // = OFF							reg 29 b2
	int Reverse_Motor_Direction; // = OFF					reg 45 b5
	int Fullstep_During_Fastfeed; // = OFF					reg 45 b6
} merlininfo;

};


