/*******************************************************************************
FILE        : Scan_LM983x.cpp 
REVISION    : 1.0
DATE        : 8/31/99

DESCRIPTION :

Provides the implementation of all exported functions

Copyright (c) 1999 - National Semiconductor Ltd - All rights reserved
*******************************************************************************/

#include "stdafx.h"
#include "Scan_LM983x.h"
#include "Scan_Scanner.h"

/*******************************************************************************
Implementation of Exported functions - The exported fucntions simply provide
a fucntional interface to the singleton scanner object which actually 
impements the code for these functions
*******************************************************************************/
SCAN_RESULT ScanStart()
{
    return CScanner::GetInstance()->ScanStart();
}

SCAN_RESULT ScanRead(void*	scanBuffer,
					long	reqCount,
					long	*actCount)
{
    return CScanner::GetInstance()->ScanRead(scanBuffer, 
												reqCount, 
												actCount);
}

// This method requests the scanner module to abort the current scan
// If no scan is in progress the call will be ignored. This method
// should be invoked only to abnormally terminate the scan in response to 
// a user request or a error condition. This should not be used in normal
// scan operation. Normal scan operation end is signalled by the ScanRead
// method itself.
SCAN_RESULT ScanStop()
{
     return CScanner::GetInstance()->ScanStop();
}

// This method allows a client to set the current scan paramaters. These
// paramters will be used by the scanner module for all subsequent scan
// until the settings are changed again by a call to this method i.e the
/// scan settings are persistent. 
// The scan settings however are not persistent across module loads i.e
// unloading the scan module and loading it again will reset the scan settings
// to default power on settings defined in the scan module configuration file
SCAN_RESULT ScanSetScanParams(SCAN_PARAM_REC* paramRec,
							  FN_SCAN_CALLBACK scanCB)
{
     return CScanner::GetInstance()->ScanSetParams(paramRec,scanCB);
}

// This method allows a client to get the current scan paramaters. These
// paramters will be used by the scanner module for any scan that is 
// requested. To change the scan settings use the ScanSetScanParams
// method
SCAN_RESULT ScanGetScanParams(SCAN_PARAM_REC* paramRec)
{
     return CScanner::GetInstance()->ScanGetParams(paramRec);
}

// This method returns the status of the current scan including information
// about the sie of the scan the number of lines scanned etc. If a scan is not
// in progress this method will return the computed values if a scan were
// to be started. In this case some of the field like actual bytes read will
// not be valid and will be set to zero
SCAN_RESULT ScanGetScanInfo(SCAN_INFO_REC *infoRec)
{
     return CScanner::GetInstance()->ScanGetScanInfo(infoRec);
}

// This method returns the hardware setting of the scanner
SCAN_RESULT ScanGetScanHardware(SCAN_HARDWARE_REC *hardwareRec)
{
     return CScanner::GetInstance()->ScanGetScanHardware(hardwareRec);
}

// This method allows a client to interrogate the scanner module for the 
// current value of supported capabilities like bit depth, pixel type, 
// resolutions etc
// In order to allow different capabilities to be queried the input and 
// output parameters are defined as void pointers the actual type of the 
// arguments pointed to by these pointers will depend on the capability 
// being queried.
// See the capability ID definitions for the input and output parameters 
// for that capability
SCAN_RESULT ScanGetCapability(SCAN_CAPID capId, 
							 void* input, 
							 void * output,
                             long * numOutputElements)
{
     return CScanner::GetInstance()->ScanGetCapability(capId,
														 input, 
														 output,
														 numOutputElements);
}


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
            CScanner::GetInstance()->InitScanner(hModule);
            break;
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
            break;
		case DLL_PROCESS_DETACH:
            // really hokey way to kill a singleton!
            delete CScanner::GetInstance();
			break;
    }
    return TRUE;
}



