// FILE		: scan_scan.H

// DESCRIPTION	: header file and class definitions for the CScan class
//		  this is the scan interface

// REVISIONS	: BEB first version 2.0
// 9/9/99 : SEA Changed SCANPARMS structure to include nmbrBMPLinesReturnedThisCall, maxBMPLinesPerCall, headerOffset,
//              fileOffset, *pBMPLineData
// Last revision : adding 16 bit calibration

/////////////////////////////////////////////////////////////////////////////
// CScan class
//---------------------------------------------------------------
// CLASS	:	CScan

// DESCRIPTION	:	scan interface

//---------------------------------------------------------------


#if !defined(CScan_INCLUDED)
#define CScan_INCLUDED


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "scan_lm983x.h"

class CScan
{
// Construction
public:
	CScan();
	~CScan();

	BOOL Startup(void);
	BOOL SetupScan(BOOL full = TRUE);
	BOOL ScanSome(BOOL *done,BOOL gCancel/*,int a=0*/);
	char *ComputePCTransferRate(char *basename);
	BOOL SetupScanRes(int *regs,char *filename);
	BOOL CalcBytesPerLine();
	void SetScanRes(int *regs,int horresval,int verresval,int bpp);
	void CalculateScanRect(int *regs,char *filename);
	int ComputeCalibrationMClk(int *regs);
	BOOL DoScan(char *filename/*,int a =0*/);
	BOOL SetScanParms();
	    void SumFineCalValue(int *sum,int ct,int value,char type[]);
	BOOL DoCoarseCalibration(int reg9, int regs[0x80]);
	BOOL DoFineCalibration(int reg9, int regs[0x80], char type[]);
	BOOL DoCalibration(int reg9);
	void OffsetGainToMerlin(const char type[]);
	void ComputeGainCoefficents();
    void SetupScanParameter(int lines,int datamode,int autohome,int reg26);
	BOOL ScantoBMPfile(char *filename);
    //void SetupScanParameter(int lines,int datamode,int autohome,int reg26);
	BOOL ProcessCalibrationLines(char *filename, char type[],int *actuallyavg,int totalct); 
	BOOL SubString(char *string,char *substring);
	void Cleanup();
	void UpdateScanStatus();
	void SaveCallback(FN_SCAN_CALLBACK scanCB);
	int	 ComputeStepperPhaseCorrection(int *regs,int tr);
	void PositionTest(void);

	BOOL			m_reentry;			// Reentry flag for ScanSome
	int PCTransferRate;				//PC transfer rate
	char m_scanfilename[MAX_PATH];			// name of pxl file during scans
	BOOL m_firstscan;				//first scan flag
	BOOL m_callback;				//In callback
    BOOL blackFine;                 //BlackFine Calibration Flag
    // Scan parameters
    SCAN_PARAM_REC      m_currentScanSettings;  // Current scan settings 
    // Scan status information
    SCAN_INFO_REC       m_currentScanInfo;      // Current scan status 
	SCAN_HARDWARE_REC	m_currentScanHardware;	// Hardware settings

	#define MAX_PIXELS 0x4000
	int pColor[3][MAX_PIXELS];      //data array
	//	CString m_errorString;				//Scan error return
	int pRed[MAX_PIXELS];			// array of red values to graph
	int pGreen[MAX_PIXELS];			// array of green values to graph
	int pBlue[MAX_PIXELS];			// array of blue values to graph
	int pRedGain[MAX_PIXELS];		// array of red gain coefficients
	int pGreenGain[MAX_PIXELS];		// array of green gain coefficients
	int pBlueGain[MAX_PIXELS];		// array of blue gain coefficients
	int minBlackred;		// min red pixel value
	int minBlackgreen;		// min green pixel value
	int minBlackblue;		// min blue pixel value
	int maxBlackred;		// max red pixel value
	int maxBlackgreen;		// max green pixel value
	int maxBlackblue;		// max blue pixel value
	int npts;				// number of points in each graph array
	int minWhitered;		// min red pixel value
	int minWhitegreen;		// min green pixel value
	int minWhiteblue;		// min blue pixel value
	int maxWhitered;		// max red pixel value
	int maxWhitegreen;		// max green pixel value
	int maxWhiteblue;		// max blue pixel value

// scaninfo structure used to specify mostly image processing post scan
#define	kMaxPostponeCount	10000
typedef struct {
	double roffset;	// red line separation
	double goffset;	// green line separation
	double boffset; // blue line separation
	double eoffset; //even/odd separation
	int width;	// number of pixels per line
	int bpp;	// bits/pixel
	int linerate;	// 0=pixel rate, 1=3 channel line rate, 4=1 channel mode a, 5=1 channel mode b
	int actualxres;	// actual scanning horizontal resolution
	int actualyres;	// actual scanning vertical resolution
	int desiredxres; // desired horizontal resolution
	int desiredyres; // desired vertical resolution
	int xInterpolate; //0=discard columns, 1= x interpolate
	int yInterpolate; 
// for BMP (separate out later!)
	BYTE *pScanlinebuffer;	// scan line buffer pointer
	BYTE *pBmplinebuffer;	// bmp line output buffer pointer, allocated by Brian based on bmpbytesperline
	int nmbrBMPLinesReturnedThisCall ; // Indicates the number of lines returned
	int maxBMPLinesPerCall ;
	unsigned long	bmpbytesperline ; // Nymber of BYTEs in a returned BMP line, determined by me
	int bytesperscanline ; // The number of bytes to use when allocating scanline memory
	int scanlines;	// Total number of lines to scan
	int bmpoutlines;	// number of lines in bmp file
	int scanlinecount;	// line number of scan line in buffer
	int	bmpreturnlinecount;	// The BMP line number being returned.
						// Set to zero until the first BMP line is returned
	int	bytesperscanlinepixel ; // one byte per pixel or two bytes per pixel
	int dibwidth;	// number of pixels in each bmp line
	int n; // n value of n of m
	int m; // m value for n of m
	BOOL bIsTwain ; // 1 if twain transfer, 0 otherwise
	int returnbytecount ; // Returns the number of bytes in the BMP line 
	int fileOffset ; // Offset for writing bmp file for non inverted display
	int headerOffset ; // Size of the file header, info header and palette 
	BYTE *pBMPLineData ; // Points to the beginning of the data in the BMP buffer
	BOOL initialized;	// MUST BE SET FALSE FIRST TIME FOR EACH SCAN
	int yDiscardSum ; // Counter used for discarding rows
} SCANINFO;

// scanparms structure used mostly for storing scanning parameters
typedef struct {
	int endpixel;	// data pixel end reg 24:25
	int startpixel;	// data pixel start reg 22:23
	int bpp;	// bits per pixel
	double dpi;	// horizontal res divider (eg, 12 means /12)
	int xres;	// motor steps/dpi divider
	int yres;	// motor steps/dpi divider
	double htinches; // ht of image to scan in inches
	int bytesperline; // bytes per line
	int pixelsperline;	// pixels per line
	int linerate;	// 0=pixel rate, 1=3 channel line rate, 4=1 channel mode a, 5=1 channel mode b
	int lines;	// number of lines to scan
	int autohome;	// 1=home after scan, 0=no home after scan
	int time;	// time in ms used during scan
	int bytesread; //bytes actually read
} SCANPARMS;

private:
SCANINFO scaninfo;
SCANPARMS ScanParms;

BOOL ReadScanParms(HFILE hFile,SCANINFO *pScanInfo,int *len);

public:
typedef struct  
{
	//for ScanRead
	BYTE *scanReadBuffer;
	long scanReadReqCount;
	long scanReadActCount;
} SCANREADPARMS;

SCANREADPARMS ScanReadParms;

};

#endif

