// scan_scan2bmp.cpp : implementation file
//
// 9/9/99 SEA Changed generate_bmp_line and PXLtoBMPfile to allow the transfer of data to a file and to TWAIN
//            Also fixed some resolution problems and added the ability to do 12,6,and 3 BPP color scans
#include "stdafx.h"
#include "scan_scan.h"
#include "scan_scan2bmp.h"
#include "math.h"

//
// Construct of class CDriver
//
CScan2Bmp::CScan2Bmp()
{
}

//
// Function CDriver destructor
//
CScan2Bmp::~CScan2Bmp()
{
}

// Definitions
#define SWAP(x,y)   ((x)^=(y)^=(x)^=(y))
#define IS_WIN30_DIB(lpbi)  ((*(LPDWORD) (lpbi)) == sizeof (BITMAPINFOHEADER))

#define WIDTHBYTES(bits)      (((bits) + 31) / 32 * 4)
// #define BYTE char
#define DIB_HEADER_MARKER   ((WORD) ('M' << 8) | 'B')


/////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------
// FUNCTION	:	MaskProperly() 

// DESCRIPTION	:	Do convertion of mask to bucket value
//					For 1bit/pixel, map <0x80 to 0, else to 255
//					For 2bit/pixel, map <0x40 to 0, < 0x80 to 255/3,
//									<0xc0 to 255*2/3, else to 255
//					For 4bit/pixle, map < 0x10 to 0, < 0x20 to 255/15,
//									< 0x30 to 255*2/15, etc.
//					For 8bit/pixel, no change
//
// INPUT :			byte value to mask
//					mask for pixel bits (0x80,0xc0,0xf0,0xff)
//				
// RETURNS	:		value mapped to bucket
//--------------------------------------------------------
unsigned char CScan2Bmp::MaskProperly(unsigned char value,unsigned char mask)
{
	unsigned int v =  value;
	switch (mask)
	{
	case 0x80:
		v >>= 7;
		v *= 255;
		break;
	case 0xc0:
		v >>= 6;
		v *= 255;
		v /= 3;
		break;
	case 0xf0:
		v >>= 4;
		v *= 255;
		v /=15;
		break;
	default:
		break;
	}
	return (unsigned char) v;
}


//---------------------------------------------------------------------
//
// Function	: byte_memcopy() 
//
// Purpose	: Copy bytes from one buffer to another	
//
// Parms :    pTo, pointer to destination buffer
//            pFrom, pointer to source buffer
//            number_of_bytes, the number of bytes to be copied
//				
// Returns :  pointer to the next position in the destination bvuffer
//            to receive a byte
//---------------------------------------------------------------------
BYTE* CScan2Bmp::byte_memcopy(BYTE *pTo, BYTE *pFrom, int number_of_bytes)
{
	memmove(pTo,pFrom,number_of_bytes);
	return pTo+number_of_bytes;
}

//---------------------------------------------------------------------
//
// Function:   DIBNumColors
//
// Purpose:    Given a pointer to a DIB, returns a number of colors in
//             the DIB's color table.
//
// Parms:      lpbi == pointer to DIB header (either BITMAPINFOHEADER
//                       or BITMAPCOREHEADER)
//
//---------------------------------------------------------------------
WORD CScan2Bmp::DIBNumColors (LPSTR lpbi)
{
	WORD wBitCount;
	// If this is a Windows style DIB, the number of colors in the
	//  color table can be less than the number of bits per pixel
	//  allows for (i.e. lpbi->biClrUsed can be set to some value).
	//  If this is the case, return the appropriate value.
	if (IS_WIN30_DIB (lpbi))
	{
		DWORD dwClrUsed;
		dwClrUsed = ((LPBITMAPINFOHEADER) lpbi)->biClrUsed;
		if (dwClrUsed)
		   return (WORD) dwClrUsed;
	}
	// Calculate the number of colors in the color table based on
	//  the number of bits per pixel for the DIB.
	if (IS_WIN30_DIB (lpbi))
		wBitCount = ((LPBITMAPINFOHEADER) lpbi)->biBitCount;
	else
		wBitCount = ((LPBITMAPCOREHEADER) lpbi)->bcBitCount;
	switch (wBitCount)
	{
		case 1:
			 return 2;
		default:
		   return 0;
	}
}

///////////////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------
//
// Function:   create_palette
//
// Purpose:    constructs a logical palette,
//             and writes it to the current position in the buffer
//
// Parms:	buffer	- pointer to return buffer
//		lpbi	- pointer to bitmap info

// RETURNS:	number of bytes in buffer
//---------------------------------------------------------------------
int CScan2Bmp::create_palette(RGBQUAD	*pQuad,LPSTR lpbi)
{
// RGBQUAD	*pQuad;		// array of palette entries
int	i;		// loop counter
int	wNumColors;	// number of colors in palette

wNumColors   = DIBNumColors (lpbi);

if (wNumColors) // if this is a palette bmp
{
	for (i = 0;  i < wNumColors;  i++)
	{
		// create the gray palette
		//evenly distribute colors
		//2 colors, 0,255
		//4 colors, 0,255/3,255*2/3,255
 		int v = i*255/(wNumColors-1);
		pQuad[i].rgbRed = v;
 		pQuad[i].rgbGreen = v;
 		pQuad[i].rgbBlue  = v;
		pQuad[i].rgbReserved = 0;
	}
}
return wNumColors*sizeof(pQuad);
}


//---------------------------------------------------------------------
//
// Function:   PaletteSize
//
// Purpose:    Given a pointer to a DIB, returns number of bytes
//             in the DIB's color table.
//
// Parms:      lpbi == pointer to DIB header (either BITMAPINFOHEADER
//                       or BITMAPCOREHEADER)
//---------------------------------------------------------------------
WORD CScan2Bmp::PaletteSize (LPSTR lpbi)
{
if (IS_WIN30_DIB (lpbi))
	return (DIBNumColors (lpbi) * sizeof (RGBQUAD));
else
	return (DIBNumColors (lpbi) * sizeof (RGBTRIPLE));
}

/////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------
// FUNCTION	:	BYTE *GetPXLline(int linedesired,int nmbrofscanlinesinmemory,BYTE **ppscanlinepointerarray)
// DESCRIPTION	:	get a line from the scan lines in mem
//			scanbuf is an array of pixel lines. 
//			search through the lines
//			in memory. if not found, return null.

// INPUTS	

// RETURNS	:	a pointer to the requested pxl file line
//--------------------------------------------------------
BYTE *CScan2Bmp::GetPXLline(int linedesired,int nmbrofscanlinesinmemory,BYTE **ppscanlinepointerarray)
{
int index ;
int linetoread = linedesired;
index = (linedesired-1) % nmbrofscanlinesinmemory ;
return ppscanlinepointerarray[index] ; // This points to the line desired in the buffer
}

/////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------
// FUNCTION	:	copyline
//
// DESCRIPTION	:	copy a line from the pBMPWorkingBuffer to the pBMPBuffer
//			remove columns to reduce horizontal resolution if necessary
//
//			pBMPBuffer	- beginning of bmp result line
//			pBMPWorkingBuffer - BMP line which will be copied and from which discards are
//				made if necessary
//			pScaninfo - pointer to the structure holding the scan information
// RETURNS	:	true if ok, false if error
//--------------------------------------------------------
BOOL CScan2Bmp::copyline(BYTE *pBMPBuffer,BYTE *pBMPWorkingBuffer,SCANINFO *pScaninfo,BITMAPINFOHEADER *pInfohdr,int nmbrBMPLinesReturnedThisCall)
{
int i, j ;
BYTE  *pBMP ;
int sourceShiftCount ;
int destShiftCount ;
BYTE workingByte ;

if(pScaninfo->bIsTwain) // Twain transfer
{
	pScaninfo->pBMPLineData = pBMPBuffer ; // Point to beginning of buffer
	pBMPBuffer += nmbrBMPLinesReturnedThisCall*pScaninfo->bmpbytesperline ; ;
}
else // File transfer
{
	for(int x = nmbrBMPLinesReturnedThisCall * pScaninfo->bmpbytesperline-1 ; x >= 0 ; x--)
	{
		pBMPBuffer[x+pScaninfo->bmpbytesperline] = pBMPBuffer[x] ; // Move all data up one return byte count amount
	}
	pScaninfo->pBMPLineData = pBMPBuffer ;
}
pBMP = pBMPBuffer;	// bmp line
if ((pScaninfo->dibwidth < pScaninfo->width)
		&& (!pScaninfo->xInterpolate)) // Reduce resolution through discard
{
	// remove some columns
	int a = 0 ;
	i = 0 ;
	j = 0 ;
	while(i < pScaninfo->dibwidth) 
	{
		a -= pScaninfo->dibwidth;
		if( a<0 )
		{ // Save the column
			a += pScaninfo->width;
			if ((pScaninfo->linerate == 4)&&(pScaninfo->bpp == 1))
			{ // work on bits
				sourceShiftCount = 7-j%8 ;
				destShiftCount = 7-i%8 ;
				workingByte = pBMPWorkingBuffer[j/8] >> sourceShiftCount ;
				workingByte &= 1 ;
				workingByte <<= destShiftCount ;
				if(!(i%8))
				{
					pBMP[i/8] = 0 ;
				}
				pBMP[i/8] |= workingByte ; // Set the appropriate bit
			}
			else
			{
				for (int i = 0; i < pInfohdr->biBitCount/8; i++)
					*pBMP++ = *pBMPWorkingBuffer++;
			}
			i++ ;
		}
		else
		{ // Skip the column
			if (!((pScaninfo->linerate == 4)&&(pScaninfo->bpp == 1)))
			{ // not work on bits
				for (int i = 0; i < pInfohdr->biBitCount/8; i++)
					pBMPWorkingBuffer++;
			}
		}
		j++ ;
	}
}		
else
{ // no change in resolution so just copy the bmp line
	if(pInfohdr->biBitCount >= 24)
	{
		byte_memcopy(pBMP,pBMPWorkingBuffer,pScaninfo->bytesperscanlinepixel*
			3*pScaninfo->dibwidth);
	}
	else
	{
		if (pScaninfo->bpp == 1)
			byte_memcopy(pBMP,pBMPWorkingBuffer,(int)ceil((double)pScaninfo->dibwidth/8.)); // SEA
		else
			byte_memcopy(pBMP,pBMPWorkingBuffer,pScaninfo->bytesperscanlinepixel*
				pScaninfo->dibwidth);
	}
}

if (!pScaninfo->bIsTwain)
{
// for file, 
	pBMP = pBMPBuffer ;
	switch(pInfohdr->biBitCount)
	{
	case 24:
		for (i = 0; i < pScaninfo->dibwidth; i++)
		{ // color 24 bit bmp files are b,g,r
			SWAP(*pBMP,*(pBMP+2));
			pBMP += 3;
		}
		break;
	}
}


return TRUE;
}

//---------------------------------------------------------------------
//
// Function:   generate_bmp_line
//
//---------------------------------------------------------------------

bool CScan2Bmp::generate_bmp_line(SCANINFO *pScaninfo,BOOL gCancel) 
{
static BITMAPFILEHEADER    *pFilehdr=NULL;	// hdr  bmp file header
static BITMAPINFOHEADER    *pInfohdr=NULL;		// bi  bmp info header
static int nmbrofscanlinesinmemory ;
static BYTE **ppscanlinepointerarray=NULL, *pScanlines=NULL, *pBMPWorkingBuffer=NULL ;
int scan_buffer_length ;
static double	xstepsize ;
static double ystepsize ;
static char mask ;
enum COLORS{RED,GREEN,BLUE,NUM_COLORS};
static double Offset[NUM_COLORS] ;
static double OffsetEvenOdd;
static int	lastscanline ; // The last scanline number sent
static int arrayindex ; 
static int highestlineinmemory ; 
static int lowestlineinmemory ;
static long slWorkingBufferLength ; // Length of buffer in bytes to use while building BMP line
static int yDiscardSum ;
BYTE *pBMPHeaderLine ;
BYTE *ptr, *nptr , *ptr_even, *nptr_even, *ptr_odd, *nptr_odd;
#define INT_SCALE 65536
#define F_SCALE 65536.
int k1,k2,k3,k4 ;
int dx1 ; // integer scaled 1-dx1
int dx2 ; // decimal part of current x offset
int f1, f2 , f1_even, f2_even, f1_odd, f2_odd; // Used for integer scaling d1 and d2
int x_base, x_next ; // Position of the X pixel positions for interpolation
double x_new ;   // Interpolated X pixel position and relative intensity
double d1;		// 1-d2
double d2;		// decimal part of current offset
int b;			// integer part of current offset
int pixel;		// loop counter, for each pixel in a line
BYTE p1,p11,p2,p21;
int color ;
int numberOfColors = 3 ;

if (gCancel) goto error_exit;

pScaninfo->nmbrBMPLinesReturnedThisCall = 0 ;
pScaninfo->returnbytecount = 0 ;
// this is first time through setup conversion parms
if (pScaninfo->scanlinecount == 0)
{
	pScaninfo->yDiscardSum = 0 ;
	pScaninfo->fileOffset = 0 ;
	pScaninfo->maxBMPLinesPerCall=1 ;
	yDiscardSum = 0 ;
	pBMPHeaderLine = pScaninfo->pBmplinebuffer ;
	highestlineinmemory = 0; 
	lowestlineinmemory = 0; 
	arrayindex = 0 ; // Zero the scan line array index
	lastscanline = 0 ;
	pScaninfo->bmpreturnlinecount = 0;
	Offset[RED] = pScaninfo->roffset; //linespacing;
	Offset[GREEN] = pScaninfo->goffset; //0
	Offset[BLUE] = pScaninfo->boffset; //-linespacing;
	OffsetEvenOdd = pScaninfo->eoffset; //even/odd spacing
	if (pScaninfo->linerate == 4)
	{
		Offset[RED] = 0; //linespacing;
		Offset[GREEN] = 0; //0
		Offset[BLUE] = 0; //-linespacing;
	}
	// this is first call, so setup conversion parms and headers
	mask = (BYTE)0xFF ;
	switch (pScaninfo->bpp)
	{
		case 16:
		case 8: mask = (BYTE)0xFF; break;
		case 4: mask = (BYTE)0xF0; break;
		case 2: mask = (BYTE)0xC0; break;
		case 1: mask = (BYTE)0x80; break;
	}
	xstepsize = 1 ;

	// when the resolution is scaled, the resulting bmp will have this width
	if (pScaninfo->dibwidth <= 0) pScaninfo->dibwidth = 1;
	xstepsize = (double) pScaninfo->actualxres/(double)pScaninfo->desiredxres ;
	if (pScaninfo->linerate == 4) // If gray or black and white
	{
		if (pScaninfo->bpp == 1) pScaninfo->bmpbytesperline = WIDTHBYTES(pScaninfo->dibwidth) ; // bytes in each bmp file line
		else pScaninfo->bmpbytesperline = WIDTHBYTES(pScaninfo->dibwidth * 8 * 
				pScaninfo->bytesperscanlinepixel) ; // bytes in each bmp file line
		if (pScaninfo->bpp == 1)
		{
			if(pScaninfo->dibwidth > pScaninfo->width)
			{
				slWorkingBufferLength = WIDTHBYTES(pScaninfo->dibwidth*8/*bits*/);
			}
			else
			{
				slWorkingBufferLength = WIDTHBYTES(pScaninfo->width*8/*bits*/);
			}
		}
		else
		{
			if(pScaninfo->dibwidth > pScaninfo->width)
			{
				slWorkingBufferLength = WIDTHBYTES(pScaninfo->dibwidth*8*pScaninfo->bytesperscanlinepixel/*bits*/);
			}
			else
			{
				slWorkingBufferLength = WIDTHBYTES(pScaninfo->width*8*pScaninfo->bytesperscanlinepixel/*bits*/);
			}
		}
	}
	else // (pScaninfo->linerate == 4)
	{
		pScaninfo->bmpbytesperline = WIDTHBYTES(pScaninfo->dibwidth * 3 
			*pScaninfo->bytesperscanlinepixel*8) ; // bytes in each bmp file line
		if(pScaninfo->dibwidth > pScaninfo->width) // Use the maximum of the two widths
		{
			slWorkingBufferLength = WIDTHBYTES(pScaninfo->dibwidth*24
				*pScaninfo->bytesperscanlinepixel/*bits*/);
		}
		else
		{
			slWorkingBufferLength = WIDTHBYTES(pScaninfo->width*24
				*pScaninfo->bytesperscanlinepixel/*bits*/);
		}
	}
	if((int)(ceil((double)pScaninfo->desiredyres/(double)pScaninfo->actualyres) > 1))
	{
		pScaninfo->maxBMPLinesPerCall = (int)(ceil((double)pScaninfo->desiredyres/(double)pScaninfo->actualyres)) ;
	}
	//compute actual scan lines needed
	int tmplines = pScaninfo->bmpoutlines;
	ystepsize = 1;
	if (pScaninfo->desiredyres != pScaninfo->actualyres)
	{
		// calculate the y step size for doing interpolation
		ystepsize = (double) pScaninfo->actualyres/(double)pScaninfo->desiredyres;
		if (ystepsize <= 0) ystepsize = 1.;
		// Change the number of bmp lines to output based on the difference in y resolution
		tmplines = (int) ceil((double)tmplines*ystepsize);
	}
	double fullOffsetLines = ceil(fabs(Offset[RED])) + ceil(fabs(Offset[GREEN])) + 
								ceil(fabs(Offset[BLUE])) + ceil(fabs(OffsetEvenOdd));
	pScaninfo->scanlines = tmplines + (int)(fullOffsetLines) + 1;
	//be sure at least 1 more than out lines
//	pScaninfo->scanlines = max(tmplines+1,pScaninfo->scanlines);

	nmbrofscanlinesinmemory = int (fullOffsetLines + ceil(fabs(ystepsize)) +1+1 ) ;
	scan_buffer_length = nmbrofscanlinesinmemory 
			* pScaninfo->bytesperscanline;
	if(pScaninfo->linerate == 0) scan_buffer_length *= 3 ;
	// Allocate buffer to hold scan lines being operated on
	if(pScanlines) delete pScanlines ;
	pScanlines = new  BYTE[scan_buffer_length] ;
	if(!pScanlines) goto error_exit ;

	// Allocate buffer to hold scan lines being operated on
	if(ppscanlinepointerarray) delete ppscanlinepointerarray ;
	ppscanlinepointerarray = new  BYTE*[nmbrofscanlinesinmemory] ;
	if(!ppscanlinepointerarray) goto error_exit ;

	// Initialize the array holding the scan line pointers
	for(int i = 0 ; i < nmbrofscanlinesinmemory ; i++)
	{
		ppscanlinepointerarray[i] = pScanlines+i*sizeof(BYTE)*(pScaninfo->bytesperscanline) ;
	}

	// Allocate space for BITMAPFILEHEADER

	if(pFilehdr) delete(pFilehdr) ;
	pFilehdr = new  BITMAPFILEHEADER ;
	// init the BITMAPFILEHEADER to zero
	memset(pFilehdr, 0, sizeof (BITMAPFILEHEADER));

	// Allocate apace for BITMAPINFOHEADER

	if(pInfohdr) delete(pInfohdr) ;
	pInfohdr = new  BITMAPINFOHEADER ;
	// init the BITMAPINFOHEADER to zero
	memset(pInfohdr, 0, sizeof (BITMAPINFOHEADER));

	// init the bmp info header
	pInfohdr->biSize      = sizeof (BITMAPINFOHEADER);
	pInfohdr->biWidth     = pScaninfo->dibwidth;
	pInfohdr->biHeight    = pScaninfo->bmpoutlines;
	pInfohdr->biPlanes    = 1;
	pInfohdr->biBitCount  = 24;		// rgb
	if (pScaninfo->bpp == 16) pInfohdr->biBitCount = 48;
	if (pScaninfo->linerate == 4)
	{
		//special test for 1 bit gray bmp
		if (pScaninfo->bpp == 1) pInfohdr->biBitCount = 1;
		else if (pScaninfo->bpp == 16) pInfohdr->biBitCount = 16;
		else pInfohdr->biBitCount  = 8;	// 8 bit palette image for grayscale
	}
	pInfohdr->biCompression = BI_RGB; //no compression
	pInfohdr->biSizeImage = WIDTHBYTES (pScaninfo->dibwidth * pInfohdr->biBitCount) * pInfohdr->biHeight ;
	if (pScaninfo->bIsTwain)
	{ // if twain, store res in dpi
		pInfohdr->biXPelsPerMeter = (int) (pScaninfo->desiredxres); //*39.37);
		pInfohdr->biYPelsPerMeter = (int) (pScaninfo->desiredyres); //*39.37);
		if (pScaninfo->desiredxres == 0)
			pInfohdr->biXPelsPerMeter = (int) (pScaninfo->actualxres); //*39.37);
		if (pScaninfo->desiredyres == 0)
			pInfohdr->biYPelsPerMeter = (int) (pScaninfo->actualyres); //*39.37);
	}
	else
	{
		pInfohdr->biXPelsPerMeter = (int) (pScaninfo->desiredxres*39.370079);
		pInfohdr->biYPelsPerMeter = (int) (pScaninfo->desiredyres*39.370079);
		if (pScaninfo->desiredxres == 0)
			pInfohdr->biXPelsPerMeter = (int) (pScaninfo->actualxres*39.370079);
		if (pScaninfo->desiredyres == 0)
			pInfohdr->biYPelsPerMeter = (int) (pScaninfo->actualyres*39.370079);
	}
	pInfohdr->biClrUsed = 0;
	pInfohdr->biClrImportant = 0;

	/* Fill in the fields of the file header */
	pFilehdr->bfType	= DIB_HEADER_MARKER;

	// setup bmp file header
	DWORD dwHeaderSize = (DWORD)sizeof(BITMAPFILEHEADER) + pInfohdr->biSize + 
				PaletteSize((LPSTR)pInfohdr);
	pFilehdr->bfSize	= dwHeaderSize + pInfohdr->biSizeImage;
	pFilehdr->bfReserved1 = 0;
	pFilehdr->bfReserved2 = 0;
	pFilehdr->bfOffBits   = dwHeaderSize;
	// Return the header information
	// Copy the file header to the buffer
	pBMPHeaderLine = byte_memcopy(pBMPHeaderLine,(BYTE *)pFilehdr,sizeof(BITMAPFILEHEADER)) ;

	/* Write the DIB info header */
	pBMPHeaderLine = byte_memcopy(pBMPHeaderLine,(BYTE *)pInfohdr,sizeof(BITMAPINFOHEADER)) ;

	// Copy the Palette to the return buffer
	pScaninfo->returnbytecount = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER)
		+ create_palette((RGBQUAD *)pBMPHeaderLine,(LPSTR)pInfohdr) ;
	pScaninfo->headerOffset=pScaninfo->returnbytecount ;
	// Allocate memory for creating the BMP line
	if(pBMPWorkingBuffer) delete pBMPWorkingBuffer ;
	pBMPWorkingBuffer = new  BYTE[slWorkingBufferLength] ;
	if(!pBMPWorkingBuffer) goto error_exit ;
	// Initialize the offset array
	while(Offset[RED]<1.0 || Offset[GREEN]<1.0 || Offset[BLUE]<1.0 ||
		  Offset[RED]+OffsetEvenOdd <1.0 || Offset[GREEN]+OffsetEvenOdd<1.0 || 
		  Offset[BLUE]+OffsetEvenOdd<1.0)
	{
		Offset[RED] += ystepsize ;
		Offset[GREEN] += ystepsize ;
		Offset[BLUE] += ystepsize ;
	}
	return TRUE ; // Function variable initialization done 
}


++lastscanline ;
if(lastscanline != pScaninfo->scanlinecount) goto error_exit ; // Something went wrong 
// copy scan line into scan line buffer
byte_memcopy(ppscanlinepointerarray[arrayindex],pScaninfo->pScanlinebuffer,pScaninfo->bytesperscanline) ;
++arrayindex ; // Increment the array index for next time
// Roll it over if necessary
if(arrayindex >= nmbrofscanlinesinmemory) arrayindex = 0 ;
// Increment the scan line in memory counter

++highestlineinmemory ; // Increment the line number of the highest line number in memory
lowestlineinmemory = highestlineinmemory - nmbrofscanlinesinmemory+1 ; 
if(lowestlineinmemory <= 0) lowestlineinmemory = 1 ; // Limit it to first line

if (pScaninfo->linerate == 4) // Is it gray?
{
	// Just want to go through the following routines once since gray
	numberOfColors = 1 ; // One color, gray
}
if(pScaninfo->bmpreturnlinecount < pScaninfo->bmpoutlines)
{
	memset(pScaninfo->pBmplinebuffer,0,pScaninfo->dibwidth/8*sizeof(BYTE)); // init output bmp line
	while( pScaninfo->nmbrBMPLinesReturnedThisCall < pScaninfo->maxBMPLinesPerCall)
	{
		BOOL cont = TRUE;
		for (int i = 0; i < NUM_COLORS; i++)
		{
			if ((int)Offset[i] < lowestlineinmemory) cont = FALSE;
			if ((int)Offset[i]+1 > highestlineinmemory) cont = FALSE;
			if ((int)(Offset[i]+OffsetEvenOdd) < lowestlineinmemory) cont = FALSE;
			if ((int)(Offset[i]+OffsetEvenOdd)+1 > highestlineinmemory) cont = FALSE;
		}
		if (!cont) break;

		pScaninfo->yDiscardSum -= pScaninfo->desiredyres ;
		if((pScaninfo->yDiscardSum < 0) || (pScaninfo->yInterpolate))
		{
			pScaninfo->yDiscardSum += pScaninfo->actualyres ;
			for (color=0; color < numberOfColors; color++)
			{
				//normal offset
				ptr_even = NULL;
				nptr_even = NULL;
				// compute the weighted avg of 2 lines to get a line 
				// for the current color
				b = (int) Offset[color];
				d2 = Offset[color] - b;
				d1 = 1. - d2;
				f1_even = (int) (F_SCALE * d1);
				f2_even = (int) (F_SCALE * d2);
				ptr_even = GetPXLline(b,nmbrofscanlinesinmemory,ppscanlinepointerarray) ;
				// Only get the next line appropriate with the ystepsize if required
				if(d2 > 0.) nptr_even = GetPXLline(b+1,nmbrofscanlinesinmemory,ppscanlinepointerarray) ;

				//with evenodd offset
				ptr_odd = NULL;
				nptr_odd = NULL;
				// compute the weighted avg of 2 lines to get a line 
				// for the current color
				b = (int) (Offset[color]+OffsetEvenOdd);
				d2 = (Offset[color]+OffsetEvenOdd) - b;
				d1 = 1. - d2;
				f1_odd = (int) (F_SCALE * d1);
				f2_odd = (int) (F_SCALE * d2);
				ptr_odd = GetPXLline(b,nmbrofscanlinesinmemory,ppscanlinepointerarray) ;
				// Only get the next line appropriate with the ystepsize if required
				if(d2 > 0.) nptr_odd = GetPXLline(b+1,nmbrofscanlinesinmemory,ppscanlinepointerarray) ;

				if((pScaninfo->desiredxres == pScaninfo->actualxres)
						|| ((pScaninfo->desiredxres <= pScaninfo->actualxres)
								&& (!pScaninfo->xInterpolate))) 
				{ // If x NOT to be interpolated
					if ((pScaninfo->linerate)&&(pScaninfo->linerate!=4))
					{
						goto error_exit ;// Do lineration in next version
					} // if linerate
					else 
					{ // Not linerated, no x interpolation
						int end = (pScaninfo->width)*numberOfColors + color;
						switch (pScaninfo->bpp)
						{
							default: // 1/2/4 bpp
								for (pixel=color ; pixel < end ; pixel+=numberOfColors )
								{
									if ((pixel/numberOfColors)&1)	
									{				
										ptr = ptr_odd;
										nptr = nptr_odd;
										f1 = f1_odd;
										f2 = f2_odd;
									}
									else
									{
										ptr = ptr_even;
										nptr = nptr_even;
										f1 = f1_even;
										f2 = f2_even;
									}
									k1 = pixel*pScaninfo->bpp/8;
									k2 = pixel*pScaninfo->bpp - 8*k1;
									p1 = 0 ;
									p2 = 0 ;
									if(ptr)
									{
										p1 = ptr[k1]; // this byte contains the pixel
										p1 <<= k2; // move the pixels to the top of the byte
									}
									if(nptr)
									{
										p2 = nptr[k1]; // this byte contains the pixel
										p2 <<= k2; // move the pixels to the top of the byte
									}
									if ((pScaninfo->linerate==4)&&(pScaninfo->bpp==1))
									{
										double sum = 0 ;
										if (ptr || nptr)
										{
											if (ptr) sum += (double)((double)f1*(double)MaskProperly(p1,mask)) ;
											if (nptr) sum += (double)((double)f2*(double)MaskProperly(p2,mask)) ;
											pBMPWorkingBuffer[pixel/8] <<= 1 ; // Shift left one bit
											pBMPWorkingBuffer[pixel/8] &= 254 ; // Clear the lsb
											sum = sum / F_SCALE ;
											if(sum >= 0x7f) 
												pBMPWorkingBuffer[pixel/8] |= 1 ;
										}
									}
									else
									{
										if (ptr) pBMPWorkingBuffer[pixel] = (BYTE) (f1*(int)(MaskProperly(p1, mask))/INT_SCALE);
										if (nptr) pBMPWorkingBuffer[pixel] += (BYTE) (f2*(int)(MaskProperly(p2, mask))/INT_SCALE);
									}
								} // for each pixel
								break;
							case 8: // 8 bpp
								for(pixel=color ; pixel < end ; pixel+=numberOfColors )
								{
									if ((pixel/numberOfColors)&1)	
									{				//Make this a function!!
										ptr = ptr_odd;
										nptr = nptr_odd;
										f1 = f1_odd;
										f2 = f2_odd;
									}
									else
									{
										ptr = ptr_even;
										nptr = nptr_even;
										f1 = f1_even;
										f2 = f2_even;
									}
									if (ptr) pBMPWorkingBuffer[pixel] = (BYTE) (f1*ptr[pixel]/INT_SCALE) ;
									if (nptr) pBMPWorkingBuffer[pixel] += (BYTE) (f2*nptr[pixel]/INT_SCALE);
								}
								break;
							case 16: // 16 bbp
								for(pixel=color ; pixel < end ; pixel+=numberOfColors )
								{
									if ((pixel/numberOfColors)&1)	
									{				//Make this a function!!
										ptr = ptr_odd;
										nptr = nptr_odd;
										f1 = f1_odd;
										f2 = f2_odd;
									}
									else
									{
										ptr = ptr_even;
										nptr = nptr_even;
										f1 = f1_even;
										f2 = f2_even;
									}
									int index = pixel*2;
									USHORT value = 0;
									if (ptr)
									{
											value += (f1*(ptr[index]*256+ptr[index+1])/INT_SCALE) ;
									}
									if (nptr)
									{
											value += (f2*(nptr[index]*256+nptr[index+1])/INT_SCALE);
									}
									pBMPWorkingBuffer[index] =   (BYTE) (value) ;	
									pBMPWorkingBuffer[index+1] = (BYTE) (value/256) ;
#if 0
									if (pScaninfo->linerate == 4)
										//workaround for Photoshop 4.0
										pBMPWorkingBuffer[index] = pBMPWorkingBuffer[index+1];
#endif
								}
								break;
						} // switch bpp
					} // if linerate
				} // If x NOT to be interpolated
				else // 
				{ // The following is where x IS to be interpolated
					if (pScaninfo->linerate && pScaninfo->linerate!=4)
					{
						goto error_exit ;// Do lineartion next version
					} // if linerate // The above code has never been tested
					else 
					{ // The following is not linerated and does interpolation
						switch (pScaninfo->bpp)
						{
							default: // 1/2/4 bpp
								for (pixel = 0; pixel < pScaninfo->dibwidth; pixel++)
								{
									if ((pixel/numberOfColors)&1)	
									{				//Make this a function!!
										ptr = ptr_odd;
										nptr = nptr_odd;
										f1 = f1_odd;
										f2 = f2_odd;
									}
									else
									{
										ptr = ptr_even;
										nptr = nptr_even;
										f1 = f1_even;
										f2 = f2_even;
									}
									x_new = (double) pixel * xstepsize ;
									x_base = (int) x_new ;
									x_next = x_base+1 ;
									if(x_next >= pScaninfo->width) x_next = pScaninfo->width-1 ; // Don't let x_next go passed the end of the buffer
									//dx1 = x_new - x_base ;
									//dx2 = 1 - dx1 ;
									dx1 = (int)(F_SCALE * (x_new - (double)x_base)) ; // First Fractional portion
									dx2 = (int)(F_SCALE - dx1) ; // Second Fractional portion
									k1 = (numberOfColors*x_base+color)*pScaninfo->bpp/8;
									k2 = (numberOfColors*x_base+color)*pScaninfo->bpp - 8*k1;
									k3 = (numberOfColors*x_next+color)*pScaninfo->bpp/8;
									k4 = (numberOfColors*x_next+color)*pScaninfo->bpp - 8*k3;
									p1 = 0 ;
									p11 = 0 ;
									p2 = 0 ;
									p21 = 0 ;
									if(ptr)
									{
										p1 = ptr[k1]; // this byte contains the pixel
										p1 <<= k2; // move the pixels to the top of the byte
										p11 = ptr[k3]; // this byte contains the pixel
										p11 <<= k4; // move the pixels to the top of the byte
									}
									if(nptr)
									{
										p2 = nptr[k1]; // this byte contains the pixel
										p2 <<= k2; // move the pixels to the top of the byte
										p21 = nptr[k3]; // this byte contains the pixel
										p21 <<= k4; // move the pixels to the top of the byte
									}
									if ((pScaninfo->linerate==4)&&(pScaninfo->bpp==1))
									{
										double sum = 0 ;
										if (ptr || nptr)
										{
											if (ptr)
											{
												sum += (double)((double)f1*((double)dx2*(double)MaskProperly(p1,mask) + (double)dx1*(double)MaskProperly(p11,mask)));
											}
											if (nptr)
											{
												sum += (double)((double)f2*((double)dx2*(double)MaskProperly(p2,mask) + (double)dx1*(double)MaskProperly(p21,mask))) ;
											}
											pBMPWorkingBuffer[pixel/8] <<= 1 ; // Shift left one bit
											pBMPWorkingBuffer[pixel/8] &= 254 ; // Clear the lsb
											sum = sum / F_SCALE  / F_SCALE ;
											if(sum >= 0x7f) 
												pBMPWorkingBuffer[pixel/8] |= 1 ; ;
										}
									}
									else
									{
										if (ptr)
											pBMPWorkingBuffer[numberOfColors*pixel+color] = (BYTE)  ((f1*((dx2*MaskProperly(p1,mask) + dx1*MaskProperly(p11,mask))/INT_SCALE))/INT_SCALE);
										if (nptr)
											pBMPWorkingBuffer[numberOfColors*pixel+color] += (BYTE) ((f2*((dx2*MaskProperly(p2,mask) + dx1*MaskProperly(p21,mask))/INT_SCALE))/INT_SCALE);
									}
								} // for each pixel
								// The pixels may not have been shifted 8 bits in the last byte in 1 bit gray mode
								if ((pScaninfo->linerate==4)&&(pScaninfo->bpp==1))
								{
									pixel-- ; // Went past pScaninfo->dibwidth in loop so decrement pixel
									pBMPWorkingBuffer[pixel/8] <<= (7-pixel%8) ; // Complete the 8 bit shift
								}
							break;
							case 8: // 8 bpp
								for(pixel=0 ; pixel < pScaninfo->dibwidth ; pixel++ )
								{
									if ((pixel/numberOfColors)&1)	
									{				//Make this a function!!
										ptr = ptr_odd;
										nptr = nptr_odd;
										f1 = f1_odd;
										f2 = f2_odd;
									}
									else
									{
										ptr = ptr_even;
										nptr = nptr_even;
										f1 = f1_even;
										f2 = f2_even;
									}
									x_new = (double) pixel * xstepsize ;
									x_base = (int) x_new ;
									x_next = x_base+1 ;
									if(x_next >= pScaninfo->width) x_next = pScaninfo->width-1 ; // Don't let x_next go passed the end of the buffer
									dx1 = (int)(F_SCALE * (x_new - (double)x_base)) ; // First Fractional portion
									dx2 = (int)(F_SCALE - dx1) ; // Second Fractional portion
									if(ptr)
									pBMPWorkingBuffer[numberOfColors*pixel+color] = (BYTE) ((f1*((dx2*ptr[numberOfColors*x_base+color] + dx1*ptr[numberOfColors*x_next+color])/INT_SCALE))/INT_SCALE);
									if(nptr)
									pBMPWorkingBuffer[numberOfColors*pixel+color] += (BYTE)((f2*((dx2*nptr[numberOfColors*x_base+color] + dx1*nptr[numberOfColors*(x_next)+color])/INT_SCALE))/INT_SCALE);
								}// for each pixel
							break;
							case 16: // 16 bbp
								for(pixel=0 ; pixel < pScaninfo->dibwidth ; pixel++ )
								{
									if ((pixel/numberOfColors)&1)	
									{				//Make this a function!!
										ptr = ptr_odd;
										nptr = nptr_odd;
										f1 = f1_odd;
										f2 = f2_odd;
									}
									else
									{
										ptr = ptr_even;
										nptr = nptr_even;
										f1 = f1_even;
										f2 = f2_even;
									}
									x_new = (double) pixel * xstepsize ;
									x_base = (int) x_new ;
									x_next = x_base+1 ;
									if(x_next >= pScaninfo->width) x_next = pScaninfo->width-1 ; // Don't let x_next go passed the end of the buffer
									dx1 = (int)(F_SCALE * (x_new - (double)x_base)) ; // First Fractional portion
									dx2 = (int)(F_SCALE - dx1) ; // Second Fractional portion
									USHORT value = 0;
									int index_base = 2*(numberOfColors*x_base+color);
									int index_next = 2*(numberOfColors*x_next+color);	
									if(ptr)
									{
										value += ((f1*((dx2*(ptr[index_base]*256+ptr[index_base+1]) + 
													dx1*(ptr[index_next]*256+ptr[index_next+1]))
														/INT_SCALE))/INT_SCALE);
									}
									if(nptr)
									{
										value += ((f2*((dx2*(nptr[index_base]*256+nptr[index_base+1]) + 
													dx1*(nptr[index_next]*256+nptr[index_next+1]))
														/INT_SCALE))/INT_SCALE);
									}
									int index = 2*(numberOfColors*pixel+color);
									pBMPWorkingBuffer[index] =   (BYTE) (value) ;	
									pBMPWorkingBuffer[index+1] = (BYTE) (value/256) ;
#if 0
									if (pScaninfo->linerate == 4)
										//workaround for Photoshop 4.0
										pBMPWorkingBuffer[index] = pBMPWorkingBuffer[index+1];
#endif
								}// for each pixel
							break;
						} // switch bpp
					} //if linerate else
				} // The previous is where x IS to be interpolated
			} // for each color
			Offset[0] += ystepsize;
			Offset[1] += ystepsize;
			Offset[2] += ystepsize;
			copyline(pScaninfo->pBmplinebuffer,pBMPWorkingBuffer,pScaninfo,pInfohdr,pScaninfo->nmbrBMPLinesReturnedThisCall) ;
			++pScaninfo->nmbrBMPLinesReturnedThisCall ;
			++pScaninfo->bmpreturnlinecount;
			pScaninfo->returnbytecount += pScaninfo->bmpbytesperline ;
			if(pScaninfo->bIsTwain)
				pScaninfo->fileOffset = pScaninfo->headerOffset + (pScaninfo->bmpreturnlinecount-pScaninfo->nmbrBMPLinesReturnedThisCall)*pScaninfo->bmpbytesperline ;
			else
				pScaninfo->fileOffset = pScaninfo->headerOffset + (pScaninfo->bmpoutlines - pScaninfo->bmpreturnlinecount)*pScaninfo->bmpbytesperline ;

			if(pScaninfo->bmpreturnlinecount == pScaninfo->bmpoutlines)
			{
				if(pBMPWorkingBuffer)
				{
					delete pBMPWorkingBuffer ;
					pBMPWorkingBuffer = NULL ;
				}
				if(pFilehdr)
				{
					delete pFilehdr;
					pFilehdr = NULL ;
				}
				if(pInfohdr)
				{
					delete pInfohdr;
					pInfohdr = NULL ;
				}
				if(ppscanlinepointerarray)
				{
					delete ppscanlinepointerarray;
					ppscanlinepointerarray = NULL ;
				}
				if(pScanlines)
				{
					delete pScanlines;
					pScanlines = NULL ;
				}
				return TRUE ;
			}
		}
	}
	return TRUE ; // Line done
}
else
{
	goto error_exit ;
}
return TRUE ;
// Using goto error_exit seems to be the simplest way to do an error exit
// and get all the memory de-allocated
error_exit:
if(pBMPWorkingBuffer)
{
	delete pBMPWorkingBuffer ;
	pBMPWorkingBuffer = NULL ;
}
if(pFilehdr)
{
	delete pFilehdr;
	pFilehdr = NULL ;
}
if(pInfohdr)
{
	delete pInfohdr;
	pInfohdr = NULL ;
}
if(ppscanlinepointerarray)
{
	delete ppscanlinepointerarray;
	ppscanlinepointerarray = NULL ;
}
if(pScanlines)
{
	delete pScanlines;
	pScanlines = NULL ;
}
return FALSE ;
}

