// Scan_Scanner.cpp: implementation of the CScanner class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Scan_Scanner.h"
#include <algorithm>
#include "scan_scan.h"
#include "scan_twndebug.h"

CScan Scan;
CScanner*     CScanner::s_theInstance = NULL;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CScanner::CScanner()
{
    TRACE("In CScanner()\n");

    m_scanning      = false;

    m_nRowsCopied   = 0;
    m_pHeader       = NULL;
    m_pImageData    = NULL;
    m_pCurrentLine  = NULL;
    m_dwLineSize    = 0l;
//  m_hDIB          = NULL;

    InitializeCapabilities();

}

CScanner::~CScanner()
{
    TRACE("In ~CScanner()\n");
    s_theInstance = NULL;
}


CScanner* CScanner::GetInstance()
{
    if (!s_theInstance)
    {
        s_theInstance = new CScanner();
    }

    return s_theInstance;
}

//repeat scan test mode
//static int scanloopct = 0;

SCAN_RESULT CScanner::ScanStart()
{
    TRACE("In ScanStart()\n");

	SCAN_RESULT result = SCAN_OK;

    if (!m_scanning)
    {
		if (!Scan.Startup() || !Scan.SetupScan())
		{
			result = SCAN_ERROR;
		}
		else 
		{

			// Set the current scanning state
			m_scanning = true;
		}
    }

	if (result != SCAN_OK)
	{
		m_scanning = false;
		Scan.Cleanup();
	}

    return result;    
}
    
SCAN_RESULT CScanner::ScanRead(  void*	scanBuffer,
                                    long	reqCount,
                                    long*	actCount)
{
    TRACE("In ScanRead()\n");

    // See if we are within a scan 
    if (!m_scanning)
    {
        TRACE("Error in In ScanRead outside scan\n");
        return SCAN_ERROR;
    }

	Scan.ScanReadParms.scanReadBuffer = (BYTE *) scanBuffer;
	Scan.ScanReadParms.scanReadReqCount = reqCount;
	Scan.ScanReadParms.scanReadActCount = *actCount = 0;

	BOOL done = FALSE;
	SCAN_RESULT result = SCAN_OK;
	if (!Scan.ScanSome(&done,FALSE)) result = SCAN_ERROR;

	*actCount = Scan.ScanReadParms.scanReadActCount;

	if (result != SCAN_OK)
	{
		m_scanning = false;
		Scan.Cleanup();
	}
	else

	// See if we are done transferring the image data already
	// If we are finished transferrring all data transition 
	// back to the ready state
	if (done)
	{
		TRACE("Scan Done \n");
		            
		m_scanning = false;
		Scan.Cleanup();

		result = SCAN_DONE;
	}

    TRACE("Normal Exit in ScanRead\n");
    return result;   
}

SCAN_RESULT CScanner::ScanStop()
{
    TRACE("In ScanStop()\n");

    // Reset out scan info

 	BOOL done;
	Scan.ScanSome( &done, TRUE ); //abort
	m_scanning = false;
	Scan.Cleanup();

    return SCAN_OK;
}

SCAN_RESULT CScanner::ScanSetParams(SCAN_PARAM_REC* paramRec,
									   FN_SCAN_CALLBACK scanCB)
{
    TRACE("In ScanSetParams()\n");

	Scan.SaveCallback(scanCB);

    // We will not entertain change requets in the middle of a scan
    if (m_scanning)
        return SCAN_ERROR;

    // Make sure that all the parameters requested are ones that 
	// we support - the ones we are interested in are :
	// bitsPerPixel, hRes, vRes, scanRect, boundingRect
    PIXELTYPE_ITERATOR it;
    it = find(m_capPixelTypes.begin(), 
                m_capPixelTypes.end(), 
                    paramRec->pixelType);

    if (it == m_capPixelTypes.end())
        return SCAN_ERROR;

    CAP_ITERATOR itBitDepth;
    itBitDepth  = find((*it).pixelDepths.begin(), 
                            (*it).pixelDepths.end(),
                                paramRec->bitsPerPixel);

    if (itBitDepth == (*it).pixelDepths.end())
        return SCAN_ERROR;

	// Now change our internal scan param settings
	Scan.m_currentScanSettings = *paramRec;
	
    // calculate and update scan size
	if (!Scan.Startup()) return SCAN_ERROR;
	if (!Scan.SetupScan(FALSE)) return SCAN_ERROR; //calculate new values

	Scan.Cleanup();

	return SCAN_OK;
}

SCAN_RESULT CScanner::ScanGetParams(SCAN_PARAM_REC* paramRec)
{
    TRACE("In ScanGetParams()\n");

    *paramRec = Scan.m_currentScanSettings;

    return SCAN_OK;
}

SCAN_RESULT CScanner::ScanGetScanInfo(SCAN_INFO_REC* infoRec)
{
    TRACE("In ScanGetScanInfo()\n");

//	SCAN_RESULT result = ScanStart(NULL);

	Scan.UpdateScanStatus();

    *infoRec = Scan.m_currentScanInfo;

    return SCAN_OK;
}

SCAN_RESULT CScanner::ScanGetScanHardware(SCAN_HARDWARE_REC* hardwareRec)
{
    TRACE("In ScanGetScanHardware()\n");

	memset(hardwareRec,0,sizeof(hardwareRec));

	if (!Scan.Startup()) return SCAN_ERROR;	

    *hardwareRec = Scan.m_currentScanHardware;
    
    return SCAN_OK;
}

SCAN_RESULT CScanner::ScanGetCapability(SCAN_CAPID capId, 
								 void*   input, 
								 void*   output,
                                 long*   numOutputElements)
{
    TRACE("In ScanGetCapability()\n");

    // The "right" way to do this is to use a function
    // dispatch table but since we have only a few
    // caps and this is a simulation we can get away 
    // with a simple switch statement which is quick and dirty
    SCAN_RESULT result = SCAN_OK;
    *numOutputElements = 0;

    switch (capId)
    {
        case SCAN_CAP_HRES:
            // output is an array of integers so typecast the 
            // output pointer to the correct type
            CopyCapVectorToArray(m_capHResolutions,
                                (int*)output, 
                                numOutputElements);
            break;
        case SCAN_CAP_VRES:
            // output is an array of integers so typecast the 
            // output pointer to the correct type
            CopyCapVectorToArray(m_capVResolutions,
                                (int*)output, 
                                numOutputElements);
            break;

        case SCAN_CAP_PIXELTYPE:
            CopyPixelTypesToArray((int*)output, numOutputElements);
            break;

        case SCAN_CAP_PIXELDEPTH:
            result = CopyPixelDepthsToArray(*((int*)input), 
											(int*)output, 
											numOutputElements);
            break;
        case SCAN_MAX_SCAN_WIDTH:
			{
				SCAN_HARDWARE_REC hRec;
				SCAN_PARAM_REC pRec;
				result = ScanGetScanHardware(&hRec);
				if (result == SCAN_OK) result = ScanGetParams(&pRec);
				if (result != SCAN_OK) return result;
				*((int*)output)      = (int)(hRec.widthOpticalPixels*pRec.hRes/
											hRec.opticalRes);
				*numOutputElements  = 1;
			}
            break;
        case SCAN_MAX_SCAN_HEIGHT:
			{
				SCAN_HARDWARE_REC hRec;
				SCAN_PARAM_REC pRec;
				result = ScanGetScanHardware(&hRec);
				if (result == SCAN_OK) result = ScanGetParams(&pRec);
				if (result != SCAN_OK) return result;
				*((int*)output)      = (int)(hRec.heightOpticalPixels*pRec.vRes/
											hRec.opticalRes);
				*numOutputElements  = 1;
			}
            break;
        default :
            result = SCAN_ERROR;    
            break;
    }

    return result; 
}


/*******************************************************************************
DESCRIPTION:

This is a helper function that takes a STL container of type CAP_CONTAINER 
and copies it into a  plain C array of ints.

PARAMETERS:

[in]
container   - The STL vector that is the source for the copy operation
outArray    - The int array that is the target of the copy operation

[out]
numOutputElements - The number of elements copied successfully

RETURN VALUE:
	- 

NOTES:
	The input array is expected to be atleast as long as SCAN_CAPID_MAXNUM.
    If the container has more than SCAN_CAPID_MAXNUM elements (which is highly 
    unlikely) then only the first SCAN_CAPID_MAXNUM elements will be copied
    
    It is assumed that the input container has ints stored in it or atleast
    some data type that has a conversion to int defined.
*******************************************************************************/
void CScanner::CopyCapVectorToArray(CAP_CONTAINER& container,
                                      int *outArray,
                                      long *numOutputElements)
{
    CAP_ITERATOR    it;    // input iterator for the capability vector
    int             i = 0; // Loop control variable

    for (it = container.begin(), i = 0; 
            (i < SCAN_CAPID_MAXNUM) && (it != container.end());
                i++, it ++)
    {
        outArray[i] = *it;
    }
    
    *numOutputElements = i;
}

void CScanner::CopyPixelTypesToArray(int* outArray, long *numOutputElements)
{
    PIXELTYPE_ITERATOR    it;    // input iterator for the capability vector
    int                 i = 0;   // Loop control variable

    for (it = m_capPixelTypes.begin(), i = 0; 
            (i < SCAN_CAPID_MAXNUM) && (it != m_capPixelTypes.end());
                i++, it ++)
    {
        outArray[i] = (*it).pixelType;
    }
    
    *numOutputElements = i;
}

SCAN_RESULT CScanner::CopyPixelDepthsToArray(int pixelType, 
                                                int* outArray, 
                                                long* numOutputElements)
{
	// We assume that we might not find the given pixel type in 
	// our supported pixel types array
    SCAN_RESULT result = SCAN_ERROR;

    // find the input pixel type and copy its contents over into our 
	// int array - this is a very small array so we can do a simple
	// sequential search 
    PIXELTYPE_ITERATOR    it;    // input iterator for the capability vector

    for (it = m_capPixelTypes.begin(); it != m_capPixelTypes.end(); it ++)
    {
        if (pixelType == (*it).pixelType)
		{
			CopyCapVectorToArray(	(*it).pixelDepths, 
									outArray, 
									numOutputElements);
			result = SCAN_OK;
		}
    }
    
    return result;
}

void CScanner::InitScanner(HANDLE hModule)
{
    TRACE("In InitScanner()\n");
    
    m_hModule = (HINSTANCE)hModule;
    
    InitializeScanParams();

    return;
}

bool CScanner::InitializeScanParams()
{
    TRACE("In InitializeScanParams()\n");

    memset(&Scan.m_currentScanSettings, 0, sizeof(SCAN_PARAM_REC));

	//init scan value to some valid values
	Scan.m_currentScanSettings.scanToTwain = FALSE;	
	Scan.m_currentScanSettings.boundingRect.left = 
		Scan.m_currentScanSettings.boundingRect.top = 0;
	Scan.m_currentScanSettings.boundingRect.bottom = 
		Scan.m_currentScanSettings.boundingRect.right = 100;
	Scan.m_currentScanSettings.scanRect = Scan.m_currentScanSettings.boundingRect;		
	Scan.m_currentScanSettings.pixelType = SCAN_PIXELTYPE_RGB;		
	Scan.m_currentScanSettings.bitsPerPixel = 24;	
	Scan.m_currentScanSettings.hRes = 100;			
	Scan.m_currentScanSettings.vRes = 100;			
	Scan.m_currentScanSettings.hInterp = TRUE;		
	Scan.m_currentScanSettings.vInterp = TRUE;	

    return true;
}

bool CScanner::InitializeCapabilities()
{
    TRACE("In InitializeCapabilities()\n");

	SCAN_HARDWARE_REC hRec;
	ScanGetScanHardware(&hRec);
	for (int i=10; i <= 9600; i++)
	{
		int res = i;
		// These are the supported horizontal resolutions
		m_capHResolutions.push_back(res);
		// .. and supporrted vertical resolutions
		m_capVResolutions.push_back(res);
	}

    /// We will then intialize the list of supported pixel 
    // types and the corrsponding bit depths  for each pixel type
    
    // For each pixel type store the supported bit depths
    CPixelType tempPixelType;
    
    // Black & White
    tempPixelType.pixelType = SCAN_PIXELTYPE_BW;
    tempPixelType.pixelDepths.clear();
    tempPixelType.pixelDepths.push_back(1);

    m_capPixelTypes.push_back(tempPixelType);

    // Gray Scale
    tempPixelType.pixelType = SCAN_PIXELTYPE_GRAY;
    tempPixelType.pixelDepths.clear();
    tempPixelType.pixelDepths.push_back(2);
    tempPixelType.pixelDepths.push_back(4);
    tempPixelType.pixelDepths.push_back(8);
	if (hRec.chip == 1)
		tempPixelType.pixelDepths.push_back(14);
	else if (hRec.chip == 2)
		tempPixelType.pixelDepths.push_back(16);

    m_capPixelTypes.push_back(tempPixelType);

    // RGB
    tempPixelType.pixelType = SCAN_PIXELTYPE_RGB;
    tempPixelType.pixelDepths.clear();
    tempPixelType.pixelDepths.push_back(3);
    tempPixelType.pixelDepths.push_back(6);
    tempPixelType.pixelDepths.push_back(12);
    tempPixelType.pixelDepths.push_back(24);
	if (hRec.chip == 1)
		tempPixelType.pixelDepths.push_back(42);
	else if (hRec.chip == 2)
		tempPixelType.pixelDepths.push_back(48);
    m_capPixelTypes.push_back(tempPixelType);

    return true;
}

