/*******************************************************************************
Defintions of STL containers used in this module - STL containers end up
having really long  names for the types and this can cause the compiler
to choke on them so it is better to define new types for the template
refinements of the STL containers.
*******************************************************************************/
typedef vector<int>             CAP_CONTAINER;
typedef CAP_CONTAINER::iterator CAP_ITERATOR;

// This ADT is just a glorified struct that holds the supported pixel
// depths for a given pixel type
struct CPixelType
{
    int             pixelType;
    CAP_CONTAINER   pixelDepths;

    // Any type that has to be stored in a vector has to have these
    // operations defined on it - ==, < and the copy constructor 
    /// We need the default constructor if we create temp objects

    // < operator compares just the pixel type portion
    bool operator < (const CPixelType& in) 
            {return (pixelType < in.pixelType);}

    // Comparison operator
    bool operator == (const CPixelType& in)
            {return (pixelType == in.pixelType);}

    // Comparison operator for ints 
    bool operator == (const int& in)
            {return (pixelType == in);}

    // Copy constructor
    CPixelType(const CPixelType& in)
    {
        pixelType   = in.pixelType;
        pixelDepths = in.pixelDepths;
    }

    // Default constructor sets up a pixel type of black and white
    // with bit depth of 1
    CPixelType()
    {
        pixelType = 0;
        pixelDepths.push_back(1);
    }
};

// Shorthand names for the vector of CPixelTypes
typedef vector<CPixelType>              PIXELTYPE_CONTAINER;
typedef PIXELTYPE_CONTAINER::iterator   PIXELTYPE_ITERATOR;

