/*******************************************************************************
FILE        : Twain_capbase.cpp
REVISION    : 1.0
DATE        : 9/12/99

DESCRIPTION :

The CCapBase class is the abstract base class for the Capability object
heirarchy. The class provides an implementation for the HandleCapMessage
function which is the handler that is invoked by the operation triplet 
dispatch code for capability messages. Most other methods are pure virtual.

Copyright (c) 1999 - National Semiconductor Ltd - All rights reserved
*******************************************************************************/
#include "stdafx.h"
#include "Twain_capbase.h"

#include "Twain_Utility.h"

/*******************************************************************************
DESCRIPTION:

CCapBase::HandleCapMessage is the main handler function of a capability 
message object.

PARAMETERS: 

PTWMSG pTWMSG

RETURN VALUE:
TW_UINT16 - The Twain status response code

NOTES:
This method simply delegates the actual handling to the virtual functions 
which are implemented in the derived classes i.e. it is a template function.

*******************************************************************************/
TW_UINT16 CCapBase::HandleCapMessage(PTWMSG pTWMSG)
{
    // ASSERT(pTWMSG->DAT == DAT_CAPABILITY)
	TW_UINT16	dcRc = TWRC_SUCCESS;
    
    switch(pTWMSG->MSG)
    {
    case MSG_QUERYSUPPORT:
            TRACE("Twain_LM983x : Handling Capability Msg - MSG_QUERYSUPPORT \n"); 
            dcRc = HandleQuerySupportedCaps((pTW_CAPABILITY)pTWMSG->pData);
            break;
        case MSG_GET:
            TRACE("Twain_LM983x : Handling Capability Msg - MSG_GET \n");
            dcRc = HandleGetCapValues((pTW_CAPABILITY)pTWMSG->pData);
            break;
        case MSG_GETCURRENT:
            TRACE("Twain_LM983x : Handling Capability Msg - MSG_GETCURRENT \n");
            dcRc = HandleGetCurrentCap((pTW_CAPABILITY)pTWMSG->pData);
            break;
        case MSG_GETDEFAULT:
            TRACE("Twain_LM983x : Handling Capability Msg - MSG_GETDEFAULT \n");
            dcRc = HandleGetDefaultCap((pTW_CAPABILITY)pTWMSG->pData);
            break;
        case MSG_RESET:
            TRACE("Twain_LM983x : Handling Capability Msg - MSG_RESET \n");
            dcRc = HandleResetCap((pTW_CAPABILITY)pTWMSG->pData);
            break;
        case MSG_SET:
            TRACE("Twain_LM983x : Handling Capability Msg - MSG_SET \n");
            dcRc = HandleSetCap((pTW_CAPABILITY)pTWMSG->pData);
            break;
        default:
            // Unrecognized message code 
            break;
    }

    return dcRc;
}


/*******************************************************************************
DESCRIPTION:

CCapBase::~CCapBase is the class destructor 

PARAMETERS: 
none

RETURN VALUE:
none

NOTES:
    // The virtual destructor is provided just to make sure that 
    // we can safely delete base class pointers to derived class 
    // objects - it does nothing else - yet!

*******************************************************************************/
CCapBase::~CCapBase()
{
}
