/*******************************************************************************
FILE        : Twain_capbase.h
REVISION    : 1.0
DATE        : 9/12/99

DESCRIPTION :

This file contains the defintion of the CCapBase class which is
the abstract base class for all capability classes. In this data source the
capabilities are represented by objects which can handle messages to 
get and set capabilties and also act as the store for the supported
cap values and current and default settings. 


Copyright (c) 1999 - National Semiconductor Ltd - All rights reserved
*******************************************************************************/
#ifndef _TWAIN_CAPBASE_H__
#define _TWAIN_CAPBASE_H__

#include "Twain_Types.h"
#include <string>

class CCapBase
{
public:
    // This is a template function that simply delegates the actual work 
    // to one of the Handle_*  virtualfunctions 
    TW_UINT16 HandleCapMessage(PTWMSG);
    
    virtual TW_UINT16 GetCapID()=0;
    virtual string    GetCapName()=0;

    virtual ~CCapBase();

protected:
    // These fns will be overridden  in the derived classes to provide
    // for cap specific funtionality
    virtual TW_UINT16 HandleQuerySupportedCaps(pTW_CAPABILITY)=0;
    virtual TW_UINT16 HandleGetCapValues(pTW_CAPABILITY)=0;
    virtual TW_UINT16 HandleGetCurrentCap(pTW_CAPABILITY)=0;
    virtual TW_UINT16 HandleGetDefaultCap(pTW_CAPABILITY)=0;
    virtual TW_UINT16 HandleResetCap(pTW_CAPABILITY)=0;
    virtual TW_UINT16 HandleSetCap (pTW_CAPABILITY)=0;
    
    // Cap specific attributes that will be used in derived class
    // implementations
    virtual TW_INT32  GetSupportedCaps()=0;
    virtual TW_INT32  GetItemType()=0;
    
};  


#endif // _TWAIN_CAPBASE_H__