/*******************************************************************************
FILE        : Twain_CapHelpers.h
REVISION    : 1.0
DATE        : 10/27/99

DESCRIPTION :

Getting and setting the capabilities using the cpability objects
involves a small amount of bolier plate code that can somewhat obscure 
the flow of a function. These helpers are meant to be used in the 
non-capability implementation portions of the TWAIN module for 
quick and easy access to the capability engine.

Copyright (c) 1999 - National Semiconductor Ltd - All rights reserved
*******************************************************************************/
#ifndef _TWAIN_CAP_HELPERS_H__
#define _TWAIN_CAP_HELPERS_H__

// Function wrappers for simplified access to the capability objects
// and current scan parameters
TWAIN_RESULT GetCurrentXResolution(float& xResolution);
TWAIN_RESULT GetCurrentYResolution(float& yResolution);
TWAIN_RESULT GetCurrentPixelType(TW_UINT16& currentPixelType);
TWAIN_RESULT GetCurrentBitDepth(TW_UINT16&  bitDepth);

TWAIN_RESULT GetCurrentUnits(TW_UINT16& currentUnits);
TWAIN_RESULT GetCurrentXferMech(TW_UINT16& xferMech);

TWAIN_RESULT SetCurrentBitDepth(TW_UINT16  bitDepth);
TWAIN_RESULT SetCurrentPixelType(TW_UINT16  pixelType);
TWAIN_RESULT SetCurrentXResolution(float xResolution);
TWAIN_RESULT SetCurrentYResolution(float yResolution);

TWAIN_RESULT GetScanWidthInPixels(int& scanWidth);
TWAIN_RESULT GetScanHeightInPixels(int& scanHeight);

TWAIN_RESULT GetCurrentScanWidth(float& scanWidth);
TWAIN_RESULT GetCurrentScanHeight(float& scanHeight);

TWAIN_RESULT GetMaxScanWidth(float& scanWidth);
TWAIN_RESULT GetMaxScanHeight(float& scanHeight);

TWAIN_RESULT CalculateWidthInCurrentUnits(int pixelWidth, float& scanWidth);
TWAIN_RESULT CalculateHeightInCurrentUnits(int pixelHeight, float& scanHeight);
TWAIN_RESULT CalculateWidthInPixels(float scanWidth, int& pixelWidth);
TWAIN_RESULT CalculateHeightInPixels(float scanHeight,  int& pixelHeight);

// Type conversions used by the templatized container management routines
bool operator ==(TW_FIX32 first, TW_FIX32 second);

#endif // _TWAIN_CAP_HELPERS_H__
