/*******************************************************************************
FILE        : Twain_ImageCaps.h
REVISION    : 1.0
DATE        : 9/12/99

DESCRIPTION :

This file contains the definition of all the image related capability handling
classes. 

Copyright (c) 1999 - National Semiconductor Ltd - All rights reserved
*******************************************************************************/
#include "Twain_CapImplementation.h"

class CICapCompression : public CCapImplementation<TW_UINT16>
{
public:
    CICapCompression();

    TW_UINT16 GetCapID(){return ICAP_COMPRESSION;} 
    string    GetCapName(){return string("ICAP_COMPRESSION");}

protected:
    TW_INT32   GetSupportedCaps(){return (  TWQC_GET | 
                                            TWQC_GETDEFAULT | 
                                            TWQC_GETCURRENT ) ; }

    TW_INT32  GetItemType(){return TWTY_UINT16;}

    // Overridden handlers for the supported Cap operations 
    TW_UINT16 HandleGetCapValues(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetCurrentCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetDefaultCap(pTW_CAPABILITY pCapability);
};

class CICapPlanarChunky : public CCapImplementation<TW_UINT16>
{
public:
    CICapPlanarChunky();
    TW_UINT16 GetCapID(){return ICAP_PLANARCHUNKY;}
    string    GetCapName(){return string("ICAP_PLANARCHUNKY");}

protected:
    TW_INT32   GetSupportedCaps(){return (  TWQC_GET | 
                                            TWQC_GETDEFAULT | 
                                            TWQC_GETCURRENT ) ; }

    TW_INT32  GetItemType(){return TWTY_UINT16;}

    // Overridden handlers for the supported Cap operations 
    TW_UINT16 HandleGetCapValues(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetCurrentCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetDefaultCap(pTW_CAPABILITY pCapability);
};

class CICapPhysicalWidth : public CCapImplementation<TW_FIX32>
{
public:
    CICapPhysicalWidth();
    TW_UINT16 GetCapID(){return ICAP_PHYSICALWIDTH;}
    string    GetCapName(){return string("ICAP_PHYSICALWIDTH");}

    TWAIN_RESULT UpdateCapList();
    TWAIN_RESULT UpdateIndices();

protected:
    TW_INT32   GetSupportedCaps(){return (  TWQC_GET | 
                                            TWQC_GETDEFAULT | 
                                            TWQC_GETCURRENT ) ; }

    TW_INT32  GetItemType(){return TWTY_FIX32;}

    // Overridden handlers for the supported Cap operations 
    TW_UINT16 HandleGetCapValues(pTW_CAPABILITY  pCapability);
    TW_UINT16 HandleGetCurrentCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetDefaultCap(pTW_CAPABILITY pCapability);
};

class CICapPhysicalHeight : public CCapImplementation<TW_FIX32>
{
public:
    CICapPhysicalHeight();
    TW_UINT16 GetCapID(){return ICAP_PHYSICALHEIGHT;}
    string    GetCapName(){return string("ICAP_PHYSICALHEIGHT");}

    TWAIN_RESULT UpdateCapList();
    TWAIN_RESULT UpdateIndices();

protected:
    TW_INT32   GetSupportedCaps(){return (  TWQC_GET | 
                                            TWQC_GETDEFAULT | 
                                            TWQC_GETCURRENT ) ; }

    TW_INT32  GetItemType(){return TWTY_FIX32;}

    // Overridden handlers for the supported Cap operations 
    TW_UINT16 HandleGetCapValues(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetCurrentCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetDefaultCap(pTW_CAPABILITY pCapability);
};


class CICapPixelFlavor : public CCapImplementation<TW_UINT16>
{
public:
    CICapPixelFlavor();
    TW_UINT16 GetCapID(){return ICAP_PIXELFLAVOR;}
    string    GetCapName(){return string("ICAP_PIXELFLAVOR");}

protected:
    TW_INT32   GetSupportedCaps(){return (  TWQC_GET | 
                                            TWQC_GETDEFAULT | 
                                            TWQC_GETCURRENT ) ; }

    TW_INT32  GetItemType(){return TWTY_UINT16;}

    // Overridden handlers for the supported Cap operations 
    TW_UINT16 HandleGetCapValues(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetCurrentCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetDefaultCap(pTW_CAPABILITY pCapability);
};


class CICapBitDepth : public CCapImplementation<TW_UINT16>
{
public:
    CICapBitDepth();
    TW_UINT16 GetCapID(){return ICAP_BITDEPTH;} 
    string    GetCapName(){return string("ICAP_BITDEPTH");}
    
    TWAIN_RESULT UpdateCapList();
    TWAIN_RESULT UpdateIndices();

    // Helper methods 
    TWAIN_RESULT SetCurrentBitDepth(TW_UINT16  bitDepth);
protected:
    TW_INT32   GetSupportedCaps(){return (  TWQC_GET | 
                                            TWQC_SET | 
                                            TWQC_GETDEFAULT | 
                                            TWQC_GETCURRENT | 
                                            TWQC_RESET) ; }

    TW_INT32  GetItemType(){return TWTY_UINT16;}

    // Overridden handlers for the supported Cap operations 
    TW_UINT16 HandleGetCapValues(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetCurrentCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetDefaultCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleResetCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleSetCap (pTW_CAPABILITY pCapability);
};

class CICapBitOrder : public CCapImplementation<TW_UINT16>
{
public:
    CICapBitOrder();
    TW_UINT16 GetCapID(){return ICAP_BITORDER;} 
    string    GetCapName(){return string("ICAP_BITORDER");}

protected:
    TW_INT32   GetSupportedCaps(){return (  TWQC_GET | 
                                            TWQC_SET | 
                                            TWQC_GETDEFAULT | 
                                            TWQC_GETCURRENT | 
                                            TWQC_RESET) ; }

    TW_INT32  GetItemType(){return TWTY_UINT16;}

    // Overridden handlers for the supported Cap operations 
    TW_UINT16 HandleGetCapValues(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetCurrentCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetDefaultCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleResetCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleSetCap (pTW_CAPABILITY pCapability);
};

class CICapPixelType : public CCapImplementation<TW_UINT16>
{
public:
    CICapPixelType();
    TW_UINT16 GetCapID(){return ICAP_PIXELTYPE;} 
    string    GetCapName(){return string("ICAP_PIXELTYPE");}

    TWAIN_RESULT UpdateCapList();
    TWAIN_RESULT UpdateIndices();

    TWAIN_RESULT SetCurrentPixelType(TW_UINT16  pixelType);

protected:
    TW_INT32   GetSupportedCaps(){return (  TWQC_GET | 
                                            TWQC_SET | 
                                            TWQC_GETDEFAULT | 
                                            TWQC_GETCURRENT | 
                                            TWQC_RESET) ; }

    TW_INT32  GetItemType(){return TWTY_UINT16;}

    // Overridden handlers for the supported Cap operations 
    TW_UINT16 HandleGetCapValues(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetCurrentCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetDefaultCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleResetCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleSetCap (pTW_CAPABILITY pCapability);

};

class CICapUnits : public CCapImplementation<TW_UINT16>
{
public:
    CICapUnits();
    TW_UINT16   GetCapID(){return ICAP_UNITS;} 
    string    GetCapName(){return string("ICAP_UNITS");}
    
protected:
    TW_INT32   GetSupportedCaps(){return (  TWQC_GET | 
                                            TWQC_SET | 
                                            TWQC_GETDEFAULT | 
                                            TWQC_GETCURRENT | 
                                            TWQC_RESET) ; }

    TW_INT32  GetItemType(){return TWTY_UINT16;}

    // Overridden handlers for the supported Cap operations 
    TW_UINT16 HandleGetCapValues(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetCurrentCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetDefaultCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleResetCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleSetCap (pTW_CAPABILITY pCapability);
};

class CICapXferMech : public CCapImplementation<TW_UINT16>
{
public:
    CICapXferMech();
    TW_UINT16 GetCapID(){return ICAP_XFERMECH;} 
    string    GetCapName(){return string("ICAP_XFERMECH");}

protected:
    TW_INT32   GetSupportedCaps(){return (  TWQC_GET | 
                                            TWQC_SET | 
                                            TWQC_GETDEFAULT | 
                                            TWQC_GETCURRENT | 
                                            TWQC_RESET) ; }

    TW_INT32  GetItemType(){return TWTY_UINT16;}

    // Overridden handlers for the supported Cap operations 
    TW_UINT16 HandleGetCapValues(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetCurrentCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetDefaultCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleResetCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleSetCap (pTW_CAPABILITY pCapability);
};

class CICapXResolution : public CCapImplementation<TW_FIX32>
{
public:
    CICapXResolution();
    TW_UINT16 GetCapID(){return ICAP_XRESOLUTION;} 
    string    GetCapName(){return string("ICAP_XRESOLUTION");}

    TWAIN_RESULT    UpdateCapList();
    TWAIN_RESULT    UpdateIndices();

    // Helpers 
    TWAIN_RESULT    SetCurrentXResolution(float xResolution);

protected:
    TW_INT32   GetSupportedCaps(){return (  TWQC_GET | 
                                            TWQC_SET | 
                                            TWQC_GETDEFAULT | 
                                            TWQC_GETCURRENT | 
                                            TWQC_RESET) ; }

    TW_INT32  GetItemType(){return TWTY_FIX32;}

    // Overridden handlers for the supported Cap operations 
    TW_UINT16 HandleGetCapValues(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetCurrentCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetDefaultCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleResetCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleSetCap (pTW_CAPABILITY pCapability);

};

class CICapYResolution : public CCapImplementation<TW_FIX32>
{
public:
    CICapYResolution();
    TW_UINT16 GetCapID(){return ICAP_YRESOLUTION;} 
    string    GetCapName(){return string("ICAP_YRESOLUTION");}

    TWAIN_RESULT    UpdateCapList();
    TWAIN_RESULT    UpdateIndices();

    // Helpers 
    TWAIN_RESULT    SetCurrentYResolution(float yResolution);

protected:
    TW_INT32   GetSupportedCaps(){return (  TWQC_GET | 
                                            TWQC_SET | 
                                            TWQC_GETDEFAULT | 
                                            TWQC_GETCURRENT | 
                                            TWQC_RESET) ; }

    TW_INT32  GetItemType(){return TWTY_FIX32;}

    // Overridden handlers for the supported Cap operations 
    TW_UINT16 HandleGetCapValues(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetCurrentCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleGetDefaultCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleResetCap(pTW_CAPABILITY pCapability);
    TW_UINT16 HandleSetCap (pTW_CAPABILITY pCapability);
};

