/*******************************************************************************
FILE        : Twain_LM983x.cpp 
REVISION    : 1.0
DATE        : 9/3/99

DESCRIPTION :

Contains the implementation of functions exported from this DLL

Copyright (c) 1999 - National Semiconductor Ltd - All rights reserved
*******************************************************************************/
#include "stdafx.h"
#include "Twain_LM983x.h"

#include "Twain_ScanInterface.h"
#include "Twain_UIInterface.h"
#include "Twain_OperationTriplets.h"
#include "Twain_Utility.h"
#include "Twain_Session.h"


/*******************************************************************************
Exported Functions 
*******************************************************************************/
// This method should be called to initiate a scan and send the 
// results to the TWAIN application driving this data source
// This method is asynchronous and will return immediately the
// caller must transition to the "Scanning" mode upon successful
// return
// An error return indicates that the scan request could not be 
// completed. The error code specifies the nature of the error
// 
Twain_LM983x_EXPORT TWAIN_RESULT TwainScan()
{
    // just let the application know that we are ready to start scanning
    // Again it is upto the TWAIN application to actually start the 
    // scan this function is just the trigger and does not do anything else
    NotifyXferReady();

    return TWAIN_OK;
}



TWAIN_RESULT TwainPreview(LPCSTR pszFileName)
{
    HGLOBAL hDib;
    TWAIN_RESULT result = CScanInterface::GetInstance()->ScanToDIB(hDib);
    
    if (result == TWAIN_OK)
    {
  		if (pszFileName != NULL)
			SaveBitmap(hDib, pszFileName);	//save in file
        GlobalFree(hDib);
    }

    return result;
}

Twain_LM983x_EXPORT TWAIN_RESULT TwainSetScanParams(TWAIN_PARAM_REC* paramRec)
{
    return CScanInterface::GetInstance()->ScanSetScanParams(paramRec);
}

Twain_LM983x_EXPORT TWAIN_RESULT TwainGetScanParams(TWAIN_PARAM_REC* paramRec)
{
    return CScanInterface::GetInstance()->ScanGetScanParams(paramRec);
}


TWAIN_RESULT TwainGetScanInfo(TWAIN_INFO_REC *infoRec)
{
    return CScanInterface::GetInstance()->ScanGetScanInfo(infoRec);
}

TWAIN_RESULT TwainGetScanHardware(TWAIN_HARDWARE_REC *hardwareRec)
{
    return CScanInterface::GetInstance()->ScanGetScanHardware(hardwareRec);
}

/*******************************************************************************
DESCRIPTION:

TwainGetCapability

PARAMETERS: 

TWAIN_CAPID capId
void* input
void* output
long* numOutputElements

RETURN VALUE:
TWAIN_RESULT 

NOTES:

*******************************************************************************/
TWAIN_RESULT TwainGetCapability(TWAIN_CAPID capId,
                                void* input,
                                void* output,
                                long* numOutputElements)
{
    // Forward this query on to the Scan Module - right now we dont
    // allow the restriction of capability values i.e. we dont accept
    // capability MSG_SET messages if the container is not a TW_ONEVALUE
    // So at any point of time the capability that we expose to the 
    // TWAIN application and the UI module are essentialli the same as those
    // that are exposed by the SCNA module itself
    // We do however have to convert the CAP ID to the corresponding SCAN Cap ID
    // and that will be done for us by the Scan Interface layer.
    return CScanInterface::GetInstance()->ScanGetCapability(capId, 
                                                    input, 
                                                    output, 
                                                    numOutputElements);
    
}

Twain_LM983x_EXPORT TWAIN_RESULT TwainClose()
{
    // Ask the TWAIN application so properly shut us down using the 
    // CloseDS call.
    NotifyCloseDSReq();
    
    // Just calling this function has no effect whatsoever on the 
    // TWAIN module or on the SCAN module but this will signal the 
    // TWAIN application to send us a closeDS message and subsequently
    // we will close down the UI and the Scan Module

    return TWAIN_OK;
}

Twain_LM983x_EXPORT TWAIN_RESULT TwainAbort()
{
	//Kill scan
	if((CSession::GetInstance()->GetState()==CSession::E_STATE6) ||
       (CSession::GetInstance()->GetState()==CSession::E_STATE7))
	{
         // Since we have no more images to transfer we go 
         // to state 5
         CSession::GetInstance()->SetState(CSession::E_STATE5);
	}
    CScanInterface::GetInstance()->ScanStop();
    CUIInterface::GetInstance()->EndScan(TWAIN_ABORT);
    return TWAIN_OK;
}


// The main entry point function for this DLL called by the c runtime
// as part of the DLL load process
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
            // Initialize the capability container ?
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}
