// Twain_ScanInterface.h: interface for the CScanInterface class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TWAIN_SCANINTERFACE_H__62F9BAA4_6AB3_11D3_9859_00104BF5E7FA__INCLUDED_)
#define AFX_TWAIN_SCANINTERFACE_H__62F9BAA4_6AB3_11D3_9859_00104BF5E7FA__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Twain_Singleton.h"

// Typedefs for the entry points into the Scan dll
typedef SCAN_RESULT (*pfnScanStart)();
typedef SCAN_RESULT (*pfnScanRead)(void*,long,long*);
typedef SCAN_RESULT (*pfnScanStop)();
typedef SCAN_RESULT (*pfnScanSetScanParams)(SCAN_PARAM_REC*,FN_SCAN_CALLBACK);
typedef SCAN_RESULT (*pfnScanGetScanParams)(SCAN_PARAM_REC*);
typedef SCAN_RESULT (*pfnScanGetScanInfo)(SCAN_INFO_REC *);
typedef SCAN_RESULT (*pfnScanGetScanHardware)(SCAN_HARDWARE_REC *);
typedef SCAN_RESULT (*pfnScanGetCapability)(SCAN_CAPID, void*, void *,long*);

typedef map<TWAIN_RESULT, SCAN_RESULT>      TWAIN2SCAN_MAP;
typedef TWAIN2SCAN_MAP::iterator            TWAIN2SCAN_ITERATOR;

typedef map<SCAN_RESULT, TWAIN_RESULT>      SCAN2TWAIN_MAP;
typedef SCAN2TWAIN_MAP::iterator            SCAN2TWAIN_ITERATOR;

typedef map<TWAIN_CAPID, SCAN_CAPID>        TWAIN2SCAN_CAPMAP;
typedef TWAIN2SCAN_CAPMAP::iterator         TWAIN2SCAN_CAPITERATOR;

typedef map<SCAN_CAPID, TWAIN_CAPID>        SCAN2TWAIN_CAPMAP;
typedef SCAN2TWAIN_CAPMAP::iterator         SCAN2TWAIN_CAPITERATOR;

class CScanInterface  : public CSingleton<CScanInterface>
{
public:
    enum SCAN_FORMAT
    {
        E_DIBDATA,
        E_RGBDATA
    };

	CScanInterface();
	virtual ~CScanInterface();

    TWAIN_RESULT ScanStart();
    TWAIN_RESULT ScanRead(  void*	scanBuffer,
							long	regCount,
							long*	actCount);

    TWAIN_RESULT ScanStop();
    

    TWAIN_RESULT ScanSetScanParams(TWAIN_PARAM_REC* paramRec);
    TWAIN_RESULT ScanGetScanParams(TWAIN_PARAM_REC* paramRec);
    TWAIN_RESULT ScanGetScanInfo(TWAIN_INFO_REC *infoRec);
    TWAIN_RESULT ScanGetScanHardware(TWAIN_HARDWARE_REC *hardwareRec);
    TWAIN_RESULT ScanGetCapability(  TWAIN_CAPID capId,    
									 void*  input, 
									 void * output,
                                     long*  numOutputElements);

    TWAIN_RESULT ScanToDIB(HGLOBAL&);
    TWAIN_RESULT SetScanFormat(SCAN_FORMAT);
    
    void LoadScanModule();

    static SCAN_RESULT TwainCallback(BOOL check_messages);
    
    SCAN_RESULT HandleCallBack(BOOL check_messages);

protected:
    TWAIN_RESULT    Scan2TwainResultCode(SCAN_RESULT scanResultCode);
    SCAN_RESULT     Twain2ScanResultCode(TWAIN_RESULT twainResultCode);

    TWAIN_RESULT    Scan2TwainCapID(SCAN_CAPID scanCapID, 
                                    TWAIN_CAPID* twainCapID);
    TWAIN_RESULT    Twain2ScanCapID(TWAIN_CAPID twainCapID, 
                                    SCAN_CAPID* scanCapID);


private:
    pfnScanStart            m_fnScanStart;
    pfnScanRead             m_fnScanRead;
    pfnScanStop             m_fnScanStop;
    pfnScanSetScanParams    m_fnScanSetScanParams;
    pfnScanGetScanParams    m_fnScanGetScanParams;
    pfnScanGetScanInfo      m_fnScanGetScanInfo;
	pfnScanGetScanHardware  m_fnScanGetScanHardware;
    pfnScanGetCapability    m_fnScanGetCapability;

    HINSTANCE               m_handleScanModule;

    SCAN_RESULT             m_lastScanResult;

    void LoadCodeConversionTables();

    TWAIN2SCAN_MAP          m_twain2ScanTable;
    SCAN2TWAIN_MAP          m_scan2TwainTable;

    TWAIN2SCAN_CAPMAP       m_twain2ScanCapMap;
    SCAN2TWAIN_CAPMAP       m_scan2TwainCapMap;
};

#endif // !defined(AFX_TWAIN_SCANINTERFACE_H__62F9BAA4_6AB3_11D3_9859_00104BF5E7FA__INCLUDED_)
