/*******************************************************************************
FILE        : Twain_Session.cpp
REVISION    : 1.0
DATE        : 9/30/99

DESCRIPTION :

    implementation of the CSession class.

Copyright (c) 1999 - National Semiconductor Ltd - All rights reserved
*******************************************************************************/

#include "stdafx.h"
#include "Twain_Session.h"
#include "resource.h"
#include "TWain_Utility.h"

CSession* CSingleton<CSession>::m_theInstance=0;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSession::CSession()
{
    m_status.ConditionCode  = TWCC_SUCCESS;
    m_status.Reserved       = 0;

    memset(&m_appIdentity, 0, sizeof(TW_IDENTITY));
    memset(&m_sourceIdentity, 0, sizeof(TW_IDENTITY));

    m_hInstance = NULL;
    m_state     = E_STATE3;
}

CSession::~CSession()
{

}

void CSession::SetAppIdentity(const TW_IDENTITY& stIdentity)
{
    m_appIdentity = stIdentity;
}

void CSession::SetSourceIdentity(const TW_IDENTITY& stIdentity)
{
    m_sourceIdentity = stIdentity;
}

void CSession::SetStatusCode(TW_UINT16 conditionCode)
{
    m_status.ConditionCode = conditionCode;
}


/*******************************************************************************
DESCRIPTION:

CSession::HandleError

PARAMETERS: 

char *specificMsg
TW_UINT16 conditionCode

RETURN VALUE:
TW_UINT16 

NOTES:

*******************************************************************************/
TW_UINT16 CSession::HandleError(char *specificMsg, TW_UINT16 conditionCode)
{
    // Set the status code to the passed in condition code 
    SetStatusCode(conditionCode);

    // Add the error message corresponsing to the condition code
	char details [180];

    switch (conditionCode)
	{
		case TWCC_BADCAP:
			lstrcpy (details, "TWCC_BADCAP - Unknown capability.");
			break;

		case TWCC_BADPROTOCOL:
			lstrcpy (details, "TWCC_BADPROTOCOL - Unrecognized MSG DG DAT combination.");
			break;

		case TWCC_BADVALUE:
			lstrcpy (details, "TWCC_BADVALUE - Data parameter out of range .");
			break;

		case TWCC_BUMMER:
			lstrcpy (details, "TWCC_BUMMER - Failure due to unknown causes.");
			break;

		case TWCC_OPERATIONERROR:
			lstrcpy (details, "TWCC_OPERATIONERROR - DS internal operation error.");
			break;

		case TWCC_LOWMEMORY:
			lstrcpy (details, "TWCC_LOWMEMORY - Not enough memory to perform operation.");
			break;

		case TWCC_MAXCONNECTIONS:
			lstrcpy (details, "TWCC_MAXCONNECTIONS - DS is connected to max possible apps.");
			break;

		case TWCC_NODS:
			lstrcpy (details, "TWCC_NODS - No Data Source .");
			break;

		case TWCC_SEQERROR:
			lstrcpy (details, "TWCC_SEQERROR - DG DAT MSG out of expected sequence .");
			break;

        case TWCC_BADDEST:
			lstrcpy (details, "TWCC_BADDEST - Unknown destination Application/Source in DSM_Entry.");
			break;

        case TWCC_CAPUNSUPPORTED:
			lstrcpy (details, "TWCC_CAPUNSUPPORTED - Capability not supported by source.");
			break;

        case TWCC_CAPBADOPERATION:
            lstrcpy (details, "TWCC_CAPBADOPERATION - Operation not supported by capability");
            break;

        case TWCC_CAPSEQERROR:
            lstrcpy (details, "TWCC_CAPSEQERROR - Capability has dependancy on other capability");
            break;

		case TWCC_SUCCESS:
			lstrcpy (details, "Success message from detailed error request ??");
			break;

		default:
			lstrcpy (details, "Unknown Error value in condition code.");
			break;
	}

    TRACE("Twain_LM983x : %s\n", specificMsg);
    TRACE("Twain_LM983x : %s\n", details);

    // This return code is simply to make it neater for code
    // to call this function (see usage elsewhere in this module
    // for examples ) and is not really necessary
    return TWRC_FAILURE;
}


/*******************************************************************************
DESCRIPTION:

CSession::Init

PARAMETERS: 


RETURN VALUE:
void 

NOTES:

*******************************************************************************/
void CSession::Init()
{
    TRACE("Twain_LM983x : In CSession::Init()\n");

    m_hInstance = GetModuleHandle("Twain_LM983x.ds");

    // Setup Source's identity and group supported
    m_sourceIdentity.Id = 0L;
    m_sourceIdentity.ProtocolMajor   = TWON_PROTOCOLMAJOR;
    m_sourceIdentity.ProtocolMinor   = TWON_PROTOCOLMINOR;
    m_sourceIdentity.Version.MajorNum = 1;
    m_sourceIdentity.Version.MinorNum = 0;
    m_sourceIdentity.Version.Language = TWLG_USA;
    m_sourceIdentity.Version.Country  = TWCY_USA;
    m_sourceIdentity.SupportedGroups = DG_CONTROL | DG_IMAGE;


    LoadString(m_hInstance, IDS_VERSION,      m_sourceIdentity.Version.Info, sizeof(TW_STR32));
    LoadString(m_hInstance, IDS_MANUFACTURER, m_sourceIdentity.Manufacturer, sizeof(TW_STR32));
    LoadString(m_hInstance, IDS_PRODFAMILY,   m_sourceIdentity.ProductFamily, sizeof(TW_STR32));
    LoadString(m_hInstance, IDS_PRODNAME,     m_sourceIdentity.ProductName, sizeof(TW_STR32));

    // Also we dont have any apps talking to us yet !
    m_appIdentity.Id = 0L;
}

void CSession::SetState(SESSION_STATE state)
{
    m_state = state;
}