/*******************************************************************************
FILE        : Twain_Session.h
REVISION    : 1.0
DATE        : 9/28/99

DESCRIPTION :

The CSession class holds all information that is globalto the current
TWAIN session. Since it is a singlton this information can be 
accessed from  anywhere in the module. This class also implements
the global error handling strategies for this module.

Copyright (c) 1999 - National Semiconductor Ltd - All rights reserved
*******************************************************************************/

#if !defined(AFX_SESSION_H__9714D704_694D_11D3_9859_00104BF5E7FA__INCLUDED_)
#define AFX_SESSION_H__9714D704_694D_11D3_9859_00104BF5E7FA__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Twain_Singleton.h"
    
class CSession : public CSingleton<CSession>  
{
public:
	CSession();
	virtual ~CSession();

    enum SESSION_STATE
    {
        E_STATE3 = 3,
        E_STATE4 = 4,
        E_STATE5 = 5,
        E_STATE6 = 6,
        E_STATE7 = 7
    };

    TW_IDENTITY GetAppIdentity(){return m_appIdentity;}
    TW_IDENTITY GetSourceIdentity(){return m_sourceIdentity;}
    HINSTANCE   GetInstanceHandle(){return m_hInstance;}
    TW_STATUS   GetSourceStatus(){return m_status;}

    void SetAppIdentity(const TW_IDENTITY&);
    void SetSourceIdentity(const TW_IDENTITY&);

    void SetStatusCode(TW_UINT16 conditionCode);

    void Init();

    SESSION_STATE   GetState(){return m_state;}
    void            SetState(SESSION_STATE state);

    // Error handling
    TW_UINT16 HandleError(char *specificMsg, TW_UINT16 conditionCode);


private:
    TW_STATUS       m_status;
    TW_IDENTITY     m_appIdentity;
    TW_IDENTITY     m_sourceIdentity;
    HINSTANCE       m_hInstance;
    SESSION_STATE   m_state;

};

/* 
Error message strings for the operation triplet handlers - These will be written 
to the error log file in the debug mode 
*/

#define E_IDENTITYMSG   (char *)"General failure in response to DAT_IDENTITY.  "
#define E_INTERFACEMSG  (char *)"General failure in response to DAT_USERINTERFACE.  "

#define E_CONTROLMSG    (char *)"General failure in response to DAT_NULL.  "
#define E_ENABLEDS      (char *)"General failure in response to MSG_ENABLEDS.  "
#define E_DISABLEDS     (char *)"General failure in response to MSG_DISABLEDS.  "
#define E_OPENDS        (char *)"General failure in response to MSG_OPENDS.  "
#define E_CLOSEDS       (char *)"General failure in response to MSG_CLOSEDS.  "

#define E_CAPABILITYMSG     (char *)"General failure in response to DAT_CAPABILITY.  "
#define E_STATUSMSG         (char *)"General failure in response to DAT_STATUS.  "
#define E_PENDINGXFERMSG    (char *)"General failure in response to DAT_PENDINGXFER.  "

#define E_SETUPXFERMECHMSG  (char *)"General failure in response to DAT_XFERMECH.  "
#define E_SETUPXFERMECHSET  (char *)"General failure in response to DAT_XFERMECH, MSG_SET, non-supported type.  "

#define E_SETUPMEMXFER      (char *)"General failure in response to DAT_SETUPMEMXFER, not supported.  "
#define E_SETUPFILEXFER     (char *)"General failure in response to DAT_SETUPFILEXFER, not supported.  "

#define E_XFERGROUP         (char *)"General failure in response to DAT_XFERGROUP, not supported.  "

#define E_SETUPNATIVEXFERSET (char *)"General failure in response to DAT_SETUPNATIVEXFER, non-supported type.  "
#define E_SETUPNATIVEXFERMSG (char *)"General failure in response to DAT_SETUPNATIVEXFER.  "

#define E_PALETTE8           (char *)"General failure in response to DAT_PALETTE8.  "

#define E_IMAGEINFOFOPEN     (char *)"General failure in transfer file open.  "
#define E_IMAGEINFORHEAD     (char *)"General failure in transfer reading file header.  "
#define E_IMAGEINFOFTYPE     (char *)"General failure in transfer file is wrong type.  "
#define E_IMAGEINFOIHEAD     (char *)"General failure in transfer reading file info header.  "
#define E_IMAGEINFOBTYPE     (char *)"General failure in transfer bad info header info.  "
#define E_IMAGEINFOMSG       (char *)"General failure in transfer.  "

#define E_IMAGELAYOUTMSG     (char *)"General failure in response to DAT_IMAGEINFO.  "
#define E_IMAGEMEMXFERWTYPE  (char *)"General failure in response to DAT_IMAGEMEMXFER, wrong setup type.  "
#define E_IMAGEMEMXFERMSG    (char *)"General failure in response to DAT_IMAGEMEMXFER.  "
#define E_IMAGENATIVEXFERMSG (char *)"General failure in response to DAT_IMAGENATIVEXFER.  "

#define E_CONTAINERCREATE    (char *)"General failure creating a container.  "
#define E_CONTAINEROPEN      (char *)"General failure opening a container.  "

#endif // !defined(AFX_SESSION_H__9714D704_694D_11D3_9859_00104BF5E7FA__INCLUDED_)
