/*******************************************************************************
FILE        : Twain_Singleton.h
REVISION    : 1.0
DATE        : 9/12/99

DESCRIPTION :

This class serves as the base class for all singletons in this module. it is
parametrized on the type of singleton class and provides common implementations
for the GetInstance function and the static declaration of the single
instance.

USAGE:
To implement a singleton class MySingleton
Derive the class from CSingleton parametrized on MySingleton as shown :

class MySingleton : public CSingleton<MySingleton>
{
public:
// methods
protected:
// methods and data
private:
//data

};

and remember to declare the static pointer in the implementaion file for 
MySingleton as shown :

  MySingleton* CSingleton<MySingleton>::m_theInstance;

Copyright (c) 1999 - National Semiconductor Ltd - All rights reserved
*******************************************************************************/
#ifndef _CSINGLETON_H__
#define _CSINGLETON_H__

template <class T>
class CSingleton
{
public:
    static T* GetInstance()
    {
        if (!m_theInstance)
        {
            m_theInstance = new T;
            m_theInstance->Init();    
        }

        return m_theInstance;
    };
    
    static UnloadInstance()
    {
        delete m_theInstance;
        m_theInstance = 0;
    };

protected:
    static T*  m_theInstance;
    virtual void Init(){}
    
};

#endif // _CSINGLETON_H__