// Twain_UIInterface.cpp: implementation of the CUIInterface class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Twain_UIInterface.h"
#include "Twain_Session.h"

CUIInterface* CSingleton<CUIInterface>::m_theInstance;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CUIInterface::CUIInterface()
{
    m_handleUIModule    = NULL;

    m_fnActivate        = NULL;
    m_fnHandleMessage   = NULL;
    m_fnCallback        = NULL;
    m_fnEndScan         = NULL;
}

CUIInterface::~CUIInterface()
{
    if (m_handleUIModule)
    {
        FreeLibrary(m_handleUIModule);
    }

    m_theInstance = 0;
}

TWAIN_RESULT CUIInterface::Activate(BOOL open)
{
    if (!m_fnActivate)
        return TWAIN_ERROR;

    return ((*m_fnActivate)(open));
}

bool CUIInterface::HandleMessage(MSG *pMsg)
{
    if (!m_fnHandleMessage)
        return false;
    
    return ((*m_fnHandleMessage)(pMsg) != TWAIN_UNKNOWN_MSG);
}

TWAIN_RESULT CUIInterface::Callback(BOOL check_messages)
{
    if (!m_fnCallback)
        return TWAIN_ERROR;
    
    return ((*m_fnCallback)(check_messages));
}

void	CUIInterface::EndScan(TWAIN_RESULT tResult )
{
    if (m_fnEndScan)
        ((*m_fnEndScan)(tResult ));
}

void CUIInterface::LoadUIModule()
{
    if (m_handleUIModule)
        return;

	// Load the full filename with the absolute path for the Scan  module dll 
	char szModulePath[_MAX_PATH];
	char szCompletePath[_MAX_PATH];
	char szFileLocation[_MAX_PATH];
	char szDriveName[_MAX_DRIVE];
    
    HINSTANCE hDSInst = CSession::GetInstance()->GetInstanceHandle();

	GetModuleFileName(hDSInst, szModulePath, _MAX_PATH);
	_splitpath(szModulePath, szDriveName, szFileLocation, NULL, NULL);
	wsprintf(szCompletePath, "%s%s", szDriveName, szFileLocation);
    strcat(szCompletePath, "UI_Lm983x.dll");

    m_handleUIModule = LoadLibrary(szCompletePath);

    if (m_handleUIModule != NULL) 
    {
        m_fnActivate    = (pfnUIActivate)
            GetProcAddress(m_handleUIModule, "UIActivate");
        
        m_fnCallback    = (pfnUICallback)
            GetProcAddress(m_handleUIModule, "UICallback");
        
        m_fnEndScan     = (pfnUIEndScan)
            GetProcAddress(m_handleUIModule, "UIEndScan");

        m_fnHandleMessage =  (pfnUIHandleMessage)
            GetProcAddress(m_handleUIModule, "UIHandleMessage");
    }

    return;
}
