/*******************************************************************************
FILE        : Twain_UIInterface.h
REVISION    : 1.0
DATE        : 10/27/99

DESCRIPTION :

The CUIInterface class provides a facade for accessing all UI module 
funtionality including loading and unloading the UI dll for explicit 
dynamic linking. This class can also insulates the rest of the twain 
module from changes to the UI module.

Copyright (c) 1999 - National Semiconductor Ltd - All rights reserved
*******************************************************************************/

#if !defined(AFX_TWAIN_UIINTERFACE_H__518E6C25_691E_11D3_9859_00104BF5E7FA__INCLUDED_)
#define AFX_TWAIN_UIINTERFACE_H__518E6C25_691E_11D3_9859_00104BF5E7FA__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Twain_Singleton.h"

// The functions exported by the UI module will be accessed 
// using explicit dynamic linking. What this means is that the 
// function pointers have to manually loaded using GetProcAddress
// for which we need the following typedefs
// Typedefs for the entry points into the UI dll
typedef TWAIN_RESULT (*pfnUIActivate)(BOOL);
typedef TWAIN_RESULT (*pfnUIHandleMessage)(MSG *);
typedef TWAIN_RESULT (*pfnUICallback)(BOOL);
typedef void	     (*pfnUIEndScan)(TWAIN_RESULT);

class CUIInterface  : public CSingleton<CUIInterface>
{
public:
	CUIInterface();
	virtual ~CUIInterface();

    TWAIN_RESULT Activate(BOOL open);

    bool  HandleMessage(MSG *pMsg);

    TWAIN_RESULT Callback(BOOL check_messages);

    void	EndScan(TWAIN_RESULT result);

    void LoadUIModule();

protected:
    

private:
    
    pfnUIActivate           m_fnActivate;
    pfnUIHandleMessage      m_fnHandleMessage;
    pfnUICallback           m_fnCallback;
    pfnUIEndScan            m_fnEndScan;

    HINSTANCE               m_handleUIModule;
};

#endif // !defined(AFX_TWAIN_UIINTERFACE_H__518E6C25_691E_11D3_9859_00104BF5E7FA__INCLUDED_)
