// FILE		: UI_GammaDlg.H

// DESCRIPTION	: header file and class definitions for the CGammaWnd class
//		  and the CGammaDlg class

// REVISIONS	: WS 8/19/98, first version
//		: ws 8/20/98, remove local gamma file name
//		: ws 8/20/98, add bischanged flag
//		: ws 8/25/98, restore local gamma file
//-------------------------------------------------------------------
#ifndef _UI_GammaDlg_h_
#define _UI_GammaDlg_h_

#include "UI_image.h"

/////////////////////////////////////////////////////////////////////////////
// CGammaWnd window
//---------------------------------------------------------------
// CLASS	:	CGammaWnd 

// DESCRIPTION	:	image window displayed on gamma dialog

//---------------------------------------------------------------
class CGammaWnd : public CStatic
{
// Construction
public:
	CGammaWnd();

// Attributes
public:
	GAMMACURVE curve;	// current gamma curve parameters
	int xaxis;		// x axis in screen coordinates
	int yaxis;		// y axis in screen coordinates
	BOOL bDrawCircles;	// flag for drawing circles on graph (not used)
	BOOL bEvalCurve;	// display array or computed curve
	int npts;		// number of points in the gamma curve
	int *pRedY;		// red gamma coefficient array
	int *pGreenY;		// green gamma coefficient array
	int *pBlueY;		// blue gamma coefficient array
	int *pAllY;		// all gamma coefficient array
	double constantPart[4];	// intermediate calculation for computing gamma arrays
	int drawColor;		// currently selected tab

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CGammaWnd)
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CGammaWnd();
	void SetGammaValues(BOOL bEvalCurve,GAMMACURVE *pCurve,int npts=0,
				int *predy=NULL,
				int *pgreeny=NULL,
				int *pbluey=NULL,
				int *pally=NULL);
	int MapXToScreen(int x,int width);
	int MapYToScreen(int y);
	int MapScreenToX(int x,int width);
	int MapScreenToY(int y);
	void DrawAxes(CDC *pdc);
	void DrawCircles(CDC *pdc);
	void DrawCurve(CDC *pdc);
	void MoveDot(int i,CPoint pt);
	int EvalCurve(int x,int color);
	void InitGraph(int drawcolor);

	// Generated message map functions
protected:
	//{{AFX_MSG(CGammaWnd)
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnPaint();
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
// CGammaDlg dialog
//---------------------------------------------------------------
// CLASS	:	CGammaDlg 

// DESCRIPTION	:	gamma curve window

//---------------------------------------------------------------
class CGammaDlg : public CDialog
{
// Construction
public:
	CGammaDlg( CString csGammaFileName, GAMMACURVE *pCurve,CDibImage *pImg, CWnd* pParent = NULL);   // standard constructor

// Dialog Data
	//{{AFX_DATA(CGammaDlg)
	enum { IDD = IDD_GAMMA };
	CTabCtrl	m_TabControl;	// tab control
	CGammaWnd	m_GraphWnd;	// gamma graphs
	//}}AFX_DATA

	CString m_filename;	// local gamma file
	CDibImage *pImage;	// image in ui window
	GAMMACURVE curve;	// current gamma curve parameters
	int iSameGamma;		// use all or individual curves (not used)
	int npts;		// number of points in the gamma arrays
#define GAMMA_SIZE 4096
	int redy[GAMMA_SIZE];		// red gamma coefficients
	int greeny[GAMMA_SIZE];	// green gamma coefficients
	int bluey[GAMMA_SIZE];	// blue gamma coefficients
	int ally[GAMMA_SIZE];		// all gamma coefficients
	int drawColor;		// current color selected
	BOOL bCancelProgram;	// cancel read/write of gamma values to merlin
	BOOL bIsModifiable;	// flag indicating the gamma coefficients are known
	BOOL bIsChanged;	// indicate whether gamma curve has changed

	void SaveGammaFile();
	void SetGammaValues(int iSame,GAMMACURVE *pCurve);
	void GetGammaValues(int *iSame,GAMMACURVE *pCurve);
	void SetupScrollControls(int color);
	void ResetAll();
	BOOL SaveDefaultGamma(FILE* fp);
	BOOL ReadGammaFile(FILE *fp);
	CString GetFullFilePath(CString file);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CGammaDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CGammaDlg)
	virtual void OnCancel();
	virtual void OnOK();
	virtual BOOL OnInitDialog();
	afx_msg void OnDestroy();
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnFilesave();
	afx_msg void OnClickCurveselect(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnCurveSelchange(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnGraphcurve();
	afx_msg void OnKillfocusBright();
	afx_msg void OnKillfocusContrast();
	afx_msg void OnKillfocusGamma();
	afx_msg void OnKillfocusHighlight();
	afx_msg void OnKillfocusShadow();
	afx_msg void OnReset();
	afx_msg void OnSetall();
	afx_msg void OnReallycancel();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
#endif
