// image.h : header file
//
#ifndef _image_h_
#define _image_h_

#define SCROLL_RATIO    4     // WM_VSCROLL scrolls DIB by 1/x of client area.

#define BRIGHT_MIN -255
#define BRIGHT_MAX 255
#define CONTRAST_MIN -255
#define CONTRAST_MAX 255
#define HIGHLIGHT_MIN -255
#define HIGHLIGHT_MAX 255
#define SHADOW_MIN -255
#define SHADOW_MAX 255
#define HIGHLIGHT_PLUS_SHADOW_MAX 256

#define IDRB_USEDIBS          104
#define IDRB_USEDDBS          105
#define IDRB_USESETDIBITS     106
#define IDRB_BESTFIT          107
#define IDRB_STRETCH          108

#define PRINT_BESTFIT         IDRB_BESTFIT         // Best proportional stretch fit
#define PRINT_STRETCH         IDRB_STRETCH         // Stretch to fill page
#define PRINT_SCALE           IDRB_SCALE           // Independent X/Y scaling

#define OPT_DEF_STRETCH   FALSE         // Don't stretch the DIB on display.
#define OPT_DEF_BANDING   TRUE          // Band DIB to printer.
#define OPT_DEF_USE31PRNAPIS FALSE      // Don't use the 3.1 Printing APIs.
#define OPT_DEF_DISP      DISP_USE_DDBS // Display DDBs instead of DIBs.
#define OPT_DEF_PRNSIZE   PRINT_BESTFIT // Print in "best fit" mode.
#define OPT_DEF_PRNSCALEX 1             // X stretch factor = 1 (in PRINT_STRETCH mode)
#define OPT_DEF_PRNSCALEY 1             // Y stretch factor = 1 (in PRINT_STRETCH mode)


typedef struct
{
   BOOL bStretch;                         // True = stretch to window
   BOOL bPrinterBand;                     // True = want to band DIB to printer.
   BOOL bUse31PrintAPIs;                  // True = Use the 3.1 Printing API
   WORD wDispOption;                      // See defines below
   WORD wPrintOption;                     // See defines below
   WORD wXScale;                          // X Scale Edit control value
   WORD wYScale;                          // Y Scale Edit control value
} FAR *LPOPTIONSINFO, OPTIONSINFO;

   // Values used for wDispOption in OPTIONSINFO structure.

#define DISP_USE_DIBS         IDRB_USEDIBS
#define DISP_USE_DDBS         IDRB_USEDDBS
#define DISP_USE_SETDIBITS    IDRB_USESETDIBITS

#define PALVERSION   0x300
#define DIB_HEADER_MARKER   ((WORD) ('M' << 8) | 'B')
#define BYTES_PER_READ  32767
#define BI_PM       3L
#define BI_PCX      4L

typedef struct InfoStruct
  {
  char  szName[13];
  char  szType[15];
  DWORD cbWidth;
  DWORD cbHeight;
  DWORD cbColors;
  char  szCompress[5];
  }  INFOSTRUCT;

// WIDTHBYTES takes # of bits in a scan line and rounds up to nearest
//  word.

#define WIDTHBYTES(bits)      (((bits) + 31) / 32 * 4)


// Given a pointer to a DIB header, return TRUE if is a Windows 3.0 style
//  DIB, false if otherwise (PM style DIB).

#define IS_WIN30_DIB(lpbi)  ((*(LPDWORD) (lpbi)) == sizeof (BITMAPINFOHEADER))

enum {
      ERR_MIN = 0,                     // All error #s >= this value
      ERR_NOT_DIB = 0,                 // Tried to load a file, NOT a DIB!
      ERR_MEMORY,                      // Not enough memory!
      ERR_READ,                        // Error reading file!
      ERR_LOCK,                        // Error on a GlobalLock()!
      ERR_OPEN,                        // Error opening a file!
      ERR_CREATEPAL,                   // Error creating palette.
      ERR_GETDC,                       // Couldn't get a DC.
      ERR_CREATECHILD,                 // Error creating MDI child.
      ERR_CREATEDDB,                   // Error create a DDB.
      ERR_STRETCHBLT,                  // StretchBlt() returned failure.
      ERR_STRETCHDIBITS,               // StretchDIBits() returned failure.
      ERR_NODIBORDDB,                  // Error painting -- need BOTH DDB & DIB.
      ERR_SETDIBITSTODEVICE,           // SetDIBitsToDevice() failed.
      ERR_STARTDOC,                    // Error calling StartDoc().
      ERR_NOGDIMODULE,                 // Couldn't find GDI module in memory.
      ERR_SETABORTPROC,                // Error calling SetAbortProc().
      ERR_STARTPAGE,                   // Error calling StartPage().
      ERR_NEWFRAME,                    // Error calling NEWFRAME escape.
      ERR_ENDPAGE,                     // Error calling EndPage().
      ERR_ENDDOC,                      // Error calling EndDoc().
      ERR_ANIMATE,                     // Only one DIB animated @ a time.
      ERR_NOTIMERS,                    // No timers avail for pal animation.
      ERR_NOCLIPWINDOW,                // Now current window for clipboard.
      ERR_CLIPBUSY,                    // Clipboard is busy.
      ERR_NOCLIPFORMATS,               // During paste can't find DIB or DDB.
      ERR_SETDIBITS,                   // Error calling SetDIBits().
      ERR_FILENOTFOUND,                // Error opening file in GetDib()
      ERR_WRITEDIB,                    // Error writing DIB file.
      ERR_MAX                          // All error #s < this value
     };

LPSTR FindDIBBits (LPSTR lpbi);
//BOOL ReadScanParms(HFILE hFile,SCANINFO *pScanInfo,int *len,BOOL bFillSI,BOOL bShowError=TRUE);

class CImageInfo;

/////////////////////////////////////////////////////////////////////////////
// DIBINFO class

class DIBINFO
{
// Construction
public:
	DIBINFO();

// Attributes
public:
	HANDLE      hDIB;                // Handle to the DIB
	HANDLE      hOrigDIB;            // Handle to the original DIB
	CPalette    *pPal;               // Handle to the bitmap's palette.
	HBITMAP	    hBitmap;

	WORD        wDIBType;            // DIB's type - RGB, RLE4, RLE8, PM
	WORD        wDIBBits;            // Bits per pixel
	WORD        wDIBWidth;           // Print width of the DIB
	WORD        wDIBHeight;          // Print height of the DIB

	OPTIONSINFO Options;             // Option info from Options dialog box.

// Operations
public:

// Implementation
public:
	virtual ~DIBINFO();

};


/////////////////////////////////////////////////////////////////////////////
// CImage window

enum {
	MAGCURSOR,
	ARROWCURSOR,
	DRAWCURSOR,
	MOVECURSOR,
	LEFTCURSOR,
	TOPCURSOR,
	RIGHTCURSOR,
	BOTTOMCURSOR,
	ULCURSOR,
	URCURSOR,
	LLCURSOR,
	LRCURSOR
};

/////////////////////////////////////////////////////////////////////////////
class CDibImage : public CWnd
{
// Construction
public:
	CDibImage();

	DIBINFO *pDIBInfo;

// Attributes
public:

// Operations
public:
	void DrawSelectLine(CDC *pDC, int y);
	void DrawSelectRegion(CDC *pDC,CPoint pt);
	void MoveSelectRegion(CDC *pDC,CPoint pt);
	void AdjustSelectRegion(CDC *pDC,CPoint pt);
	void TrackMouse(CDC *pDC, CRect *lpClipRect);
	void DrawSelect (CDC *pDC, CRect *rcClip);
	HCURSOR hMoveCursor;
	HCURSOR hMagCursor;
	BOOL bMagnify;
	int magTimes;
	CRect selectRect;
	CRect outputRect;
	int eCursorType;
	double constantPart[4];
	GAMMACURVE curve;
	int iSameGamma;
	int *hlines;
	void EnableLines(BOOL bEnable);
	void MoveSelectLine(CDC *pDC,CPoint pt);
	void SetHLines(int *hl,int count);
	int nLines;
	int currentHline;
	BOOL bLinesEnabled;
	void DrawLines(CDC *pDC);

	void InitPalette();
	void SetGammaValues(GAMMACURVE *pCurve,int iSame);
//	HANDLE BitmapToDib(CBitmap *pBitmap, CPalette *pPal);
	int EvalCurve(int x,int color);
	CPalette *CreateDIBPalette (HANDLE hDIB);
	void SetImage(HANDLE hdib,BOOL bStretch=FALSE);
	void ScrollBarsOff();
	void SetupScrollBars(int cxDIB, int cyDIB);
	void DIBPaint (CDC *pdc,CRect *lpDCRect,HANDLE hDIB,CRect *lpDIBRect,CPalette * pPal);
	void DDBPaint (CDC *pDC, CRect *lpDCRect, HBITMAP hDDB, CRect *lpDDBRect, CPalette * pPal);
	HBITMAP DIBToBitmap (HANDLE hDIB, CPalette *pPal);
	void DeleteDibInfo();
	void InvertImage(HANDLE hdib);

	HANDLE ConvertToDIB(char *filename,BOOL bFillSI,BOOL bInvert=TRUE,BOOL bShowError=TRUE);
	BOOL WriteDIB (LPSTR szFile);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CImage)
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CDibImage();

	// Generated message map functions
protected:
	//{{AFX_MSG(CImage)
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnPaint();
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#endif
