/*******************************************************************************
FILE        : UI_ScanDialog.cpp
REVISION    : X.X
DATE        : 10/26/99

DESCRIPTION : Implementation file for user interface dialog class


WRITTEN BY	: Steven W. Roth (swroth@grasshut.com / swroth@kionet.com)

Class or Function set description - "Implementation View"

Copyright (c) 1999-2002 - National Semiconductor Corp - All rights reserved
*******************************************************************************/

#include "stdafx.h"
#include "UI_LM983x.h"
#include "UI_ScanDialog.h"
#include "UI_GammaDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MAX_IDLEMESSAGES    50
#define PREVIEW_FILENAME GetFullFilePath("LM983x Preview.Bmp")
#define INIFILE			 GetFullFilePath("LM983x.DEF")

#define UPDATE_TIMER 1

//#define TESTMODE
#ifdef TESTMODE
int m_loop = 0;
#endif

// External Functions

CScanDialog*		g_ScanDialog = NULL;


/*******************************************************************************
DESCRIPTION:	Opens the user interface dialog

PARAMETERS:

				CWnd* pParent	in		parent window

RETURN VALUE:
				TRUE if success / FALSE if failure
*******************************************************************************/
BOOL CScanDialog::OpenScanDialog(CWnd* pParent)
{
	if( g_ScanDialog == NULL )
	{
		g_ScanDialog = new CScanDialog( pParent );
		if( g_ScanDialog != NULL )
		{
			g_ScanDialog->Create( CScanDialog::IDD, pParent );
			g_ScanDialog->ShowWindow( SW_SHOW );
			return TRUE;
		}
	}

	return FALSE;
}


/*******************************************************************************
DESCRIPTION:	Close the user interface dialog.

PARAMETERS:
				NONE

RETURN VALUE:
				TRUE if success / FALSE if failure
*******************************************************************************/
BOOL CScanDialog::CloseScanDialog()
{
	if( g_ScanDialog != NULL )
	{
		g_ScanDialog->SaveSettings();				// Save the current settings
		g_ScanDialog->DestroyWindow();				// Destroy the window
		delete g_ScanDialog;						// Delete the global
		g_ScanDialog = NULL;						// NULL the global
		return TRUE;								// All is well
	}

	return FALSE;
}



/*******************************************************************************
DESCRIPTION:	MFC CDialog Constructor

PARAMETERS:
				See MFC documentation

RETURN VALUE:
				See MFC documentation
*******************************************************************************/
CScanDialog::CScanDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CScanDialog::IDD, pParent)
{
	m_hPreviewBitmap = NULL;					// Init the preview bitmap
	m_zooming = FALSE;							// Init the zoom flag
	m_scanningState = FALSE;					// Init the scanning flag
	m_zoomRect.SetRectEmpty();					// Init the selection rectangle
	m_zoomRectDrawn = FALSE;
	m_preview = FALSE;
	m_updateImageSize = FALSE;
	m_rectPreview.top = m_rectPreview.bottom = 
		m_rectPreview.left = m_rectPreview.right = 0;
}

/*******************************************************************************
DESCRIPTION:	MFC CDialog destructor

PARAMETERS:
				See MFC documentation

RETURN VALUE:
				See MFC documentation
*******************************************************************************/
CScanDialog::~CScanDialog()
{
	// Release the bitmap
	if( m_hPreviewBitmap )
	{
		::DeleteObject( m_hPreviewBitmap );
		m_hPreviewBitmap = NULL;
	}
}


void CScanDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CScanDialog)
	DDX_Control(pDX, IDCANCEL, m_cancel);
	DDX_Control(pDX, IDC_PERCENT100, m_percent100);
	DDX_Control(pDX, IDC_PERCENT0, m_percent0);
	DDX_Control(pDX, IDC_SCANSTATUS, m_scanStatus);
	DDX_Control(pDX, IDC_SCANSTATUS2, m_scanner);
	DDX_Control(pDX, IDC_TOTALTIME, m_totalTime);
	DDX_Control(pDX, IDC_SCANTIME, m_scanTime);
	DDX_Control(pDX, IDC_SCANSIZE, m_scanSize);
	DDX_Control(pDX, IDC_IMAGESIZE, m_imageSize);
	DDX_Control(pDX, IDC_PERCENT, m_percentText);
	DDX_Control(pDX, IDC_PROGRESS, m_progress);
	DDX_Control(pDX, IDC_PREVIEW_VIEW, m_previewArea);
	DDX_Control(pDX, IDC_VERRES, m_vRes);
	DDX_Control(pDX, IDC_BOTHRES, m_bRes);
	DDX_Control(pDX, IDC_HORRES, m_hRes);
	DDX_Control(pDX, IDC_GRAYDEPTH, m_grayDepth);
	DDX_Control(pDX, IDC_COLORDEPTH, m_colorDepth);
	//}}AFX_DATA_MAP
}


/*******************************************************************************
DESCRIPTION:	MFC/Windows Messages

PARAMETERS:		See MFC documentation

RETURN VALUE:	See MFC documentation
*******************************************************************************/

BEGIN_MESSAGE_MAP(CScanDialog, CDialog)
	//{{AFX_MSG_MAP(CScanDialog)
	ON_WM_CLOSE()
	ON_WM_CTLCOLOR()
 	ON_BN_CLICKED(IDC_GAMMA_BUTTON, OnGammaButton)
	ON_BN_CLICKED(IDC_PREVIEW_BUTTON, OnPreviewButton)
	ON_BN_CLICKED(IDC_SAMERES, OnSameres)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_BN_CLICKED(IDC_SCAN_BUTTON, OnScanButton)
	ON_CBN_EDITCHANGE(IDC_HORRES, OnUpdateImageSize)
	ON_CBN_EDITCHANGE(IDC_VERRES, OnUpdateImageSize)
	ON_CBN_EDITCHANGE(IDC_BOTHRES, OnUpdateImageSize)
	ON_CBN_CLOSEUP(IDC_HORRES, OnUpdateImageSize)
	ON_CBN_CLOSEUP(IDC_VERRES, OnUpdateImageSize)
	ON_CBN_CLOSEUP(IDC_BOTHRES, OnUpdateImageSize)
	ON_CBN_SELCHANGE(IDC_HORRES, OnUpdateImageSize)
	ON_CBN_SELCHANGE(IDC_VERRES, OnUpdateImageSize)
	ON_CBN_SELCHANGE(IDC_BOTHRES, OnUpdateImageSize)
	ON_BN_CLICKED(IDC_GRAY, OnUpdateImageSize)
	ON_BN_CLICKED(IDC_COLOR, OnUpdateImageSize)
	ON_CBN_CLOSEUP(IDC_GRAYDEPTH, OnUpdateImageSize)
	ON_CBN_CLOSEUP(IDC_COLORDEPTH, OnUpdateImageSize)
	ON_CBN_SELCHANGE(IDC_GRAYDEPTH, OnUpdateImageSize)
	ON_CBN_SELCHANGE(IDC_COLORDEPTH, OnUpdateImageSize)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/*******************************************************************************
DESCRIPTION:	Handle the close message

PARAMETERS:		none

RETURN VALUE:	none
*******************************************************************************/
void CScanDialog::OnClose() 
{
	OnCancel();							// Tell twain to close
}

/*******************************************************************************
DESCRIPTION:	Handle the gamma button pressed message

PARAMETERS:		none

RETURN VALUE:	none
*******************************************************************************/
void CScanDialog::OnGammaButton() 
{
	int i;
	if (IsDlgButtonChecked(IDC_COLOR))
		i = GetDlgItemInt(IDC_COLORDEPTH)/3;
	else 
		i = GetDlgItemInt(IDC_GRAYDEPTH);
	if (i > 8) return; //not 16 bit
	CString string;
	string.Format("%i",i);
	CString gammaFileName;
	gammaFileName = GetFullFilePath("lm983x_" + string + ".gam");

	GAMMACURVE gammaCurve;
	CGammaDlg		dialog(gammaFileName, &gammaCurve,NULL ,this);

	dialog.DoModal();
}

/*******************************************************************************
DESCRIPTION:	Handle the scan button pressed message

PARAMETERS:		none

RETURN VALUE:	none
*******************************************************************************/
void CScanDialog::OnScanButton() 
{
	TWAIN_RESULT		tResult;
	TWAIN_PARAM_REC		paramRec;
	UpdateImageSize();
	ShowProgressBar(TRUE);				// Show the progress state
	SetScanningState( TRUE );			// Set scanning state
	tResult = GetSettings( &paramRec, FALSE );			// Get the settings w/ preview mode
	m_preview = FALSE;
	if (tResult == TWAIN_OK)
	{
		tResult = TwainSetScanParams(&paramRec);
		if( tResult == TWAIN_OK )
		{
			if( IsDlgButtonChecked(IDC_SAMERES) )
			{
				SetDlgItemInt(IDC_BOTHRES,paramRec.hRes);
			}
			else
			{
				SetDlgItemInt(IDC_HORRES,paramRec.hRes);
				SetDlgItemInt(IDC_VERRES,paramRec.vRes);
			}
			tResult = TwainScan();				// Start scan
		}
	}
	// If we got errors
	if( tResult != TWAIN_OK )
	{
		SetScanningState( FALSE );					// Clear scanning state
		ShowProgressBar(FALSE);						// Hide the progress
		ShowResult( tResult, TWAIN_RESULT_TYPE );	// Report the error
	}
}

/*******************************************************************************
DESCRIPTION:	Handle the cancel button pressed message

PARAMETERS:		none

RETURN VALUE:	none
*******************************************************************************/
void CScanDialog::OnCancel() 
{
	// Handle abort/close
	if( GetScanningState() )
		AbortScan();									// Abort the scan/preview
	else
		TwainClose();						// Tell twain to close
}

/*******************************************************************************
DESCRIPTION:	Handle the preview button pressed message

PARAMETERS:		none

RETURN VALUE:	none
*******************************************************************************/
void CScanDialog::OnPreviewButton() 
{
	TWAIN_RESULT		tResult;
	TWAIN_PARAM_REC		paramRec;

	UpdateImageSize();
#ifdef TESTMODE
	m_loop = 0;
loop:
#endif
	ShowProgressBar(FALSE);				// Disable
	SetScanningState( TRUE );			// Set scanning state
	tResult = GetSettings( &paramRec, TRUE );		// Get the settings w/ preview mode
	m_preview = TRUE;
	if( tResult == TWAIN_OK )
	{
		tResult = TwainSetScanParams(&paramRec);
		if( tResult == TWAIN_OK )
		{
			tResult = TwainPreview(PREVIEW_FILENAME);
			// If we didn't get any errors
			if( tResult == TWAIN_OK )
			{
				// Dump old preview
				if( m_hPreviewBitmap )
				{
					::DeleteObject( m_hPreviewBitmap );
					m_hPreviewBitmap = NULL;
				}
			}
		}
	}
	ShowProgressBar(TRUE);				// Force update
	ShowProgressBar(FALSE);				// Hide progress state
	SetScanningState( FALSE );			// Set scanning state	
	// Report errors
	if( tResult != TWAIN_OK )
	{
		ShowResult( tResult, TWAIN_RESULT_TYPE );	// Report the error
	}
#ifdef TESTMODE
	else 
	{
		Sleep(1000);
		CString string;
		string.Format("test %d complete",++m_loop);
		m_scanStatus.SetWindowText( string );
		goto loop;
	}
#endif
}

/*******************************************************************************
DESCRIPTION:	Handle dialog initialization message

PARAMETERS:		none

RETURN VALUE:	TRUE/FALSE - See MFC CDialog documentaton
*******************************************************************************/
BOOL CScanDialog::OnInitDialog() 
{
	int				i;								// Misc purpose integer
	char			sBuf[256];						// Buffer


	// Call the base class initialization code
	CDialog::OnInitDialog();
	
	// Get the capabilities
	GetCapabilities();

	// Restore last settings
	GetPrivateProfileString("SETUP","SAMERES","0",sBuf,sizeof(sBuf),INIFILE);
	i = atoi(sBuf);
	CheckDlgButton( IDC_SAMERES, i );
	
	GetPrivateProfileString("SETUP","XRES","0",sBuf,sizeof(sBuf),INIFILE);
	i = atoi(sBuf);
	CheckDlgButton( IDC_XINTERPOLATE, i ? TRUE : FALSE );
	CheckDlgButton( IDC_XDISCARD, i ? FALSE : TRUE );

	GetPrivateProfileString("SETUP","YRES","0",sBuf,sizeof(sBuf),INIFILE);
	i = atoi(sBuf);
	CheckDlgButton( IDC_YINTERPOLATE, i ? TRUE : FALSE );
	CheckDlgButton( IDC_YDISCARD, i ? FALSE : TRUE );
	
	GetPrivateProfileString("SETUP","SCANCOLOR","0",sBuf,sizeof(sBuf),INIFILE);
	i = atoi(sBuf);
	CheckDlgButton( IDC_COLOR, i ? TRUE : FALSE );
	CheckDlgButton( IDC_GRAY, i ? FALSE : TRUE );

	GetPrivateProfileString("SETUP","HRES","0",sBuf,sizeof(sBuf),INIFILE);
	i = atoi(sBuf);
	SetDlgItemInt(IDC_HORRES,i);
	
	GetPrivateProfileString("SETUP","VRES","0",sBuf,sizeof(sBuf),INIFILE);
	i = atoi(sBuf);
	SetDlgItemInt(IDC_VERRES,i);
	
	GetPrivateProfileString("SETUP","BRES","0",sBuf,sizeof(sBuf),INIFILE);
	i = atoi(sBuf);
	SetDlgItemInt(IDC_BOTHRES,i);

	GetPrivateProfileString("SETUP","SCANCOLORDEPTH","0",sBuf,sizeof(sBuf),INIFILE);
	if (m_colorDepth.SelectString(-1,sBuf) == CB_ERR)
		m_colorDepth.SetCurSel(3);

	GetPrivateProfileString("SETUP","SCANGRAYDEPTH","0",sBuf,sizeof(sBuf),INIFILE);
	if (m_grayDepth.SelectString(-1,sBuf) == CB_ERR)
		m_grayDepth.SetCurSel(3);
	
	// Make the dialog look right
	OnSameres();								// Same resolution stuff
	ShowProgressBar(FALSE);						// Hide progress, show preview
	SetScanningState( FALSE );					// Clear scanning state

	// Return result
	return TRUE;  // return TRUE unless you set the focus to a control
}

/*******************************************************************************
DESCRIPTION:	Discover the scaner's capabilities.  This goes out and requests
				what the scanner is capable of and fills in internal variables
				for use in the user interface.

PARAMETERS:		none

RETURN VALUE:	result code
*******************************************************************************/
HRESULT CScanDialog::GetCapabilities()
{
	HRESULT			retResult = UI_OK;			// Result code
	HRESULT			result = UI_OK;				// Result code
	int				cap[TWAIN_CAPID_MAXNUM];	// Array of ints
	long            capCount;					// Number of ints returned
	int				index;						// Generic index
	CString			string;						// Generic string
	
	// Clean up for new capabilities
	while( m_hRes.DeleteString(0) >= 0 );			// Dump all entries in the comboBox
	m_hRes.EnableWindow(FALSE);						// Disable until we put something there
	while( m_vRes.DeleteString(0) >= 0 );			// Dump all entries in the comboBox
	m_vRes.EnableWindow(FALSE);						// Disable until we put something there
	while( m_bRes.DeleteString(0) >= 0 );			// Dump all entries in the comboBox
	m_bRes.EnableWindow(FALSE);						// Disable until we put something there
	while( m_colorDepth.DeleteString(0) >= 0 );		// Dump all entries in the comboBox
	m_colorDepth.EnableWindow(FALSE);				// Disable until we put something there
	while( m_grayDepth.DeleteString(0) >= 0 );		// Dump all entries in the comboBox
	m_grayDepth.EnableWindow(FALSE);				// Disable until we put something there


	// Fill in the resolution capabilities
	TWAIN_HARDWARE_REC hRec;
	TwainGetScanHardware(&hRec);
	for (int i=0;i < BASE_RESOLUTION_SIZE; i++)
	{
		int res = hRec.baseResolution[i];
		if (!res) break;
		if (GetClosestCap(TWAIN_CAP_HRES,res) != res) continue;
		if (GetClosestCap(TWAIN_CAP_VRES,res) != res) continue;
		string.Format( "%d", res );	// Build the string
		m_hRes.AddString( (LPCTSTR)string );// Insert it into the comboBox
		m_vRes.AddString( (LPCTSTR)string );// Insert it into the comboBox
		m_bRes.AddString( (LPCTSTR)string );// Insert it into the comboBox
	}

	m_hRes.EnableWindow(TRUE);			//  Yes, enable the comboBox
	m_vRes.EnableWindow(TRUE);			//  Yes, enable the comboBox
	m_bRes.EnableWindow(TRUE);			//  Yes, enable the comboBox

	// Build the pixel depth configurations
	result = TwainGetCapability( TWAIN_CAP_PIXELTYPE, NULL, cap, &capCount );
	if( result == UI_OK )
	{
		CComboBox*		pComboBox;						// Pointer to correct comboBox

		// Walk through all pixel types
		for( index = 0; index < capCount; index++ )
		{
			switch( cap[index] )						// Get the pixel type
			{
			case TWAIN_PIXELTYPE_BW:
			case TWAIN_PIXELTYPE_GRAY:
				pComboBox = &m_grayDepth;				// Used gray depth comboBox
				break;

			case TWAIN_PIXELTYPE_RGB:
				pComboBox = &m_colorDepth;				// Used color depth comboBox
				break;
			default:
				pComboBox = NULL;						// Unknown association
			}
			
			// Process pixel depths for this pixel type
			if( pComboBox != NULL )
			{
				int		pdIndex;								// Pixel type index
				int		depthCap[TWAIN_CAPID_MAXNUM];			// Array of ints
				long    depthCapCount;							// Number of ints returned
				
				// Get pixel depths for this type
				result = TwainGetCapability( TWAIN_CAP_PIXELDEPTH, (void*)&cap[index], depthCap, &depthCapCount );
				if( result == UI_OK )
				{
					for( pdIndex = 0; pdIndex < depthCapCount; pdIndex++ )
					{
						string.Format( "%d", depthCap[pdIndex] );	// Build the string
						pComboBox->AddString( (LPCTSTR)string );	// Insert it into the comboBox
					}
					
					if( depthCapCount > 0 )							// Do we have entries?
						pComboBox->EnableWindow(TRUE);				//  Yes, enable the comboBox
				}
				else
					retResult = result;
			}
		}
	}
	else
		retResult = result;
	
	// Get maximum scan width
	result = TwainGetCapability( TWAIN_MAX_SCAN_WIDTH, NULL, cap, &capCount );
	if( result == UI_OK )
		m_capMaxWidth = cap[0];
	else
		retResult = result;

	// Get maximum scan height
	result = TwainGetCapability( TWAIN_MAX_SCAN_HEIGHT, NULL, cap, &capCount );
	if( result == UI_OK )
		m_capMaxHeight = cap[0];
	else
		retResult = result;

	// return the result
	return retResult;
}

/*******************************************************************************
DESCRIPTION:	Handle the same resolution check message.  Configures the
				interface accordingly.

PARAMETERS:		none

RETURN VALUE:	none
*******************************************************************************/
void CScanDialog::OnSameres() 
{
	BOOL		both;
	int			bSetting;
	int			hvSetting;
	

	// Figure out the settings
	both = IsDlgButtonChecked( IDC_SAMERES );
	bSetting = both ? SW_SHOW : SW_HIDE;
	hvSetting = both ? SW_HIDE : SW_SHOW;
	
	// For each control either show it or hide it
	GetDlgItem(IDC_BOTHRES)->ShowWindow( bSetting );
	GetDlgItem(IDC_BOTHRESTITLE)->ShowWindow( bSetting );
	GetDlgItem(IDC_HORRES)->ShowWindow( hvSetting );
	GetDlgItem(IDC_HORRESTITLE)->ShowWindow( hvSetting );
	GetDlgItem(IDC_VERRES)->ShowWindow( hvSetting );
	GetDlgItem(IDC_VERRESTITLE)->ShowWindow( hvSetting );

	OnUpdateImageSize();
}


/*******************************************************************************
DESCRIPTION:	Loads the privew bitmap from disk for use in the preview pane.

PARAMETERS:		none

RETURN VALUE:	result code from LoadImage call (see Windows documentation for
				possible result codes).
*******************************************************************************/
HRESULT CScanDialog::LoadPreviewBitmap()
{
	// Dump old preview
	if( m_hPreviewBitmap )
	{
		::DeleteObject( m_hPreviewBitmap );
		m_hPreviewBitmap = NULL;
	}

	// Load the preview bitmap image
	m_hPreviewBitmap = (HBITMAP)::LoadImage(	NULL,
												PREVIEW_FILENAME,
												IMAGE_BITMAP,
												0,0,
												LR_LOADFROMFILE );
	
	// Return result code
	if( m_hPreviewBitmap != NULL )
		return( 0 );
	else
		return( GetLastError() );
}

/*******************************************************************************
DESCRIPTION:	Returns full path name
PARAMETERS:		none

RETURN VALUE:	CString containing the path.  If error, then a null
				string is returned.
*******************************************************************************/
CString CScanDialog::GetFullFilePath(CString file)
{
	CString		path;
	char		directory[MAX_PATH];

	if (!GetWindowsDirectory(directory,MAX_PATH))
	{
		AfxMessageBox("Can't find Windows directory!");
		return "";
	}
	path = directory;
	if (path.Right(1) != "\\") path += "\\";
	path += "twain_32\\lm983x\\";
	path += file;
	return( path );
}

/*******************************************************************************
DESCRIPTION:	Handle the on paint window message.  See MFC CDialog
				documentation.

PARAMETERS:		none

RETURN VALUE:	none
*******************************************************************************/
void CScanDialog::OnPaint() 
{
    if (IsIconic())
	{
/*		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);*/
	}
	else
	{ 
		if( m_hPreviewBitmap == NULL )
			LoadPreviewBitmap();

        CPaintDC dc(this); // device context for painting
		if( m_hPreviewBitmap )
		{
			// Paint the preview
			CRect			r;
			CDC				memDC;
			CDC*			pDC;
			void*			old;
			CBitmap			bm;
			BITMAP			bitmap;
			CBrush			brush;

			pDC = m_previewArea.GetDC(); 
			memDC.CreateCompatibleDC( pDC );
			old = memDC.SelectObject( m_hPreviewBitmap );
			bm.Attach( m_hPreviewBitmap );
			bm.GetBitmap( &bitmap );

	        m_previewArea.GetClientRect( &r );
//			brush.CreateSysColorBrush(COLOR_BTNFACE);
//			pDC->FillRect(&r,&brush);

			//keep same ratio as bitmap
			long rwidth = min(r.Width(),r.Height()*bitmap.bmWidth/bitmap.bmHeight);
			long rheight = min(r.Height(),r.Width()*bitmap.bmHeight/bitmap.bmWidth);
			pDC->SetStretchBltMode( COLORONCOLOR );
			pDC->StretchBlt( r.left, r.top, rwidth, rheight,
							&memDC, 0, 0, bitmap.bmWidth, bitmap.bmHeight, SRCCOPY );
			memDC.SelectObject( old );
			bm.Detach();

			//Save actual display rect
			m_rectPreview.top  = r.top;
			m_rectPreview.left = r.left;
			m_rectPreview.right = r.left + rwidth;
			m_rectPreview.bottom = r.top + rheight;
		
			if( m_previewArea.IsWindowVisible() )
			{
				if( !m_zoomRect.IsRectNull() )
				{
					InvertZoomRect( &m_zoomRect );
					m_zoomRectDrawn = TRUE;
				}
			}
		}
	}
}


/*******************************************************************************
DESCRIPTION:	The TrackZoom functon tracks the mouse movement while
				selecting the area to scan.

PARAMETERS:
				startPt contains the point at which the mouse was clicked and
				serves as the starting selection point.

RETURN VALUE:	The selected rectangle is returned.
*******************************************************************************/
CRect CScanDialog::TrackZoom(CPoint startPt)
{	CRect			r;
	int				left,top,bottom,right;
	CRect			bounds;
	CPoint			pt;
	BOOL			track = TRUE;
	MSG				msg;
	CRect			oldCursorClip;


	// Erase old
	if( m_zoomRectDrawn )
	{
		InvertZoomRect( &m_zoomLastRect );
		m_zoomRectDrawn = FALSE;
	}

	// Prep
	m_zoomRect.SetRectEmpty();
	if( startPt.x == 0 && startPt.y == 0 )
		GetCursorPos(&pt);
	else
		pt = startPt;

	m_zoomStartPoint = pt;
	m_zoomStartPoint = pt;
	bounds = m_rectPreview;

	// Clip cursor
	GetClipCursor(&oldCursorClip);
	r = bounds;
	m_previewArea.ClientToScreen(&r);
	ClipCursor(&r);

	// Track
	while( track )
	{
		if (PeekMessage (&msg, NULL, WM_MOUSEFIRST, WM_MOUSELAST, PM_REMOVE))
		{
			if( m_zoomRectDrawn )
			{
				InvertZoomRect( &m_zoomLastRect );
				m_zoomRectDrawn = FALSE;
			}

			if (msg.message == WM_LBUTTONUP)
			{
				track = FALSE;
				break;
			}
			
			GetCursorPos(&pt);
			m_previewArea.ScreenToClient( &pt );
			m_zoomEndPoint = pt;
			
			left = pt.x <= m_zoomStartPoint.x ? pt.x : m_zoomStartPoint.x;
			top = pt.y <= m_zoomStartPoint.y ? pt.y : m_zoomStartPoint.y;
			right = pt.x >= m_zoomStartPoint.x ? pt.x : m_zoomStartPoint.x;
			bottom = pt.y >= m_zoomStartPoint.y ? pt.y : m_zoomStartPoint.y;

			if( left < bounds.left )
				left = bounds.left;
			if( top < bounds.top )
				top = bounds.top;
			if( right > bounds.right )
				right = bounds.right;
			if( bottom > bounds.bottom )
				bottom = bounds.bottom;

			r.SetRect(left,top,right,bottom);
			m_zoomRect = r;

			if( !m_zoomRect.IsRectNull() )
			{
				InvertZoomRect( &m_zoomRect );
				m_zoomRectDrawn = TRUE;
			}

			m_zoomLastRect = m_zoomRect;
		}
	}
	
	if( m_zoomRect.Width() < 2 ||
		m_zoomRect.Height() < 2 )
	{
		pt.x = 0;
		pt.y = 0;
		m_zoomStartPoint = pt;
		m_zoomEndPoint = pt;
		m_zoomRect.SetRectEmpty();
	}

	ClipCursor(&oldCursorClip);

	return( m_zoomRect );
}

/*******************************************************************************
DESCRIPTION:	InvertZoomRect toggles the displayed zoom (selection)
				rectangle's visibility (drawn or not).

PARAMETERS:
				pRect is a pointer to the rectangle to be drawn/erased.

RETURN VALUE:
				none
*******************************************************************************/
void CScanDialog::InvertZoomRect( CRect *pRect )
{
	CDC*			pDC;

	
	if (pRect->IsRectEmpty()) return;

	pDC = m_previewArea.GetDC();

	pDC->PatBlt (
		pRect->left,
		pRect->top,
		pRect->right - pRect->left,
		1,
		DSTINVERT);

	pDC->PatBlt (
		pRect->left,
		pRect->bottom,
		1,
		-(pRect->bottom - pRect->top),
		DSTINVERT);

	pDC->PatBlt (
		pRect->right,
		pRect->top,
		1,
		pRect->bottom - pRect->top,
		DSTINVERT);

	pDC->PatBlt (
		pRect->right,
		pRect->bottom,
		-(pRect->right - pRect->left),
		1,
		DSTINVERT);
}


/*******************************************************************************
DESCRIPTION:	ShowProgressBar toggles the view of the progress bar.

PARAMETERS:
				bShow is a TRUE/FALSE flag to show or hide the progress bar.

RETURN VALUE:
				none
*******************************************************************************/
void CScanDialog::ShowProgressBar(BOOL bShow)
{
	SetProgressBar(0,0);								// Set progress bar to zero

	GetDlgItem(IDC_PREVIEW_VIEW)->ShowWindow(TRUE);		// the preview area
	GetDlgItem(IDC_PROGRESS)->ShowWindow(bShow);			// the progress bar
	GetDlgItem(IDC_PERCENT)->ShowWindow(bShow);				// percent indicator
	GetDlgItem(IDC_PERCENT0)->ShowWindow(FALSE);			// the progress bar
	GetDlgItem(IDC_PERCENT100)->ShowWindow(FALSE);			// percent indicator
	
}

/*******************************************************************************
DESCRIPTION:	SetProgress sets the current progress percentage text

PARAMETERS:
				pInfoRec is a pointer to the TWAIN_INFO_REC which contains
				necessary progress information.

RETURN VALUE:
				none
*******************************************************************************/
void CScanDialog::SetProgress( TWAIN_INFO_REC* pInfoRec )
{
	// Fill in the progress bar
	SetProgressBar( pInfoRec->rawTotalBytes, pInfoRec->rawActBytes );

	// Fill in the time and other stats
	long		tDiffTime;
	CString		string;

	tDiffTime = pInfoRec->scanEndTime - pInfoRec->scanStartTime;
	string = "";
	if (tDiffTime)
		string.Format("%6.1f ",tDiffTime/1000.);
	m_scanTime.SetWindowText( string );

	tDiffTime = pInfoRec->endTime - pInfoRec->startTime;
	string = "";
	if (tDiffTime)
		string.Format("%6.1f ",tDiffTime/1000.);
	m_totalTime.SetWindowText( string );

	string = "";
	if (pInfoRec->rawTotalBytes)
		string.Format("%ldK", pInfoRec->rawTotalBytes/1024);
	if (!m_preview) m_scanSize.SetWindowText( string );

	string = "";
	if (pInfoRec->totalBytes)
	{
		if (pInfoRec->totalBytes > 0)
			string.Format("%ldK", pInfoRec->totalBytes/1024);
		else //-Kbytes encoded
			string.Format("%ldK", -pInfoRec->totalBytes);
	}
	if (!m_preview) m_imageSize.SetWindowText( string );
}


/*******************************************************************************
DESCRIPTION:	SetProgressBar sets the progress bars percentage complete

PARAMETERS:
				lMax is passed in and is the maximum units

RETURN VALUE:
				lValue is passed in and contains the unit count (%)

NOTES:
				If lMax = 200 and lValue = 50, then the percentage is 25%.
******************************************************************************/
void CScanDialog::SetProgressBar(long lMax, long lValue)
{
	long		percent;
	CString		csPercent;
	CString		csFormat;

	m_progress.SetRange32( 0, lMax );
	m_progress.SetPos( lValue );
	
	percent = (lMax != 0) ? (long)(lValue*100./lMax) : 0;
	csFormat.LoadString( IDS_PERCENT_COMPLETE );
	csPercent.Format( (LPCTSTR)csFormat, percent );
	m_percentText.SetWindowText( csPercent );
}

/*******************************************************************************
DESCRIPTION:	SaveSettings saves current settings to disk so the user
				interface can restore to the same state on next launch.

PARAMETERS:
				none

RETURN VALUE:
				none
*******************************************************************************/
void CScanDialog::SaveSettings()
{
	int			i;
	char		sBuf[256];


	i = IsDlgButtonChecked(IDC_SAMERES);
	wsprintf(sBuf,"%d",i);
	WritePrivateProfileString("SETUP","SAMERES",sBuf,INIFILE);

	GetDlgItemText(IDC_BOTHRES,sBuf,sizeof(sBuf));
	WritePrivateProfileString("SETUP","BRES",sBuf,INIFILE);
	GetDlgItemText(IDC_HORRES,sBuf,sizeof(sBuf));
	WritePrivateProfileString("SETUP","HRES",sBuf,INIFILE);
	GetDlgItemText(IDC_VERRES,sBuf,sizeof(sBuf));
	WritePrivateProfileString("SETUP","VRES",sBuf,INIFILE);

	i = IsDlgButtonChecked(IDC_XINTERPOLATE);
	wsprintf(sBuf,"%d",i);
	WritePrivateProfileString("SETUP","XRES",sBuf,INIFILE);

	i = IsDlgButtonChecked(IDC_YINTERPOLATE);
	wsprintf(sBuf,"%d",i);
	WritePrivateProfileString("SETUP","YRES",sBuf,INIFILE);

	i = IsDlgButtonChecked(IDC_COLOR);
	wsprintf(sBuf,"%d",i);
	WritePrivateProfileString("SETUP","SCANCOLOR",sBuf,INIFILE);
	
	GetDlgItemText(IDC_COLORDEPTH,sBuf,sizeof(sBuf));
	WritePrivateProfileString("SETUP","SCANCOLORDEPTH",sBuf,INIFILE);
	GetDlgItemText(IDC_GRAYDEPTH,sBuf,sizeof(sBuf));
	WritePrivateProfileString("SETUP","SCANGRAYDEPTH",sBuf,INIFILE);
}

/*******************************************************************************
DESCRIPTION:	OnLButtonDown is used for tracking the selection (zoom) area.

PARAMETERS:
				See MFC documentation
RETURN VALUE:
				none
*******************************************************************************/
void CScanDialog::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CPoint		pt;
	CRect		r;

	r = m_rectPreview;
	pt = point;
	ClientToScreen(&pt);
	m_previewArea.ScreenToClient( &pt );
	if( r.PtInRect(pt) && !GetScanningState())
	{
		m_zooming = TRUE;
		TrackZoom( pt );
	
		// Show zoom rect
		if( m_zoomRectDrawn )
		{
			InvertZoomRect( &m_zoomLastRect );
			m_zoomRectDrawn = FALSE;
		}
		if( !m_zoomRect.IsRectNull() )
		{
			InvertZoomRect( &m_zoomRect );
			m_zoomRectDrawn = TRUE;
		}
		OnUpdateImageSize();
	}

	CDialog::OnLButtonDown(nFlags, point);
}

/*******************************************************************************
DESCRIPTION:	OnLButtonUp is used for tracking the selection (zoom) area.

PARAMETERS:
				See MFC documentation

RETURN VALUE:
				none
*******************************************************************************/
void CScanDialog::OnLButtonUp(UINT nFlags, CPoint point) 
{
	m_zooming = FALSE;
	CDialog::OnLButtonUp(nFlags, point);
}

HBRUSH CScanDialog::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	if (m_updateImageSize) 
	{
		OnUpdateImageSize();
	}
	return CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
}

void CScanDialog::OnTimer(UINT nIDEvent) 
{
	if (nIDEvent == UPDATE_TIMER)
	{
		UpdateImageSize();
	}
	CDialog::OnTimer(nIDEvent);
}



/*******************************************************************************
DESCRIPTION:	CallBack is used to give the user interface runtime while
				the TWAIN module is performing a scan.

PARAMETERS:
				none
RETURN VALUE:
				An error TWAIN_RESULT will cause the scan to terminate.
NOTES:
				Use this functon to update progress and other aspects of
				the user interface during a scan sequence.
*******************************************************************************/
TWAIN_RESULT CScanDialog::CallBack(BOOL check_messages)
{
    TRACE("UI_LM983x : In CScanDialog::Callback()\n");

    TWAIN_RESULT		tResult;			// Twain result code
	TWAIN_INFO_REC		infoRec;			// Twain progress info record

	// Get the status/info and update the interface
	tResult = TwainGetScanInfo(&infoRec);	// Get the progress info

    if( tResult == TWAIN_OK )
	{
		m_scanStatus.SetWindowText( infoRec.scanStatus );
		if (!check_messages) return tResult;
		SetProgress( &infoRec );			// Update the interface
	}
	
    // Pump messages that may have been queued up
    int messagesPumped = 0;
    MSG msg;
    while ( ::PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) &&
            messagesPumped < MAX_IDLEMESSAGES) 
    { 
        if ( !AfxGetThread()->PumpMessage() ) 
        { 
            AfxPostQuitMessage(0); 
		    return( TWAIN_ABORT );				// Return the abort result
        } 
        messagesPumped++;
    } 

    // let MFC do its idle processing
    LONG lIdle = 0;
    while ( AfxGetApp()->OnIdle(lIdle++ ) )
        ;  

    // Return without errors
	return( tResult );
}

/*******************************************************************************
DESCRIPTION:	GetSettings returns the current user interface settings in
				a TWAIN_PARAM_REC.

PARAMETERS:
				pParamRec	Contains a pointer to he TWAIN_PARAM_REC that is to
							be filled in.
				bPreview	TRUE/FALSE is passed in to specify the scan type
							about to be performed.  If TRUE, then the zoom rect
							is zero to perform a complete scan.

RETURN VALUE:
				result
*******************************************************************************/
TWAIN_RESULT CScanDialog::GetSettings(TWAIN_PARAM_REC* pParamRec, BOOL bPreview )
{
	// Get closest match to valid resolutions
	int hres,vres;
	if( IsDlgButtonChecked(IDC_SAMERES) )
	{
		hres = vres = GetDlgItemInt(IDC_BOTHRES);
	}
	else
	{
		hres = GetDlgItemInt(IDC_HORRES);
		vres = GetDlgItemInt(IDC_VERRES);
	}
	pParamRec->hRes = GetClosestCap(TWAIN_CAP_HRES,hres);
	pParamRec->vRes = GetClosestCap(TWAIN_CAP_VRES,vres);

	// Get interp stuff
	pParamRec->hInterp = IsDlgButtonChecked(IDC_XINTERPOLATE);
	pParamRec->vInterp = IsDlgButtonChecked(IDC_YINTERPOLATE);

	// Get color depth
    // [Vicky] - The color mode is an enum and has to be filled in 
    // accordingly
    if (!IsDlgButtonChecked(IDC_COLOR))
    {
		pParamRec->bitsPerPixel = GetDlgItemInt( IDC_GRAYDEPTH );
        if (pParamRec->bitsPerPixel == 1)
            pParamRec->pixelType = TWAIN_PIXELTYPE_BW;
         else
            pParamRec->pixelType = TWAIN_PIXELTYPE_GRAY;
    }
    else
    {
		pParamRec->bitsPerPixel = GetDlgItemInt( IDC_COLORDEPTH );
        pParamRec->pixelType = TWAIN_PIXELTYPE_RGB;
    }

	//fixed setting for preview
	if (bPreview)
	{
		pParamRec->hRes = pParamRec->vRes = 75;
		pParamRec->hInterp = TRUE;
		pParamRec->vInterp = TRUE;
		pParamRec->bitsPerPixel = 24;
		pParamRec->pixelType = TWAIN_PIXELTYPE_RGB;
#ifdef TESTMODE
		pParamRec->bitsPerPixel = 1;
		pParamRec->pixelType = TWAIN_PIXELTYPE_BW;
#endif
	}

	TWAIN_HARDWARE_REC hardwareRec;
	TWAIN_RESULT tResult = TwainGetScanHardware(&hardwareRec);
	if (tResult != TWAIN_OK) return tResult;
	// compute bounding rect in pixels
	pParamRec->boundingRect.left   = 0;
	pParamRec->boundingRect.right  = hardwareRec.widthOpticalPixels
										*pParamRec->hRes
										/hardwareRec.opticalRes;
	pParamRec->boundingRect.top    = 0;
	pParamRec->boundingRect.bottom = hardwareRec.heightOpticalPixels
										*pParamRec->vRes
										/hardwareRec.opticalRes;

    pParamRec->scanRect = pParamRec->boundingRect;

#ifdef TESTMODE
	pParamRec->scanRect.left   = 0x0d85/8;
	pParamRec->scanRect.right  = pParamRec->scanRect.left + 0x10*
									1;//7;
	pParamRec->scanRect.top    = 100;
	pParamRec->scanRect.bottom = pParamRec->scanRect.top + 512;//510;//518/4;
#endif

 	if( !bPreview )										// If not previewing
		if( !m_zoomRect.IsRectNull() )					// and we have a selected rect
		{
			//transform coordinates for scan area to optical pixels
			long pixelboundingwidth = 
				pParamRec->boundingRect.right - 
				pParamRec->boundingRect.left;
			long pixelboundingheight = 
				pParamRec->boundingRect.bottom -
				pParamRec->boundingRect.top;

			RECT coorboundingRect,coorscanRect;
			coorboundingRect = m_rectPreview;
			long coorboundingwidth = 
				coorboundingRect.right - 
				coorboundingRect.left;
			long coorboundingheight = 
				coorboundingRect.bottom - 
				coorboundingRect.top;
			coorscanRect = m_zoomRect;

			pParamRec->scanRect.left = 
				(((coorscanRect.left - coorboundingRect.left) *
				pixelboundingwidth)/coorboundingwidth) +
				pParamRec->boundingRect.left;

			pParamRec->scanRect.right = 
				(((coorscanRect.right - coorboundingRect.left) *
				pixelboundingwidth)/coorboundingwidth) +
				pParamRec->boundingRect.left;

			pParamRec->scanRect.top = 
				(((coorscanRect.top - coorboundingRect.top) *
				pixelboundingheight)/coorboundingheight) +
				pParamRec->boundingRect.top;


			pParamRec->scanRect.bottom = 
				(((coorscanRect.bottom - coorboundingRect.top) *
				pixelboundingheight)/coorboundingheight) +
				pParamRec->boundingRect.top;


		}

	//Update scanner info
	m_scanner.SetWindowText( hardwareRec.device );
	return tResult;

}

/*******************************************************************************
DESCRIPTION:	ShowResult displays an error result to the user.

PARAMETERS:
				result contains the result code
				nResultType contains the result code type
RETURN VALUE:
				none
*******************************************************************************/
void CScanDialog::ShowResult( long result, int nResultType )
{
    // [vicky] - avoid error displays on success
    if (result == TWAIN_OK)
        return;

    int			stringID;
	CString		string;
	CString		tResultString;


	// TODO: Resolve type/result to string
	switch( result )
	{
	case TWAIN_ABORT:
		stringID = IDS_ERROR_ABORT;
		string.Format( stringID, result );
		m_scanStatus.SetWindowText( string );
		break;
	default:
		stringID = IDS_ERROR_GENERIC; //not displayed now
	}
}

/*******************************************************************************
DESCRIPTION:	EndScan is called by the TWAIN module to inform the user
				interface that the scan sequence has completed.

PARAMETERS:
				tResult contans the result of the scan sequence
RETURN VALUE:
				none
*******************************************************************************/
void CScanDialog::EndScan( TWAIN_RESULT tResult )
{
	if( GetScanningState() )
	{
		SetScanningState( FALSE );					// Clear scanning state
		ShowProgressBar(FALSE);						// Hide the progress
		ShowResult( tResult );						// Show the error result
	}
}

/*******************************************************************************
DESCRIPTION:	AbortScan is called by the user interface to schedule the scan
				to be aborted in the next CallBack call from the TWAIN module.

PARAMETERS:
				none
RETURN VALUE:
				none
*******************************************************************************/
void CScanDialog::AbortScan()
{
	TwainAbort();
}

/*******************************************************************************
DESCRIPTION:	SetScanningState sets up the user interface for either a scan
				or interact state.

PARAMETERS:
				bScanning	Contains the state of scanning or not.  If this is
							set to TRUE, then the user interface disables
							controls.
				bQuiet		If TRUE, then the internal variable is set, but
							actual window controls are not manipulated.
RETURN VALUE:
				none
*******************************************************************************/
void CScanDialog::SetScanningState(BOOL bScanning, BOOL bQuiet )
{
	CString		string;
	int			item[16] = {	IDC_HORRES,
								IDC_VERRES,
								IDC_BOTHRES,
								IDC_SAMERES,
								IDC_XINTERPOLATE,
								IDC_YINTERPOLATE,
								IDC_XDISCARD,
								IDC_YDISCARD,
								IDC_GRAY,
								IDC_COLOR,
								IDC_GRAYDEPTH,
								IDC_COLORDEPTH,
								IDC_PREVIEW_BUTTON,
								IDC_SCAN_BUTTON,
								IDC_GAMMA_BUTTON,
								0 };


	m_scanningState = bScanning;					// Set new state

	// If we are to be quiet, don't update the window
	if( !bQuiet )
	{
		// Toggle abort/close button
		string.LoadString( bScanning ? IDS_ABORT : IDS_CLOSE );		// Load the correct button title
		m_cancel.SetWindowText( (LPCTSTR)string );					// Set the title

		// Activate/deactivate controls
		for( int i = 0; item[i] != 0; i++ )							// Loop through all items
			GetDlgItem(item[i])->EnableWindow(!bScanning);			// Set item state
	}
}

void CScanDialog::OnUpdateImageSize() 
{
	m_updateImageSize = TRUE;
	SetTimer(UPDATE_TIMER,200,NULL);
}

TWAIN_RESULT CScanDialog::UpdateImageSize() 
{
	KillTimer(UPDATE_TIMER);
	m_updateImageSize = FALSE;
	TWAIN_RESULT		tResult = TWAIN_OK;
	// TODO: Add your control notification handler code here
	TWAIN_PARAM_REC		paramRec;
	TWAIN_INFO_REC		infoRec;			// Twain progress info record
	tResult = GetSettings( &paramRec, FALSE );		// Get the settings w/ scan mode
	if (tResult == TWAIN_OK)
	{
		tResult = TwainSetScanParams(&paramRec);
		if (tResult == TWAIN_OK) 
		{
			m_preview = FALSE;
			// Get the status/info and update the interface
			tResult = TwainGetScanInfo(&infoRec);	// Get the progress info
			if( tResult == TWAIN_OK )
			{
				SetProgress( &infoRec );			// Update the interface
			}
		}
	}
	return tResult;
}

//Get closest cap value
int CScanDialog::GetClosestCap(TWAIN_CAPID capID,int val)
{
	int				cap[TWAIN_CAPID_MAXNUM];	// Array of ints
	long            capCount;
	int	result = TwainGetCapability( capID, NULL, cap, &capCount );
	if( result != UI_OK )
	{
		return 1;	//return 1 value
	}
	int mindiff = 0xffff;
	for (int i = 0; i < capCount; i++)
	{
		int diff = val - cap[i];
		if (abs(diff) < abs(mindiff)) mindiff = diff;
		if (!mindiff) break;		//done, found value
	}
	return val - mindiff;	//closest value
}


