/*******************************************************************************
FILE        : UI_ScanDialog.h
REVISION    : X.X
DATE        : 10/27/99

DESCRIPTION : Header file for user interface dialog class


WRITTEN BY	: Steven W. Roth (swroth@grasshut.com / swroth@kionet.com)

Class or Function set description - "interface" view

Copyright (c) 1999-2002 - National Semiconductor Corp - All rights reserved
*******************************************************************************/

#if !defined(AFX_UI_SCANDIALOG_H__8E3003E4_6540_11D3_B379_080046044C7B__INCLUDED_)
#define AFX_UI_SCANDIALOG_H__8E3003E4_6540_11D3_B379_080046044C7B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// UI_ScanDialog.h : header file
//

#include "Twain_LM983x.h"
#include "..\include\UI_LM983x.h"

enum {
	UI_RESULT_TYPE = 1,
	TWAIN_RESULT_TYPE
};

/////////////////////////////////////////////////////////////////////////////
// CScanDialog dialog

class CScanDialog : public CDialog
{
// Construction
public:
	void EndScan( TWAIN_RESULT tResult );
	TWAIN_RESULT CallBack(BOOL check_messages);

	static BOOL OpenScanDialog(CWnd* pParent=NULL);
	static BOOL CloseScanDialog();

	CScanDialog(CWnd* pParent = NULL);   // standard constructor
	virtual ~CScanDialog();
	
// Dialog Data
	//{{AFX_DATA(CScanDialog)
	enum { IDD = IDD_SCAN_DIALOG };
	CButton	m_cancel;
	CStatic	m_percent100;
	CStatic	m_percent0;
	CStatic	m_scanStatus;
	CStatic	m_scanner;
	CStatic	m_totalTime;
	CStatic	m_scanTime;
	CStatic	m_imageSize;
	CStatic	m_scanSize;
	CStatic	m_percentText;
	CProgressCtrl	m_progress;
	CStatic	m_previewArea;
	CComboBox	m_vRes;
	CComboBox	m_bRes;
	CComboBox	m_hRes;
	CComboBox	m_grayDepth;
	CComboBox	m_colorDepth;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CScanDialog)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	BOOL			GetScanningState() { return m_scanningState; };				// Get the scanning state
	void			SetScanningState( BOOL bScanning, BOOL bQuiet=FALSE );		// Set the scanning state
	void			AbortScan();												// Schedule abort scan/preview
	void			ShowResult( long result, int nResultType = UI_RESULT_TYPE );// Show result code
	TWAIN_RESULT	GetSettings(TWAIN_PARAM_REC* pParamRec, BOOL bPreview);		// Get the current settings
	void			SaveSettings();												// Save the current settings
	void			SetProgressBar(long lMax, long lValue);						// Set progress
	void			ShowProgressBar(BOOL bShow);								// Show/hide progress & preview
	void			SetProgress( TWAIN_INFO_REC* pInfoRec );					// Set progress information
	void			InvertZoomRect( CRect *pRect );				// Invert the selection rectangle
	CRect			TrackZoom( CPoint start );					// Track selection
	HRESULT			LoadPreviewBitmap();						// Load the preview bitmap
	CString			GetFullFilePath(CString filename);						// Return the preview file path
	UI_RESULT		GetCapabilities();							// Get the capabilities
	int				GetClosestCap(TWAIN_CAPID capID,int val);
	TWAIN_RESULT	UpdateImageSize();
																//----------------------
	int				m_capMaxWidth;								// Maximum width
	int				m_capMaxHeight;								// Maximum height
	HBITMAP			m_hPreviewBitmap;							// Handle to preview bitmap
	BOOL			m_zooming;									// True if drawing zoom rect
	CRect			m_zoomRect;									// Zoom rect
	CRect			m_zoomLastRect;								// Zoom last rect
	BOOL			m_zoomRectDrawn;							// Zoom rect drawn
	CPoint			m_zoomStartPoint;							// Zoom start point
	CPoint			m_zoomEndPoint;								// Zoom end point
	BOOL			m_scanningState;							// TRUE when scanning
	BOOL			m_preview;									// Preview
	BOOL			m_updateImageSize;
	CRect			m_rectPreview;
	// Generated message map functions
	//{{AFX_MSG(CScanDialog)
	afx_msg void OnClose();
	afx_msg void OnGammaButton();
	virtual void OnCancel();
	afx_msg void OnPreviewButton();
	virtual BOOL OnInitDialog();
	afx_msg void OnSameres();
	afx_msg void OnPaint();
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnScanButton();
	afx_msg void OnUpdateImageSize();
	afx_msg HBRUSH OnCtlColor( CDC*, CWnd*, UINT );
	afx_msg void OnTimer(UINT nIDEvent);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

extern CScanDialog*		g_ScanDialog;


//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_UI_SCANDIALOG_H__8E3003E4_6540_11D3_B379_080046044C7B__INCLUDED_)
